@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2013 Stuart Parmenter and others,
+ * Copyright (C) 1997-2016 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *  
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -37,9 +35,8 @@ struct _CompletionData {
 
 CompletionData *completion_data_new(InternetAddress * ia,
                                     const gchar * nick_name);
-void completion_data_free(CompletionData * data);
+void completion_data_free(CompletionData * data, gpointer unused);
 gchar *completion_data_extract(CompletionData * data);
-gint address_compare(LibBalsaAddress *a, LibBalsaAddress *b);
 gint strncmp_word(const gchar * s1, const gchar * s2, gsize n);
 
 #endif
