@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2016 Stuart Parmenter and others,
+ * Copyright (C) 1997-2019 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * Rubrica2 address book support was written by Copyright (C)
@@ -45,12 +45,6 @@
 #include <glib/gi18n.h>
 
 
-static void
-libbalsa_address_book_rubrica_class_init(LibBalsaAddressBookRubricaClass *
-					 klass);
-static void libbalsa_address_book_rubrica_init(LibBalsaAddressBookRubrica *
-					       ab_rubrica);
-
 static LibBalsaABErr libbalsa_address_book_rubrica_load(LibBalsaAddressBook
 							* ab,
 							const gchar *
@@ -74,7 +68,7 @@ libbalsa_address_book_rubrica_modify_address(LibBalsaAddressBook * ab,
 
 static LibBalsaABErr lbab_rubrica_load_xml(LibBalsaAddressBookRubrica *
 					   ab_rubrica, xmlDocPtr * docptr);
-static void lbab_insert_address_node(const LibBalsaAddress * address,
+static void lbab_insert_address_node(LibBalsaAddress * address,
 				     xmlNodePtr parent);
 static gboolean lbab_rubrica_starts_from(const gchar * str,
 					 const gchar * filter_hi);
@@ -83,12 +77,20 @@ static GSList *extract_cards(xmlNodePtr card);
 static void extract_data(xmlNodePtr entry, gchar ** first_name,
 			 gchar ** last_name, gchar ** nick_name);
 static void extract_work(xmlNodePtr entry, gchar ** org);
-static void extract_net(xmlNodePtr entry, GList ** mail_addrs);
+static guint extract_net(xmlNodePtr entry, LibBalsaAddress *address);
 static gchar *xml_node_get_attr(xmlNodePtr node, const xmlChar * attname);
 static gchar *xml_node_get_text(xmlNodePtr node);
 
 #define CXMLCHARP(x)  ((const xmlChar *)(x))
 
+struct _LibBalsaAddressBookRubrica {
+    LibBalsaAddressBookText parent;
+
+    GSList *item_list;
+    time_t mtime;
+    LibBalsaCompletion *name_complete;
+};
+
 G_DEFINE_TYPE(LibBalsaAddressBookRubrica, libbalsa_address_book_rubrica,
         LIBBALSA_TYPE_ADDRESS_BOOK_TEXT);
 
@@ -167,13 +169,10 @@ libbalsa_address_book_rubrica_load(LibBalsaAddressBook * ab,
     LibBalsaAddressBookRubrica *ab_rubrica =
 	LIBBALSA_ADDRESS_BOOK_RUBRICA(ab);
     LibBalsaAddressBookText *ab_text = LIBBALSA_ADDRESS_BOOK_TEXT(ab);
-    const gchar *path = libbalsa_address_book_text_get_path(ab_text);
     LibBalsaABErr load_res;
     gchar *filter_hi = NULL;
     GSList *list;
 
-    g_return_val_if_fail(path != NULL, LBABERR_CANNOT_READ);
-
     /* try to load the xml file if necessary */
     load_res = lbab_rubrica_load_xml(ab_rubrica, NULL);
     if (load_res != LBABERR_OK)
@@ -190,8 +189,8 @@ libbalsa_address_book_rubrica_load(LibBalsaAddressBook * ab,
 
 	if (callback &&
 	    (!filter_hi ||
-	     lbab_rubrica_starts_from(address->last_name, filter_hi) ||
-	     lbab_rubrica_starts_from(address->full_name, filter_hi)))
+	     lbab_rubrica_starts_from(libbalsa_address_get_last_name(address), filter_hi) ||
+	     lbab_rubrica_starts_from(libbalsa_address_get_full_name(address), filter_hi)))
 	    callback(ab, address, data);
     }
     if (callback)
@@ -251,7 +250,7 @@ libbalsa_address_book_rubrica_add_address(LibBalsaAddressBook * ab,
 
     /* eject if we already have this address */
     if (g_slist_find_custom(libbalsa_address_book_text_get_item_list(ab_text), new_address,
-			    (GCompareFunc) address_compare)) {
+			    (GCompareFunc) libbalsa_address_compare)) {
 	xmlFreeDoc(doc);
 	return LBABERR_DUPLICATE;
     }
@@ -320,8 +319,8 @@ libbalsa_address_book_rubrica_modify_address(LibBalsaAddressBook * ab,
     xmlDocPtr doc = NULL;
     xmlNodePtr root_element;
     xmlNodePtr card;
+    const gchar *full_name = libbalsa_address_get_full_name(address);
     LibBalsaABErr result;
-    gboolean found;
 
     /* try to load the current file */
     if ((result = lbab_rubrica_load_xml(ab_rubrica, &doc)) != LBABERR_OK)
@@ -333,19 +332,22 @@ libbalsa_address_book_rubrica_modify_address(LibBalsaAddressBook * ab,
 	xmlFreeDoc(doc);
 	return LBABERR_ADDRESS_NOT_FOUND;
     }
-    card = root_element->children;
-    found = FALSE;
-    while (card && !found) {
-	if (xmlStrcmp(card->name, CXMLCHARP("Card"))) {
-	    gchar *full_name = xml_node_get_attr(card, CXMLCHARP("name"));
 
-	    if (full_name) {
-		found = !g_ascii_strcasecmp(address->full_name, full_name);
-		g_free(full_name);
+    for (card = root_element->children; card != NULL; card = card->next) {
+	if (xmlStrcmp(card->name, CXMLCHARP("Card")) == 0) {
+	    gchar *name = xml_node_get_attr(card, CXMLCHARP("name"));
+
+	    if (name != NULL) {
+		gboolean found = g_ascii_strcasecmp(full_name, name) == 0;
+
+		g_free(name);
+                if (found)
+                    break;
 	    }
 	}
     }
-    if (!found) {
+
+    if (card == NULL) {
 	xmlFreeDoc(doc);
 	return LBABERR_ADDRESS_NOT_FOUND;
     }
@@ -389,8 +391,8 @@ static LibBalsaABErr
 lbab_rubrica_load_xml(LibBalsaAddressBookRubrica * ab_rubrica,
 		      xmlDocPtr * docptr)
 {
-    LibBalsaAddressBookText *ab_text =
-        LIBBALSA_ADDRESS_BOOK_TEXT(ab_rubrica);
+    LibBalsaAddressBook *ab = LIBBALSA_ADDRESS_BOOK(ab_rubrica);
+    LibBalsaAddressBookText *ab_text = LIBBALSA_ADDRESS_BOOK_TEXT(ab_rubrica);
     const gchar *path = libbalsa_address_book_text_get_path(ab_text);
     struct stat stat_buf;
     int fd;
@@ -450,35 +452,39 @@ lbab_rubrica_load_xml(LibBalsaAddressBookRubrica * ab_rubrica,
     completion_list = NULL;
     for (list = libbalsa_address_book_text_get_item_list(ab_text); list != NULL; list = list->next) {
 	LibBalsaAddress *address = LIBBALSA_ADDRESS(list->data);
-	GList *l;
+        guint n_addrs;
 
-	if (!address)
+	if (address == NULL)
 	    continue;
 
-	if (address->address_list->next
-	    && libbalsa_address_book_get_dist_list_mode(LIBBALSA_ADDRESS_BOOK(ab_rubrica))) {
+        n_addrs = libbalsa_address_get_n_addrs(address);
+	if (libbalsa_address_book_get_dist_list_mode(ab)
+            && n_addrs > 1) {
 	    /* Create a group address. */
 	    InternetAddress *ia =
-		internet_address_group_new(address->full_name);
+		internet_address_group_new(libbalsa_address_get_full_name(address));
             InternetAddressGroup *group = (InternetAddressGroup *) ia;
+            guint n;
 
-	    for (l = address->address_list; l; l = l->next) {
+	    for (n = 0; n < n_addrs; ++n) {
+                const gchar *addr = libbalsa_address_get_nth_addr(address, n);
 		InternetAddress *member =
-		    internet_address_mailbox_new(NULL, l->data);
+		    internet_address_mailbox_new(NULL, addr);
 		internet_address_group_add_member(group, member);
 		g_object_unref(member);
 	    }
-	    cmp_data = completion_data_new(ia, address->nick_name);
+	    cmp_data = completion_data_new(ia, libbalsa_address_get_nick_name(address));
 	    completion_list = g_list_prepend(completion_list, cmp_data);
 	    g_object_unref(ia);
 	} else {
 	    /* Create name addresses. */
-	    GList *l;
+            guint n;
 
-	    for (l = address->address_list; l; l = l->next) {
+	    for (n = 0; n < n_addrs; ++n) {
+                const gchar *addr = libbalsa_address_get_nth_addr(address, n);
 		InternetAddress *ia =
-		    internet_address_mailbox_new(address->full_name, l->data);
-		cmp_data = completion_data_new(ia, address->nick_name);
+		    internet_address_mailbox_new(libbalsa_address_get_full_name(address), addr);
+		cmp_data = completion_data_new(ia, libbalsa_address_get_nick_name(address));
 		completion_list =
 		    g_list_prepend(completion_list, cmp_data);
 		g_object_unref(ia);
@@ -495,37 +501,41 @@ lbab_rubrica_load_xml(LibBalsaAddressBookRubrica * ab_rubrica,
 }
 
 static void
-lbab_insert_address_node(const LibBalsaAddress * address,
+lbab_insert_address_node(LibBalsaAddress * address,
 			 xmlNodePtr parent)
 {
     xmlNodePtr new_addr;
     xmlNodePtr new_data;
-    GList *l;
+    guint n_addrs;
+    guint n;
 
     /* create a new card */
     new_addr = xmlNewChild(parent, NULL, CXMLCHARP("Card"), NULL);
-    xmlNewProp(new_addr, CXMLCHARP("name"), CXMLCHARP(address->full_name));
+    xmlNewProp(new_addr, CXMLCHARP("name"), CXMLCHARP(libbalsa_address_get_full_name(address)));
 
     /* create the Data section of the card */
     new_data = xmlNewChild(new_addr, NULL, CXMLCHARP("Data"), NULL);
     xmlNewChild(new_data, NULL, CXMLCHARP("FirstName"),
-		CXMLCHARP(address->first_name));
+		CXMLCHARP(libbalsa_address_get_first_name(address)));
     xmlNewChild(new_data, NULL, CXMLCHARP("LastName"),
-		CXMLCHARP(address->last_name));
+		CXMLCHARP(libbalsa_address_get_last_name(address)));
     xmlNewChild(new_data, NULL, CXMLCHARP("NickName"),
-		CXMLCHARP(address->nick_name));
+		CXMLCHARP(libbalsa_address_get_nick_name(address)));
 
     /* create the Work section of the card */
     new_data = xmlNewChild(new_addr, NULL, CXMLCHARP("Work"), NULL);
     xmlNewChild(new_data, NULL, CXMLCHARP("Organization"),
-		CXMLCHARP(address->organization));
+		CXMLCHARP(libbalsa_address_get_organization(address)));
 
     /* create the Net section of the card */
     new_data = xmlNewChild(new_addr, NULL, CXMLCHARP("Net"), NULL);
-    for (l = address->address_list; l; l = g_list_next(l)) {
+
+    n_addrs = libbalsa_address_get_n_addrs(address);
+    for (n = 0; n < n_addrs; ++n) {
+        const gchar *addr = libbalsa_address_get_nth_addr(address, n);
 	xmlNodePtr new_mail =
 	    xmlNewChild(new_data, NULL, CXMLCHARP("Uri"),
-			CXMLCHARP(l->data));
+			CXMLCHARP(addr));
 	xmlNewProp(new_mail, CXMLCHARP("type"), CXMLCHARP("email"));
     }
 }
@@ -558,27 +568,46 @@ extract_cards(xmlNodePtr card)
 	if (!xmlStrcmp(card->name, CXMLCHARP("Card"))) {
 	    LibBalsaAddress *address = libbalsa_address_new();
 	    xmlNodePtr children;
+            gchar *full_name;
+            guint n_addrs = 0;
+
+            full_name = xml_node_get_attr(card, CXMLCHARP("name"));
+	    libbalsa_address_set_full_name(address, full_name);
+            g_free(full_name);
 
-	    address->full_name =
-		xml_node_get_attr(card, CXMLCHARP("name"));
 	    children = card->children;
 	    while (children) {
-		if (!xmlStrcmp(children->name, CXMLCHARP("Data")))
-		    extract_data(children->children, &address->first_name,
-				 &address->last_name, &address->nick_name);
-		else if (!xmlStrcmp(children->name, CXMLCHARP("Work")))
-		    extract_work(children->children,
-				 &address->organization);
-		else if (!xmlStrcmp(children->name, CXMLCHARP("Net")))
-		    extract_net(children->children,
-				&address->address_list);
+		if (!xmlStrcmp(children->name, CXMLCHARP("Data"))) {
+                    gchar *first_name;
+                    gchar *last_name = NULL;
+                    gchar *nick_name = NULL;
+
+		    extract_data(children->children,
+                                 &first_name, &last_name, &nick_name);
+
+                    libbalsa_address_set_first_name(address, first_name);
+                    libbalsa_address_set_last_name(address, last_name);
+                    libbalsa_address_set_nick_name(address, nick_name);
+
+                    g_free(first_name);
+                    g_free(last_name);
+                    g_free(nick_name);
+                } else if (!xmlStrcmp(children->name, CXMLCHARP("Work"))) {
+                    gchar *organization = NULL;
+
+		    extract_work(children->children, &organization);
+                    libbalsa_address_set_organization(address, organization);
+                    g_free(organization);
+                } else if (!xmlStrcmp(children->name, CXMLCHARP("Net"))) {
+                    n_addrs += extract_net(children->children, address);
+                }
 
 		children = children->next;
 	    }
 
-	    if (address->address_list)
+	    if (n_addrs > 0)
 		addrlist = g_slist_prepend(addrlist, address);
-	    else
+            else
 		g_object_unref(address);
 	}
 
@@ -665,9 +694,11 @@ extract_work(xmlNodePtr entry, gchar ** org)
 }
 
 
-static void
-extract_net(xmlNodePtr entry, GList ** mail_addrs)
+static guint
+extract_net(xmlNodePtr entry, LibBalsaAddress *address)
 {
+    guint n_addrs = 0;
+
     while (entry) {
 	gchar *uri_type = NULL;
 	gchar *mail_addr;
@@ -675,21 +706,24 @@ extract_net(xmlNodePtr entry, GList ** mail_addrs)
 	if (!xmlStrcmp(entry->name, CXMLCHARP("Uri"))
 	    && g_strcmp0(uri_type = xml_node_get_attr(entry, CXMLCHARP("type")),
                          "email") == 0
-	    && (mail_addr = xml_node_get_text(entry)) != NULL)
-	    *mail_addrs = g_list_prepend(*mail_addrs, mail_addr);
+	    && (mail_addr = xml_node_get_text(entry)) != NULL) {
+            libbalsa_address_append_addr(address, mail_addr);
+            g_free(mail_addr);
+            ++n_addrs;
+        }
 	g_free(uri_type);
 
 	entry = entry->next;
     }
+
+    return n_addrs;
 }
 
 
 static gchar *
 xml_node_get_text(xmlNodePtr node)
 {
-    g_return_val_if_fail(node != NULL, NULL);
-    if ((node = node->children) && node->type == XML_TEXT_NODE
-	&& node->content)
+    if ((node = node->children) != NULL && node->type == XML_TEXT_NODE)
 	return g_strdup((const gchar *) node->content);
     else
 	return NULL;
@@ -701,16 +735,11 @@ xml_node_get_attr(xmlNodePtr node, const xmlChar * attname)
 {
     xmlAttrPtr props;
 
-    g_return_val_if_fail(node != NULL, NULL);
-    props = node->properties;
-    while (props) {
+    for (props = node->properties; props != NULL; props = props->next) {
 	if (props->type == XML_ATTRIBUTE_NODE
 	    && !xmlStrcmp(props->name, attname) && props->children
 	    && props->children->type == XML_TEXT_NODE)
-	    return props->children->
-		content ? g_strdup((const gchar *) props->children->
-				   content) : NULL;
-	props = props->next;
+	    return g_strdup((const gchar *) props->children->content);
     }
 
     return NULL;
