@@ -44,11 +44,6 @@
 /* FIXME: Arbitrary constant */
 #define LINE_LEN 256
 
-static void
-libbalsa_address_book_text_class_init(LibBalsaAddressBookTextClass *
-                                      klass);
-static void
-libbalsa_address_book_text_init(LibBalsaAddressBookText * ab_text);
 static void
 libbalsa_address_book_text_finalize(GObject * object);
 
@@ -266,19 +261,20 @@ lbab_text_item_compare(LibBalsaAddressBookTextItem * a,
     g_return_val_if_fail(a != NULL, -1);
     g_return_val_if_fail(b != NULL, 1);
 
-    if (!a->address)
+    if (a->address == NULL)
         return -1;
-    if (!b->address)
+    if (b->address == NULL)
         return 1;
 
-    return g_ascii_strcasecmp(a->address->full_name,
-                              b->address->full_name);
+    return g_ascii_strcasecmp(libbalsa_address_get_full_name(a->address),
+                              libbalsa_address_get_full_name(b->address));
 }
 
 /* Load the book from the stream */
 static gboolean
 lbab_text_load_file(LibBalsaAddressBookText * ab_text, FILE * stream)
 {
+    LibBalsaAddressBook *ab = ( LibBalsaAddressBook *) ab_text;
     LibBalsaAddressBookTextPrivate *priv =
         libbalsa_address_book_text_get_instance_private(ab_text);
     LibBalsaAddressBookTextClass *ab_text_class =
@@ -336,33 +332,40 @@ lbab_text_load_file(LibBalsaAddressBookText * ab_text, FILE * stream)
 #if MAKE_GROUP_BY_ORGANIZATION
         gchar **groups, **group;
 #endif                          /* MAKE_GROUP_BY_ORGANIZATION */
-        if (!address)
+        guint n_addrs;
+
+        if (address == NULL)
             continue;
 
-        if (address->address_list->next
-            && libbalsa_address_book_get_dist_list_mode(LIBBALSA_ADDRESS_BOOK(ab_text))) {
+        n_addrs = libbalsa_address_get_n_addrs(address);
+	if (libbalsa_address_book_get_dist_list_mode(ab)
+            && n_addrs > 1) {
             /* Create a group address. */
             InternetAddress *ia =
-                internet_address_group_new(address->full_name);
-            GList *l;
+                internet_address_group_new(libbalsa_address_get_full_name(address));
+            guint n;
 
-            for (l = address->address_list; l; l = l->next) {
+            for (n = 0; n < n_addrs; ++n) {
+                const gchar *addr = libbalsa_address_get_nth_addr(address, n);
                 InternetAddress *member =
-                    internet_address_mailbox_new(NULL, l->data);
+                    internet_address_mailbox_new(NULL, addr);
                 internet_address_group_add_member((InternetAddressGroup *)ia, member);
                 g_object_unref(member);
             }
-            cmp_data = completion_data_new(ia, address->nick_name);
+            cmp_data = completion_data_new(ia, libbalsa_address_get_nick_name(address));
             completion_list = g_list_prepend(completion_list, cmp_data);
             g_object_unref(ia);
         } else {
             /* Create name addresses. */
-            GList *l;
+            guint n;
 
-            for (l = address->address_list; l; l = l->next) {
+            for (n = 0; n < n_addrs; ++n) {
+                const gchar *addr = libbalsa_address_get_nth_addr(address, n);
                 InternetAddress *ia =
-                    internet_address_mailbox_new(address->full_name, l->data);
-                cmp_data = completion_data_new(ia, address->nick_name);
+                    internet_address_mailbox_new(libbalsa_address_get_full_name(address),
+                                                 addr);
+                cmp_data =
+                    completion_data_new(ia, libbalsa_address_get_nick_name(address));
                 completion_list =
                     g_list_prepend(completion_list, cmp_data);
                 g_object_unref(ia);
@@ -537,22 +540,22 @@ libbalsa_address_book_text_load(LibBalsaAddressBook * ab,
     if (filter)
         filter_hi = g_utf8_strup(filter, -1);
 
-    for (list = priv->item_list; list != NULL; list = list->next) {
-        LibBalsaAddressBookTextItem *item = list->data;
-        LibBalsaAddress *address = item->address;
+    if (callback != NULL) {
+        for (list = priv->item_list; list != NULL; list = list->next) {
+            LibBalsaAddressBookTextItem *item = list->data;
+            LibBalsaAddress *address = item->address;
 
-        if (!address)
-            continue;
+            if (address == NULL)
+                continue;
+
+            if ((!filter_hi ||
+                 lbab_text_starts_from(libbalsa_address_get_last_name(address), filter_hi) ||
+                 lbab_text_starts_from(libbalsa_address_get_full_name(address), filter_hi)))
+                callback(ab, address, data);
+        }
 
-        if (callback && (!filter_hi
-                         || lbab_text_starts_from(address->last_name,
-                                                  filter_hi)
-                         || lbab_text_starts_from(address->full_name,
-                                                  filter_hi)))
-            callback(ab, address, data);
-    }
-    if (callback)
         callback(ab, NULL, data);
+    }
 
     g_free(filter_hi);
 
@@ -792,7 +795,7 @@ libbalsa_address_book_text_get_name_complete(LibBalsaAddressBookText * ab_text)
     LibBalsaAddressBookTextPrivate *priv =
         libbalsa_address_book_text_get_instance_private(ab_text);
 
-    g_return_val_if_fail(LIBBALSA_IS_ADDRESS_BOOK_TEXT(ab_text), 0);
+    g_return_val_if_fail(LIBBALSA_IS_ADDRESS_BOOK_TEXT(ab_text), NULL);
 
     return priv->name_complete;
 }
