@@ -29,47 +29,49 @@
 #include "misc.h"
 #include <glib/gi18n.h>
 
-static GObjectClass *parent_class;
-
-static void libbalsa_address_class_init(LibBalsaAddressClass * klass);
-static void libbalsa_address_init(LibBalsaAddress * ab);
 static void libbalsa_address_finalize(GObject * object);
 
 static gchar ** vcard_strsplit(const gchar * string);
 
-GType libbalsa_address_get_type(void)
-{
-    static GType address_type = 0;
+/* General address structure to be used with address books.
+*/
+struct _LibBalsaAddress {
+    GObject parent;
 
-    if (!address_type) {
-	static const GTypeInfo address_info = {
-	    sizeof(LibBalsaAddressClass),
-            NULL,               /* base_init */
-            NULL,               /* base_finalize */
-	    (GClassInitFunc) libbalsa_address_class_init,
-            NULL,               /* class_finalize */
-            NULL,               /* class_data */
-	    sizeof(LibBalsaAddress),
-            0,                  /* n_preallocs */
-	    (GInstanceInitFunc) libbalsa_address_init
-	};
+    /*
+     * ID
+     * VCard FN: Field
+     * LDAP/LDIF: xmozillanickname
+     */
+    gchar *nick_name;
 
-	address_type =
-	    g_type_register_static(G_TYPE_OBJECT,
-	                           "LibBalsaAddress",
-                                   &address_info, 0);
-    }
+    /* First and last names
+     * VCard: parsed from N: field
+     * LDAP/LDIF: cn, givenName, surName.
+     */
+    gchar *full_name;
+    gchar *first_name;
+    gchar *last_name;
 
-    return address_type;
-}
+    /* Organisation
+     * VCard: ORG: field
+     * ldif: o: attribute.
+     */
+    gchar *organization;
+
+    /* Email addresses
+     * A list of mailboxes, ie. user@domain.
+     */
+    GList *addr_list;
+};
+
+G_DEFINE_TYPE(LibBalsaAddress, libbalsa_address, G_TYPE_OBJECT)
 
 static void
 libbalsa_address_class_init(LibBalsaAddressClass * klass)
 {
     GObjectClass *object_class;
 
-    parent_class = g_type_class_peek_parent(klass);
-
     object_class = G_OBJECT_CLASS(klass);
     object_class->finalize = libbalsa_address_finalize;
 }
@@ -82,29 +84,23 @@ libbalsa_address_init(LibBalsaAddress * addr)
     addr->first_name = NULL;
     addr->last_name = NULL;
     addr->organization = NULL;
-    addr->address_list = NULL;
+    addr->addr_list = NULL;
 }
 
 static void
 libbalsa_address_finalize(GObject * object)
 {
-    LibBalsaAddress *addr;
+    LibBalsaAddress *addr = LIBBALSA_ADDRESS(object);
 
-    g_return_if_fail(object != NULL);
+    g_free(addr->nick_name);
+    g_free(addr->full_name);
+    g_free(addr->first_name);
+    g_free(addr->last_name);
+    g_free(addr->organization);
 
-    addr = LIBBALSA_ADDRESS(object);
+    g_list_free_full(addr->addr_list, g_free);
 
-    g_free(addr->nick_name);    addr->nick_name = NULL;
-    g_free(addr->full_name);    addr->full_name = NULL;
-    g_free(addr->first_name);   addr->first_name = NULL;
-    g_free(addr->last_name);    addr->last_name = NULL;
-    g_free(addr->organization); addr->organization = NULL;
-
-    g_list_foreach(addr->address_list, (GFunc) g_free, NULL);
-    g_list_free(addr->address_list);
-    addr->address_list = NULL;
-
-    G_OBJECT_CLASS(parent_class)->finalize(object);
+    G_OBJECT_CLASS(libbalsa_address_parent_class)->finalize(object);
 }
 
 LibBalsaAddress *
@@ -121,20 +117,25 @@ libbalsa_address_new(void)
    [1] VCARD 1.2 specs: http://www.imc.org/pdi/vcard-21.txt
    [2] VCARD 3.0 specs, RFC 2426 (http://www.ietf.org/rfc/rfc2426.txt)
 */
+
+static gboolean
+is_real_name(const gchar *name)
+{
+    return name != NULL && name[0] != '\0';
+}
+
 gchar*
 libbalsa_address_extract_name(const gchar * string, gchar ** last_name,
                               gchar ** first_name)
 {
     enum GCardFieldOrder { LAST = 0, FIRST, MIDDLE, PREFIX, SUFFIX };
-    gint cpt, j;
+    guint cpt, j;
     gchar **fld, **name_arr;
     gchar *res = NULL;
 
     fld = vcard_strsplit(string);
 
-    cpt = 0;
-    while (fld[cpt] != NULL)
-	cpt++;
+    cpt = g_strv_length(fld);
 
     if (cpt == 0) {
         /* insane empty name */
@@ -142,30 +143,30 @@ libbalsa_address_extract_name(const gchar * string, gchar ** last_name,
         return NULL;
     }
 
-    if (fld[LAST] && *fld[LAST])
+    if (is_real_name(fld[LAST]))
         *last_name = g_strdup(fld[LAST]);
 
-    if (fld[FIRST] && *fld[FIRST])
-        *first_name = fld[MIDDLE] && *fld[MIDDLE] ?
+    if (is_real_name(fld[FIRST]))
+        *first_name = is_real_name(fld[MIDDLE]) ?
             g_strconcat(fld[FIRST], " ", fld[MIDDLE], NULL) :
             g_strdup(fld[FIRST]);
 
     name_arr = g_malloc((cpt + 1) * sizeof(gchar *));
 
     j = 0;
-    if (cpt > PREFIX && *fld[PREFIX] != '\0')
+    if (is_real_name(fld[PREFIX]))
 	name_arr[j++] = g_strdup(fld[PREFIX]);
 
-    if (cpt > FIRST && *fld[FIRST] != '\0')
+    if (is_real_name(fld[FIRST]))
 	name_arr[j++] = g_strdup(fld[FIRST]);
 
-    if (cpt > MIDDLE && *fld[MIDDLE] != '\0')
+    if (is_real_name(fld[MIDDLE]))
 	name_arr[j++] = g_strdup(fld[MIDDLE]);
 
-    if (cpt > LAST && *fld[LAST] != '\0')
+    if (is_real_name(fld[LAST]))
 	name_arr[j++] = g_strdup(fld[LAST]);
 
-    if (cpt > SUFFIX && *fld[SUFFIX] != '\0')
+    if (is_real_name(fld[SUFFIX]))
 	name_arr[j++] = g_strdup(fld[SUFFIX]);
 
     name_arr[j] = NULL;
@@ -295,9 +296,12 @@ vcard_strsplit(const gchar * string)
 	string_list = g_slist_prepend(string_list, g_strdup(remainder));
     }
 
-    str_array = g_new(gchar*, n + 1);
+    if (n < 5U) {
+    	str_array = g_new0(gchar*, 5U);
+    } else {
+    	str_array = g_new0(gchar*, n + 1);
+    }
 
-    str_array[n] = NULL;
     for (slist = string_list; slist; slist = slist->next) {
 	gchar * str = (gchar *) slist->data;
 	gchar * p;
@@ -330,7 +334,7 @@ libbalsa_address_new_from_vcard(const gchar *str, const gchar *charset)
     gchar *name = NULL, *nick_name = NULL, *org = NULL;
     gchar *full_name = NULL, *last_name = NULL, *first_name = NULL;
     gint in_vcard = FALSE;
-    GList *address_list = NULL;
+    GList *addr_list = NULL;
     const gchar *string, *next_line;
     gchar * vcard;
 
@@ -389,7 +393,7 @@ libbalsa_address_new_from_vcard(const gchar *str, const gchar *charset)
 		 */
 		LibBalsaAddress *address;
 
-		if (!address_list)
+		if (!addr_list)
                     break;
 
 		address = g_object_new(LIBBALSA_TYPE_ADDRESS, NULL);
@@ -408,7 +412,7 @@ libbalsa_address_new_from_vcard(const gchar *str, const gchar *charset)
                 address->first_name = first_name;
                 address->nick_name = nick_name;
                 address->organization = org;
-                address->address_list = g_list_reverse(address_list);
+                address->addr_list = g_list_reverse(addr_list);
 
                 g_free(vcard);
 
@@ -480,7 +484,9 @@ libbalsa_address_new_from_vcard(const gchar *str, const gchar *charset)
 
                 g_free(name);
                 g_free(last_name);
+                last_name = NULL;
                 g_free(first_name);
+                first_name = NULL;
                 name = libbalsa_address_extract_name(line + 2,
                                                      &last_name, &first_name);
                 name = validate_vcard_string(name, charset);
@@ -503,8 +509,8 @@ libbalsa_address_new_from_vcard(const gchar *str, const gchar *charset)
 
             } else if (g_ascii_strncasecmp(line, "EMAIL:", 6) == 0) {
 
-		address_list =
-		    g_list_prepend(address_list, g_strdup(line + 6));
+                addr_list = g_list_prepend(addr_list, g_strdup(line + 6));
+
             }
             g_free(line);
         }
@@ -516,11 +522,10 @@ libbalsa_address_new_from_vcard(const gchar *str, const gchar *charset)
     g_free(first_name);
     g_free(nick_name);
     g_free(org);
-    g_list_foreach(address_list, (GFunc) g_free, NULL);
-    g_list_free(address_list);
-
     g_free(vcard);
 
+    g_list_free_full(addr_list, g_free);
+
     return NULL;
 }
 
@@ -529,6 +534,9 @@ libbalsa_address_set_copy(LibBalsaAddress * dest, LibBalsaAddress * src)
 {
     GList *src_al, *dst_al;
 
+    g_return_if_fail(LIBBALSA_IS_ADDRESS(dest));
+    g_return_if_fail(LIBBALSA_IS_ADDRESS(src));
+
     if (dest == src)            /* safety check */
         return;
 
@@ -542,13 +550,12 @@ libbalsa_address_set_copy(LibBalsaAddress * dest, LibBalsaAddress * src)
     dest->last_name = g_strdup(src->last_name);
     g_free(dest->organization);
     dest->organization = g_strdup(src->organization);
-    g_list_foreach(dest->address_list, (GFunc) g_free, NULL);
-    g_list_free(dest->address_list);
+    g_list_free_full(dest->addr_list, g_free);
 
     dst_al = NULL;
-    for (src_al = src->address_list; src_al; src_al = src_al->next)
+    for (src_al = src->addr_list; src_al != NULL; src_al = src_al->next)
         dst_al = g_list_prepend(dst_al, g_strdup(src_al->data));
-    dest->address_list = g_list_reverse(dst_al);
+    dest->addr_list = g_list_reverse(dst_al);
 }
 
 static gchar *
@@ -597,16 +604,16 @@ libbalsa_address_to_gchar(LibBalsaAddress * address, gint n)
 
     g_return_val_if_fail(LIBBALSA_IS_ADDRESS(address), NULL);
 
-    if(!address->address_list)
+    if (address->addr_list == NULL)
         return NULL;
-    if(n==-1) {
-        if(address->address_list->next)
-            retc = rfc2822_group(address->full_name, address->address_list);
+    if (n == -1) {
+        if (address->addr_list->next != NULL)
+            retc = rfc2822_group(address->full_name, address->addr_list);
         else
             retc = rfc2822_mailbox(address->full_name,
-                                   address->address_list->data);
+                                   address->addr_list->data);
     } else {
-	const gchar *mailbox = g_list_nth_data(address->address_list, n);
+	const gchar *mailbox = g_list_nth_data(address->addr_list, n);
 	g_return_val_if_fail(mailbox != NULL, NULL);
 
 	retc = rfc2822_mailbox(address->full_name, mailbox);
@@ -617,19 +624,19 @@ libbalsa_address_to_gchar(LibBalsaAddress * address, gint n)
 
 /* Helper */
 static const gchar *
-lba_get_name_or_mailbox(InternetAddressList * address_list,
+lba_get_name_or_mailbox(InternetAddressList * addr_list,
                         gboolean get_name, gboolean in_group)
 {
     const gchar *retval = NULL;
     InternetAddress *ia;
     gint i, len;
 
-    if (address_list == NULL)
+    if (addr_list == NULL)
 	return NULL;
 
-    len = internet_address_list_length(address_list);
+    len = internet_address_list_length(addr_list);
     for (i = 0; i < len; i++) {
-        ia = internet_address_list_get_address (address_list, i);
+        ia = internet_address_list_get_address (addr_list, i);
 
         if (get_name && ia->name && *ia->name)
             return ia->name;
@@ -652,30 +659,34 @@ lba_get_name_or_mailbox(InternetAddressList * address_list,
 
 /* Get either a name or a mailbox from an InternetAddressList. */
 const gchar *
-libbalsa_address_get_name_from_list(InternetAddressList *address_list)
+libbalsa_address_get_name_from_list(InternetAddressList *addr_list)
 {
-    return lba_get_name_or_mailbox(address_list, TRUE, FALSE);
+    g_return_val_if_fail(IS_INTERNET_ADDRESS_LIST(addr_list), NULL);
+
+    return lba_get_name_or_mailbox(addr_list, TRUE, FALSE);
 }
 
 /* Get a mailbox from an InternetAddressList. */
 const gchar *
-libbalsa_address_get_mailbox_from_list(InternetAddressList *address_list)
+libbalsa_address_get_mailbox_from_list(InternetAddressList *addr_list)
 {
-    return lba_get_name_or_mailbox(address_list, FALSE, FALSE);
+    g_return_val_if_fail(IS_INTERNET_ADDRESS_LIST(addr_list), NULL);
+
+    return lba_get_name_or_mailbox(addr_list, FALSE, FALSE);
 }
 
 /* Number of individual mailboxes in an InternetAddressList. */
 gint
-libbalsa_address_n_mailboxes_in_list(InternetAddressList * address_list)
+libbalsa_address_n_mailboxes_in_list(InternetAddressList * addr_list)
 {
     gint i, len, n_mailboxes = 0;
 
-    g_return_val_if_fail(IS_INTERNET_ADDRESS_LIST(address_list), -1);
+    g_return_val_if_fail(IS_INTERNET_ADDRESS_LIST(addr_list), -1);
 
-    len = internet_address_list_length(address_list);
+    len = internet_address_list_length(addr_list);
     for (i = 0; i < len; i++) {
         const InternetAddress *ia =
-            internet_address_list_get_address(address_list, i);
+            internet_address_list_get_address(addr_list, i);
 
         if (INTERNET_ADDRESS_IS_MAILBOX(ia))
             ++n_mailboxes;
@@ -696,7 +707,7 @@ libbalsa_address_n_mailboxes_in_list(InternetAddressList * address_list)
     in entries with values from address
 */
 void
-libbalsa_address_set_edit_entries(const LibBalsaAddress * address,
+libbalsa_address_set_edit_entries(LibBalsaAddress * address,
                                   GtkWidget **entries)
 {
     gchar *new_name = NULL;
@@ -706,25 +717,26 @@ libbalsa_address_set_edit_entries(const LibBalsaAddress * address,
     gchar *last_name = NULL;
     gchar *nick_name = NULL;
     gint cnt;
+    GtkWidget *tree_view;
     GtkListStore *store;
     GtkTreeIter iter;
 
-    new_email = g_strdup(address
-                         && address->address_list
-                         && address->address_list->data ?
-                         address->address_list->data : "");
+    g_return_if_fail(address == NULL || LIBBALSA_IS_ADDRESS(address));
+
+    new_email = g_strdup(address != NULL
+                         && address->addr_list != NULL
+                         && address->addr_list->data != NULL ?
+                         address->addr_list->data : "");
     /* initialize the organization... */
-    if (!address || address->organization == NULL)
-	new_organization = g_strdup("");
-    else
-	new_organization = g_strdup(address->organization);
+    new_organization = g_strdup(address != NULL
+                                && address->organization != NULL ?
+                                address->organization : "");
 
     /* if the message only contains an e-mail address */
-    if (!address || address->full_name == NULL)
+    if (address == NULL || address->full_name == NULL) {
 	new_name = g_strdup(new_email);
-    else {
+    } else {
         gchar **names;
-        g_assert(address);
 	/* make sure address->personal is not all whitespace */
 	new_name = g_strstrip(g_strdup(address->full_name));
 
@@ -740,7 +752,7 @@ libbalsa_address_set_edit_entries(const LibBalsaAddress * address,
                                   ? address->first_name : names[0]);
 
 	    /* get last name */
-            if(address->last_name)
+            if(address->last_name != NULL)
                 last_name = g_strdup(address->last_name);
             else {
                 if (cnt == 1)
@@ -756,7 +768,7 @@ libbalsa_address_set_edit_entries(const LibBalsaAddress * address,
 	first_name = g_strdup("");
     if (last_name == NULL)
 	last_name = g_strdup("");
-    if (!address || address->nick_name == NULL)
+    if (address == NULL || address->nick_name == NULL)
 	nick_name = g_strdup("");
     else
 	nick_name = g_strdup(address->nick_name);
@@ -768,13 +780,13 @@ libbalsa_address_set_edit_entries(const LibBalsaAddress * address,
     gtk_entry_set_text(GTK_ENTRY(entries[NICK_NAME]), nick_name);
     gtk_entry_set_text(GTK_ENTRY(entries[ORGANIZATION]), new_organization);
 
-    store = GTK_LIST_STORE(gtk_tree_view_get_model
-                           (GTK_TREE_VIEW(entries[EMAIL_ADDRESS])));
+    tree_view = gtk_bin_get_child(GTK_BIN(entries[EMAIL_ADDRESS]));
+    store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(tree_view)));
     gtk_list_store_clear(store);
-    if (address) {
+    if (address != NULL) {
         GList *list;
 
-        for (list = address->address_list; list; list = list->next) {
+        for (list = address->addr_list; list != NULL; list = list->next) {
             gtk_list_store_append(store, &iter);
             gtk_list_store_set(store, &iter, 0, list->data, -1);
         }
@@ -825,12 +837,13 @@ lba_cell_edited(GtkCellRendererText * cell, const gchar * path_string,
 }
 
 static GtkWidget *
-lba_address_list_widget(GCallback changed_cb, gpointer changed_data)
+lba_addr_list_widget(GCallback changed_cb, gpointer changed_data)
 {
     GtkListStore *store;
     GtkWidget *tree_view;
     GtkCellRenderer *renderer;
     GtkTreeViewColumn *column;
+    GtkWidget *frame;
 
     store = gtk_list_store_new(1, G_TYPE_STRING);
     tree_view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
@@ -849,7 +862,11 @@ lba_address_list_widget(GCallback changed_cb, gpointer changed_data)
                                                       "text", 0, NULL);
     gtk_tree_view_append_column(GTK_TREE_VIEW(tree_view), column);
 
-    return tree_view;
+    frame = gtk_frame_new(NULL);
+    gtk_container_add(GTK_CONTAINER(frame), tree_view);
+    gtk_widget_show_all(frame);
+
+    return frame;
 }
 
 static void
@@ -890,12 +907,12 @@ addrlist_drag_received_cb(GtkWidget * widget, GdkDragContext * context,
         case LIBBALSA_ADDRESS_TRG_ADDRESS:
             addr = *(LibBalsaAddress **)
                 gtk_selection_data_get_data(selection_data);
-            if(addr && addr->address_list) {
-                g_print ("string: %s\n", (gchar*)addr->address_list->data);
+            if (addr != NULL && addr->addr_list != NULL) {
+                g_print ("string: %s\n", (gchar*)addr->addr_list->data);
                 gtk_list_store_insert_with_values(GTK_LIST_STORE(model),
                                                   &iter, 99999,
                                                   0,
-                                                  addr->address_list->data,
+                                                  addr->addr_list->data,
                                                   -1);
                 dnd_success = TRUE;
             }
@@ -949,7 +966,7 @@ addrlist_drag_drop_cb(GtkWidget *widget, GdkDragContext *context,
 
 
 GtkWidget*
-libbalsa_address_get_edit_widget(const LibBalsaAddress *address,
+libbalsa_address_get_edit_widget(LibBalsaAddress *address,
                                  GtkWidget **entries,
                                  GCallback changed_cb, gpointer changed_data)
 {
@@ -965,6 +982,8 @@ libbalsa_address_get_edit_widget(const LibBalsaAddress *address,
     GtkWidget *grid, *label, *lhs;
     gint cnt;
 
+    g_return_val_if_fail(address == NULL || LIBBALSA_IS_ADDRESS(address), NULL);
+
     grid = gtk_grid_new();
 #define HIG_PADDING 6
     gtk_grid_set_row_spacing(GTK_GRID(grid), HIG_PADDING);
@@ -979,23 +998,24 @@ libbalsa_address_get_edit_widget(const LibBalsaAddress *address,
         if (cnt == EMAIL_ADDRESS) {
             GtkWidget *box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
             GtkWidget *but = gtk_button_new_with_mnemonic(_("A_dd"));
-            entries[cnt] = lba_address_list_widget(changed_cb,
-                                                   changed_data);
+            GtkWidget *tree_view;
+
+            entries[cnt] = lba_addr_list_widget(changed_cb, changed_data);
             gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 1);
             gtk_box_pack_start(GTK_BOX(box), but,   FALSE, FALSE, 1);
             lhs = box;
-            g_signal_connect(but, "clicked", G_CALLBACK(add_row),
-                             entries[cnt]);
+            tree_view = gtk_bin_get_child(GTK_BIN(entries[cnt]));
+            g_signal_connect(but, "clicked", G_CALLBACK(add_row), tree_view);
              gtk_drag_dest_set(entries[cnt],
                                GTK_DEST_DEFAULT_MOTION |
                                GTK_DEST_DEFAULT_HIGHLIGHT,
                                libbalsa_address_target_list,
                                2,              /* size of list */
                                GDK_ACTION_COPY);
-            g_signal_connect(G_OBJECT(entries[cnt]), "drag-data-received",
+            g_signal_connect(tree_view, "drag-data-received",
                              G_CALLBACK(addrlist_drag_received_cb), NULL);
-            g_signal_connect (G_OBJECT(entries[cnt]), "drag-drop",
-                              G_CALLBACK (addrlist_drag_drop_cb), NULL);
+            g_signal_connect(tree_view, "drag-drop",
+                             G_CALLBACK(addrlist_drag_drop_cb), NULL);
         } else {
             entries[cnt] = gtk_entry_new();
             if (changed_cb)
@@ -1021,13 +1041,13 @@ libbalsa_address_get_edit_widget(const LibBalsaAddress *address,
     libbalsa_address_set_edit_entries(address, entries);
 
     if (changed_cb) {
-        GtkTreeModel *model =
-            gtk_tree_view_get_model(GTK_TREE_VIEW(entries[EMAIL_ADDRESS]));
-        g_signal_connect_swapped(G_OBJECT(model), "row-inserted",
+        GtkWidget *tree_view = gtk_bin_get_child(GTK_BIN(entries[EMAIL_ADDRESS]));
+        GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(tree_view));
+        g_signal_connect_swapped(model, "row-inserted",
                                  changed_cb, changed_data);
-        g_signal_connect_swapped(G_OBJECT(model), "row-changed",
+        g_signal_connect_swapped(model, "row-changed",
                                  changed_cb, changed_data);
-        g_signal_connect_swapped(G_OBJECT(model), "row-deleted",
+        g_signal_connect_swapped(model, "row-deleted",
                                  changed_cb, changed_data);
     }
     return grid;
@@ -1044,11 +1064,12 @@ libbalsa_address_new_from_edit_entries(GtkWidget ** entries)
     LibBalsaAddress *address;
     char *p;
     GList *list = NULL;
+    GtkWidget *tree_view;
     GtkTreeModel *model;
     gboolean valid;
     GtkTreeIter iter;
 
-    /* make sure gtk_tree_model looses focus, otherwise the list does
+    /* make sure gtk_tree_model loses focus, otherwise the list does
      * not get updated (gtk2-2.8.20) */
     gtk_widget_grab_focus(entries[FULL_NAME]);
     /* FIXME: This problem should be solved in the VCard
@@ -1058,7 +1079,7 @@ libbalsa_address_new_from_edit_entries(GtkWidget ** entries)
 
     address = libbalsa_address_new();
     SET_FIELD(address->full_name,   entries[FULL_NAME]);
-    if (!address->full_name) {
+    if (address->full_name == NULL) {
         g_object_unref(address);
         return NULL;
     }
@@ -1067,16 +1088,169 @@ libbalsa_address_new_from_edit_entries(GtkWidget ** entries)
     SET_FIELD(address->nick_name,   entries[NICK_NAME]);
     SET_FIELD(address->organization,entries[ORGANIZATION]);
 
-    model = gtk_tree_view_get_model(GTK_TREE_VIEW(entries[EMAIL_ADDRESS]));
+
+    tree_view = gtk_bin_get_child(GTK_BIN(entries[EMAIL_ADDRESS]));
+    model = gtk_tree_view_get_model(GTK_TREE_VIEW(tree_view));
     for (valid = gtk_tree_model_get_iter_first(model, &iter); valid;
          valid = gtk_tree_model_iter_next(model, &iter)) {
         gchar *email;
 
         gtk_tree_model_get(model, &iter, 0, &email, -1);
-        if (email && *email)
-            list = g_list_prepend(list, email);
+        if (email != NULL) {
+            if (email[0] != '\0')
+                list = g_list_prepend(list, email);
+            else
+                g_free(email);
+        }
     }
-    address->address_list = g_list_reverse(list);
+    address->addr_list = g_list_reverse(list);
 
     return address;
 }
+
+/*
+ * Comparison func
+ */
+
+gint
+libbalsa_address_compare(LibBalsaAddress *a, LibBalsaAddress *b)
+{
+    g_return_val_if_fail(LIBBALSA_IS_ADDRESS(a), -1);
+    g_return_val_if_fail(LIBBALSA_IS_ADDRESS(b), +1);
+
+    return g_ascii_strcasecmp(a->full_name, b->full_name);
+}
+
+/*
+ * Getters
+ */
+
+const gchar *
+libbalsa_address_get_full_name(LibBalsaAddress * address)
+{
+    g_return_val_if_fail(LIBBALSA_IS_ADDRESS(address), NULL);
+
+    return address->full_name;
+}
+
+const gchar *
+libbalsa_address_get_first_name(LibBalsaAddress * address)
+{
+    g_return_val_if_fail(LIBBALSA_IS_ADDRESS(address), NULL);
+
+    return address->first_name;
+}
+
+const gchar *
+libbalsa_address_get_last_name(LibBalsaAddress * address)
+{
+    g_return_val_if_fail(LIBBALSA_IS_ADDRESS(address), NULL);
+
+    return address->last_name;
+}
+
+const gchar *
+libbalsa_address_get_nick_name(LibBalsaAddress * address)
+{
+    g_return_val_if_fail(LIBBALSA_IS_ADDRESS(address), NULL);
+
+    return address->nick_name;
+}
+
+const gchar *
+libbalsa_address_get_organization(LibBalsaAddress * address)
+{
+    g_return_val_if_fail(LIBBALSA_IS_ADDRESS(address), NULL);
+
+    return address->organization;
+}
+
+const gchar *
+libbalsa_address_get_addr(LibBalsaAddress * address)
+{
+    g_return_val_if_fail(LIBBALSA_IS_ADDRESS(address), NULL);
+
+    return address->addr_list != NULL ? address->addr_list->data : NULL;
+}
+
+guint
+libbalsa_address_get_n_addrs(LibBalsaAddress * address)
+{
+    g_return_val_if_fail(LIBBALSA_IS_ADDRESS(address), 0U);
+
+    return g_list_length(address->addr_list);
+}
+
+const gchar *
+libbalsa_address_get_nth_addr(LibBalsaAddress * address, guint n)
+{
+    g_return_val_if_fail(LIBBALSA_IS_ADDRESS(address), NULL);
+
+    return (const gchar *) g_list_nth_data(address->addr_list, n);
+}
+
+/*
+ * Setters
+ */
+
+void
+libbalsa_address_set_full_name(LibBalsaAddress * address,
+                               const gchar     * full_name)
+{
+    g_return_if_fail(LIBBALSA_IS_ADDRESS(address));
+
+    g_free(address->full_name);
+    address->full_name = g_strdup(full_name);
+}
+
+void
+libbalsa_address_set_first_name(LibBalsaAddress * address,
+                                const gchar     * first_name)
+{
+    g_return_if_fail(LIBBALSA_IS_ADDRESS(address));
+
+    g_free(address->first_name);
+    address->first_name = g_strdup(first_name);
+}
+
+void
+libbalsa_address_set_last_name(LibBalsaAddress * address,
+                               const gchar     * last_name)
+{
+    g_return_if_fail(LIBBALSA_IS_ADDRESS(address));
+
+    g_free(address->last_name);
+    address->last_name = g_strdup(last_name);
+}
+
+void
+libbalsa_address_set_nick_name(LibBalsaAddress * address,
+                               const gchar     * nick_name)
+{
+    g_return_if_fail(LIBBALSA_IS_ADDRESS(address));
+
+    g_free(address->nick_name);
+    address->nick_name = g_strdup(nick_name);
+}
+
+void
+libbalsa_address_set_organization(LibBalsaAddress * address,
+                                  const gchar     * organization)
+{
+    g_return_if_fail(LIBBALSA_IS_ADDRESS(address));
+
+    g_free(address->organization);
+    address->organization = g_strdup(organization);
+}
+
+void
+libbalsa_address_append_addr(LibBalsaAddress * address,
+                             const gchar     * addr)
+{
+    g_return_if_fail(LIBBALSA_IS_ADDRESS(address));
+
+    /* Append to keep the order the same as in an address book;
+     * prepend would be more efficient if addresses had many addrs, but
+     * then how would we keep the address order? */
+    address->addr_list = g_list_append(address->addr_list, g_strdup(addr));
+}
