@@ -201,41 +201,46 @@ autocrypt_from_message(LibBalsaMessage  *message,
 {
 	const gchar *from_addr;
 	AutocryptData *autocrypt;
+        LibBalsaMessageHeaders *headers;
 
-	g_return_if_fail(LIBBALSA_IS_MESSAGE(message) && (message->headers != NULL) && (message->headers->from != NULL) &&
-		(message->headers->content_type != NULL) && (autocrypt_db != NULL));
+	g_return_if_fail(LIBBALSA_IS_MESSAGE(message));
+        headers = libbalsa_message_get_headers(message);
+	g_return_if_fail(headers != NULL);
+	g_return_if_fail(headers->from != NULL);
+	g_return_if_fail(headers->content_type != NULL);
+	g_return_if_fail(autocrypt_db != NULL);
 
 	// FIXME - we should ignore spam - how can we detect it?
 
 	/* check for content types which shall be ignored
 	 * Note: see Autocrypt Level 1 standard, section 2.3 (https://autocrypt.org/level1.html#updating-autocrypt-peer-state) for
 	 *       details about this and the following checks which may result in completely ignoring the message. */
-	if (autocrypt_ignore(message->headers->content_type)) {
-		g_debug("ignore %s/%s", g_mime_content_type_get_media_type(message->headers->content_type),
-			g_mime_content_type_get_media_subtype(message->headers->content_type));
+	if (autocrypt_ignore(headers->content_type)) {
+		g_debug("ignore %s/%s", g_mime_content_type_get_media_type(headers->content_type),
+			g_mime_content_type_get_media_subtype(headers->content_type));
 		return;
 	}
 
 	/* check for exactly one From: mailbox address - others shall be ignored */
-	if ((internet_address_list_length(message->headers->from) != 1) ||
-		!INTERNET_ADDRESS_IS_MAILBOX(internet_address_list_get_address(message->headers->from, 0))) {
+	if ((internet_address_list_length(headers->from) != 1) ||
+		!INTERNET_ADDRESS_IS_MAILBOX(internet_address_list_get_address(headers->from, 0))) {
 		g_debug("require exactly one From: address, ignored");
 		return;
 	}
 
 	/* ignore messages without a Date: header or with a date in the future */
-	if ((message->headers->date == 0) || (message->headers->date > time(NULL))) {
+	if ((headers->date == 0) || (headers->date > time(NULL))) {
 		g_debug("no Date: header or value in the future, ignored");
 		return;
 	}
 
 	/* get the From: address (is a mailbox, checked above) */
 	from_addr =
-		internet_address_mailbox_get_addr(INTERNET_ADDRESS_MAILBOX(internet_address_list_get_address(message->headers->from, 0)));
-	g_debug("message from '%s', date %ld", from_addr, message->headers->date);
+		internet_address_mailbox_get_addr(INTERNET_ADDRESS_MAILBOX(internet_address_list_get_address(headers->from, 0)));
+	g_debug("message from '%s', date %ld", from_addr, headers->date);
 
 	/* scan for Autocrypt headers */
-	autocrypt = scan_autocrypt_headers(message->headers->user_hdrs, from_addr);
+	autocrypt = scan_autocrypt_headers(headers->user_hdrs, from_addr);
 
     /* update the database */
     G_LOCK(db_mutex);
@@ -244,19 +249,19 @@ autocrypt_from_message(LibBalsaMessage  *message,
 
     	db_info = autocrypt_user_info(autocrypt->addr, error);
     	if (db_info != NULL) {
-    		if (message->headers->date > db_info->ac_timestamp) {
-        		add_or_update_user_info(autocrypt, message->headers->date, TRUE, error);
+    		if (headers->date > db_info->ac_timestamp) {
+        		add_or_update_user_info(autocrypt, headers->date, TRUE, error);
     		} else {
     			g_info("message timestamp %ld not newer than autocrypt db timestamp %ld, ignore message",
-    				(long) message->headers->date, (long) db_info->ac_timestamp);
+    				(long) headers->date, (long) db_info->ac_timestamp);
     		}
     		autocrypt_free(db_info);
     	} else {
-    		add_or_update_user_info(autocrypt, message->headers->date, FALSE, error);
+    		add_or_update_user_info(autocrypt, headers->date, FALSE, error);
     	}
     	autocrypt_free(autocrypt);
     } else {
-    	update_last_seen(from_addr, message->headers->date, error);
+    	update_last_seen(from_addr, headers->date, error);
     }
     G_UNLOCK(db_mutex);
 }
@@ -264,17 +269,25 @@ autocrypt_from_message(LibBalsaMessage  *message,
 
 /* documentation: see header file */
 gchar *
-autocrypt_header(const LibBalsaIdentity *identity, GError **error)
+autocrypt_header(LibBalsaIdentity *identity, GError **error)
 {
 	const gchar *mailbox;
 	gchar *use_fpr = NULL;
 	gchar *result = NULL;
+        InternetAddress *ia;
+        const gchar *force_gpg_key_id;
+        AutocryptMode autocrypt_mode;
 
-	g_return_val_if_fail((identity != NULL) && (identity->autocrypt_mode != AUTOCRYPT_DISABLE), NULL);
-	mailbox = internet_address_mailbox_get_addr(INTERNET_ADDRESS_MAILBOX(identity->ia));
+	g_return_val_if_fail(identity != NULL, NULL);
+	autocrypt_mode = libbalsa_identity_get_autocrypt_mode(identity);
+	g_return_val_if_fail(autocrypt_mode != AUTOCRYPT_DISABLE, NULL);
+
+	ia = libbalsa_identity_get_address(identity);
+	mailbox = internet_address_mailbox_get_addr(INTERNET_ADDRESS_MAILBOX(ia));
 
 	/* no key fingerprint has been passed - try to find the fingerprint of a secret key matching the passed mailbox */
-	if ((identity->force_gpg_key_id == NULL) || (identity->force_gpg_key_id[0] == '\0')) {
+        force_gpg_key_id = libbalsa_identity_get_force_gpg_key_id(identity);
+	if ((force_gpg_key_id == NULL) || (force_gpg_key_id[0] == '\0')) {
 		gpgme_ctx_t ctx;
 
 		ctx = libbalsa_gpgme_new_with_proto(GPGME_PROTOCOL_OpenPGP, NULL, NULL, error);
@@ -300,7 +313,7 @@ autocrypt_header(const LibBalsaIdentity *identity, GError **error)
 			g_debug("found fingerprint %s for '%s'", use_fpr, mailbox);
 		}
 	} else {
-		use_fpr = g_strdup(identity->force_gpg_key_id);
+		use_fpr = g_strdup(force_gpg_key_id);
 	}
 
 	if (use_fpr != NULL) {
@@ -314,7 +327,7 @@ autocrypt_header(const LibBalsaIdentity *identity, GError **error)
 
 			buffer = g_string_new(NULL);
 			g_string_append_printf(buffer, "addr=%s;", mailbox);
-			if (identity->autocrypt_mode == AUTOCRYPT_PREFER_ENCRYPT) {
+			if (autocrypt_mode == AUTOCRYPT_PREFER_ENCRYPT) {
 				g_string_append(buffer, "prefer-encrypt=mutual;");
 			}
 			g_string_append_printf(buffer, "keydata=%s", keydata);
@@ -481,7 +494,7 @@ autocrypt_db_dialog_run(const gchar *date_string, GtkWindow *parent)
     sqlite3_reset(query[5]);
 
     /* set up the tree view */
-    g_object_unref(G_OBJECT(model));
+    g_object_unref(model);
 
 	renderer = gtk_cell_renderer_text_new();
 	column = gtk_tree_view_column_new_with_attributes(_("Mailbox"), renderer, "text", 0, NULL);
@@ -866,7 +879,8 @@ add_or_update_user_info(const AutocryptData *user_info, time_t date_header, gboo
 		(sqlite3_bind_int64(query[query_idx], 5, user_info->expires) != SQLITE_OK) ||
 		(sqlite3_bind_int(query[query_idx], 6, user_info->prefer_encrypt) != SQLITE_OK) ||
 		(sqlite3_step(query[query_idx]) != SQLITE_DONE)) {
-		g_set_error(error, AUTOCRYPT_ERROR_QUARK, -1, _("%s user “%s” failed: %s"), update ? _("update") : _("insert"),
+		g_set_error(error, AUTOCRYPT_ERROR_QUARK, -1,
+                        update ? _("update user “%s” failed: %s") : _("insert user “%s” failed: %s"),
 			user_info->addr, sqlite3_errmsg(autocrypt_db));
 	} else {
 		g_debug("%s user '%s': %d", update ? "updated" : "inserted", user_info->addr, sqlite3_changes(autocrypt_db));
