@@ -20,10 +20,8 @@
 #include "libimap.h"
 #include "imap-handle.h"
 
-typedef ImapResult (*ImapAuthenticator)(ImapMboxHandle* handle);
 ImapResult imap_authenticate(ImapMboxHandle* handle);
 ImapResult imap_auth_cram(ImapMboxHandle* handle);
-ImapResult imap_auth_login(ImapMboxHandle* handle);
 ImapResult imap_auth_gssapi(ImapMboxHandle* handle);
 
 #endif
