@@ -1,7 +1,7 @@
 #ifndef __IMAP_H__
 #define __IMAP_H__ 1
 /* libimap library.
- * Copyright (C) 2003-2013 Pawel Salek.
+ * Copyright (C) 2003-2016 Pawel Salek.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,9 +14,7 @@
  * GNU General Public License for more details.
  *  
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <time.h>
@@ -44,12 +42,6 @@ typedef enum {
   IMAP_SELECT_FAILED /* SELECT command failed */
 } ImapResult;
 
-typedef enum {
-  IMAP_TLS_DISABLED,
-  IMAP_TLS_ENABLED,
-  IMAP_TLS_REQUIRED
-} ImapTlsMode;
-
 typedef enum { 
   IMLIST_MARKED = 0,
   IMLIST_UNMARKED,
@@ -275,8 +267,6 @@ void*        imap_message_serialize(ImapMessage *);
 ImapMessage* imap_message_deserialize(void *data);
 size_t imap_serialized_message_size(void *data);
 
-const char *lbi_strerror(ImapResult rc);
-
 /* RFC 4314: IMAP ACL's */
 typedef enum {
   IMAP_ACL_NONE = 0,
