@@ -33,55 +33,42 @@ G_BEGIN_DECLS
 
 
 /* a vfs file description as GObject */
-#define LIBBALSA_TYPE_VFS            (libbalsa_vfs_get_type())
-#define LIBBALSA_VFS(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj), LIBBALSA_TYPE_VFS, LibbalsaVfs))
-#define LIBBALSA_VFS_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST((klass), LIBBALSA_TYPE_VFS, LibbalsaVfsClass))
-#define LIBBALSA_IS_VFS(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj), LIBBALSA_TYPE_VFS))
-#define LIBBALSA_IS_VFS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), LIBBALSA_TYPE_VFS))
-#define LIBBALSA_VFS_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj), LIBBALSA_TYPE_VFS, LibbalsaVfsClass))
+#define LIBBALSA_TYPE_VFS (libbalsa_vfs_get_type())
 
-typedef struct _LibbalsaVfsClass LibbalsaVfsClass;
+G_DECLARE_FINAL_TYPE(LibbalsaVfs,
+                     libbalsa_vfs,
+                     LIBBALSA,
+                     VFS,
+                     GObject)
 
 
-struct _LibbalsaVfs {
-    GObject parent;
-
-    struct _LibbalsaVfsPriv * priv;
-};
-
-struct _LibbalsaVfsClass {
-    GObjectClass parent;
-};
-
 gboolean libbalsa_vfs_local_only(void);
-
-GType libbalsa_vfs_get_type(void);
 LibbalsaVfs * libbalsa_vfs_new(void);
 LibbalsaVfs * libbalsa_vfs_new_from_uri(const gchar * uri);
-LibbalsaVfs * libbalsa_vfs_append(const LibbalsaVfs * file,
+LibbalsaVfs * libbalsa_vfs_append(LibbalsaVfs * file,
                                   const gchar * text);
-LibbalsaVfs * libbalsa_vfs_dir_append(const LibbalsaVfs * dir,
+LibbalsaVfs * libbalsa_vfs_dir_append(LibbalsaVfs * dir,
                                       const gchar * filename);
-const gchar * libbalsa_vfs_get_folder(const LibbalsaVfs * file);
-const gchar * libbalsa_vfs_get_uri(const LibbalsaVfs * file);
-const gchar * libbalsa_vfs_get_uri_utf8(const LibbalsaVfs * file);
-const gchar * libbalsa_vfs_get_basename_utf8(const LibbalsaVfs * file);
-const gchar * libbalsa_vfs_get_mime_type(const LibbalsaVfs * file);
-const gchar * libbalsa_vfs_get_charset(const LibbalsaVfs * file);
-LibBalsaTextAttribute libbalsa_vfs_get_text_attr(const LibbalsaVfs * file);
-guint64 libbalsa_vfs_get_size(const LibbalsaVfs * file);
-GMimeStream * libbalsa_vfs_create_stream(const LibbalsaVfs * file,
+const gchar * libbalsa_vfs_get_folder(LibbalsaVfs * file);
+const gchar * libbalsa_vfs_get_uri(LibbalsaVfs * file);
+const gchar * libbalsa_vfs_get_uri_utf8(LibbalsaVfs * file);
+const gchar * libbalsa_vfs_get_basename_utf8(LibbalsaVfs * file);
+const gchar * libbalsa_vfs_get_mime_type(LibbalsaVfs * file);
+const gchar * libbalsa_vfs_get_charset(LibbalsaVfs * file);
+LibBalsaTextAttribute libbalsa_vfs_get_text_attr(LibbalsaVfs * file);
+guint64 libbalsa_vfs_get_size(LibbalsaVfs * file);
+GMimeStream * libbalsa_vfs_create_stream(LibbalsaVfs * file,
                                          mode_t mode, 
                                          gboolean rdwr,
                                          GError ** err);
-gboolean libbalsa_vfs_file_exists(const LibbalsaVfs * file);
-gboolean libbalsa_vfs_is_regular_file(const LibbalsaVfs * file,
+gboolean libbalsa_vfs_file_exists(LibbalsaVfs * file);
+gboolean libbalsa_vfs_is_regular_file(LibbalsaVfs * file,
                                       GError **err);
-gint libbalsa_vfs_file_unlink(const LibbalsaVfs * file,
+gint libbalsa_vfs_file_unlink(LibbalsaVfs * file,
                               GError **err);
 
 /* application launch helpers */
-gboolean libbalsa_vfs_launch_app(const LibbalsaVfs * file,
+gboolean libbalsa_vfs_launch_app(LibbalsaVfs * file,
                                  GObject * object,
                                  GError **err);
 gboolean libbalsa_vfs_launch_app_for_body(LibBalsaMessageBody * mime_body,
