@@ -34,13 +34,10 @@ typedef struct _LibBalsaMessage LibBalsaMessage;
 typedef struct _LibBalsaMessageHeaders LibBalsaMessageHeaders;
 typedef struct _LibBalsaMessageBody LibBalsaMessageBody;
 typedef struct _LibBalsaServer LibBalsaServer;
-#if ENABLE_ESMTP
 typedef struct _LibBalsaSmtpServer LibBalsaSmtpServer;
-#endif                          /* ENABLE_ESMTP */
 typedef struct _LibbalsaVfs LibbalsaVfs;
 
 
-#include <openssl/ssl.h>
 #include "message.h"
 #include "body.h"
 #include "files.h"
@@ -66,6 +63,11 @@ typedef struct _LibbalsaVfs LibbalsaVfs;
 #include "address-book-osmo.h"
 #endif
 
+/* Callback for testing whether a mailbox or server can be reached: */
+typedef void LibBalsaCanReachCallback(GObject * object,
+                                      gboolean  can_reach,
+                                      gpointer  cb_data);
+
 #include "mailbox.h"
 #include "mailbox_local.h"
 #include "mailbox_remote.h"
@@ -75,8 +77,6 @@ typedef struct _LibbalsaVfs LibbalsaVfs;
 #include "mailbox_mh.h"
 #include "mailbox_maildir.h"
 
-#define ELEMENTS(x) (sizeof (x) / sizeof (x[0]))
-
 /*
  * Error domains for GError:
  */
@@ -109,22 +109,18 @@ enum {
 /*
  * Initialize the library
  */
-void libbalsa_init(LibBalsaInformationFunc information_callback);
-void libbalsa_set_spool(const gchar * spool);
+void libbalsa_init(void);
 
 void libbalsa_show_message_source(GtkApplication * application,
                                   LibBalsaMessage * msg,
                                   const gchar * font,
-                                  gboolean *escape_specials,
-                                  gint * width, gint * height);
-gchar *libbalsa_rot(const gchar * pass);
+                                  gboolean *escape_specials);
 
 gchar *libbalsa_guess_email_address(void);
 gchar *libbalsa_guess_mail_spool(void);
 gboolean libbalsa_is_sending_mail(void);
 void libbalsa_wait_for_sending_thread(gint max_seconds);
 
-gchar *libbalsa_guess_pop_server(void);
 gchar *libbalsa_guess_imap_server(void);
 gchar *libbalsa_guess_ldap_server(void);
 
@@ -134,25 +130,14 @@ gchar *libbalsa_guess_imap_inbox(void);
 gchar* libbalsa_date_to_utf8(time_t date, const gchar *date_string);
 LibBalsaMessageStatus libbalsa_get_icon_from_flags(LibBalsaMessageFlag flags);
 
-gboolean libbalsa_is_cert_known(X509* cert, long vfy_result);
+gboolean libbalsa_is_cert_known(GTlsCertificate      *cert,
+								GTlsCertificateFlags  errors);
 void libbalsa_certs_destroy(void);
 
-gboolean libbalsa_abort_on_timeout(const char *host);
-
 GThread *libbalsa_get_main_thread(void);
 gboolean libbalsa_am_i_subthread(void);
-#if defined(BALSA_DEBUG_THREADS)
-#define gdk_threads_enter()                       \
-    do if (libbalsa_am_i_subthread())             \
-        g_warning("%s: sub-thread!\n", __func__); \
-    while (0)
-#else
-#define gdk_threads_enter()
-#endif
-#define gdk_threads_leave()
 void libbalsa_message(const char *fmt, ...)
 	G_GNUC_PRINTF(1, 2);
-gchar * libbalsa_rot(const gchar * pass);
 
 typedef enum {
     LIBBALSA_PROGRESS_NO = 0,
@@ -201,11 +186,8 @@ enum LibBalsaImageError {
 #endif                          /* HAVE_COMPFACE */
 };
 
-#if GTK_CHECK_VERSION(3, 12, 0)
+gboolean libbalsa_use_headerbar(void);
 GtkDialogFlags libbalsa_dialog_flags(void);
-#else
-#define libbalsa_dialog_flags()		(GtkDialogFlags) (0)
-#endif
 
 #if HAVE_GTKSOURCEVIEW
 GtkWidget *libbalsa_source_view_new(gboolean highlight_phrases);
