@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2013 Stuart Parmenter and others,
+ * Copyright (C) 1997-2016 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *  
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef __LIBBALSA_PRIVATE_H__
@@ -45,18 +43,10 @@ struct LibBalsaMailboxIndexEntry_ {
     unsigned foreground_set:1;
     unsigned background_set:1;
     unsigned unseen:1;
-#ifdef BALSA_USE_THREADS
     unsigned idle_pending:1;
-#endif                          /* BALSA_USE_THREADS */
 } ;
 
-#ifdef BALSA_USE_THREADS
-#include <pthread.h>
 void libbalsa_lock_mailbox(LibBalsaMailbox * mailbox);
 void libbalsa_unlock_mailbox(LibBalsaMailbox * mailbox);
-#else
-# define libbalsa_lock_mailbox(m)
-# define libbalsa_unlock_mailbox(m)
-#endif
 
 #endif				/* __LIBBALSA_PRIVATE_H__ */
