@@ -54,12 +54,6 @@ struct message_info {
     ((((orig_flags) ^ (flags)) & LIBBALSA_MESSAGE_FLAGS_REAL) != 0)
 #define FLAGS_CHANGED(msg_info) \
     FLAGS_REALLY_DIFFER(msg_info->orig_flags, msg_info->local_info.flags)
-  
-
-static LibBalsaMailboxLocalClass *parent_class = NULL;
-
-static void libbalsa_mailbox_maildir_class_init(LibBalsaMailboxMaildirClass *klass);
-static void libbalsa_mailbox_maildir_init(LibBalsaMailboxMaildir * mailbox);
 
 /* Object class method */
 static void libbalsa_mailbox_maildir_finalize(GObject * object);
@@ -107,32 +101,19 @@ static void free_message_info(struct message_info *msg_info);
 static int libbalsa_mailbox_maildir_open_temp (const gchar *dest_path,
 					  char **name_used);
 
-GType
-libbalsa_mailbox_maildir_get_type(void)
-{
-    static GType mailbox_type = 0;
+struct _LibBalsaMailboxMaildir {
+    LibBalsaMailboxLocal parent;
 
-    if (!mailbox_type) {
-	static const GTypeInfo mailbox_info = {
-	    sizeof(LibBalsaMailboxMaildirClass),
-            NULL,               /* base_init */
-            NULL,               /* base_finalize */
-	    (GClassInitFunc) libbalsa_mailbox_maildir_class_init,
-            NULL,               /* class_finalize */
-            NULL,               /* class_data */
-	    sizeof(LibBalsaMailboxMaildir),
-            0,                  /* n_preallocs */
-	    (GInstanceInitFunc) libbalsa_mailbox_maildir_init
-	};
+    GHashTable* messages_info;
+    GPtrArray* msgno_2_msg_info;
+    gchar *curdir;
+    gchar *newdir;
+    gchar *tmpdir;
+};
 
-	mailbox_type =
-	    g_type_register_static(LIBBALSA_TYPE_MAILBOX_LOCAL,
-	                           "LibBalsaMailboxMaildir",
-                                   &mailbox_info, 0);
-    }
-
-    return mailbox_type;
-}
+G_DEFINE_TYPE(LibBalsaMailboxMaildir,
+              libbalsa_mailbox_maildir,
+              LIBBALSA_TYPE_MAILBOX_LOCAL)
 
 static void
 libbalsa_mailbox_maildir_class_init(LibBalsaMailboxMaildirClass * klass)
@@ -145,8 +126,6 @@ libbalsa_mailbox_maildir_class_init(LibBalsaMailboxMaildirClass * klass)
     libbalsa_mailbox_class = LIBBALSA_MAILBOX_CLASS(klass);
     libbalsa_mailbox_local_class = LIBBALSA_MAILBOX_LOCAL_CLASS(klass);
 
-    parent_class = g_type_class_peek_parent(klass);
-
     object_class->finalize = libbalsa_mailbox_maildir_finalize;
 
     libbalsa_mailbox_class->load_config =
@@ -264,7 +243,7 @@ libbalsa_mailbox_maildir_new(const gchar * path, gboolean create)
 
     mailbox = g_object_new(LIBBALSA_TYPE_MAILBOX_MAILDIR, NULL);
 
-    mailbox->is_directory = TRUE;
+    libbalsa_mailbox_set_is_directory(mailbox, TRUE);
 
     if (libbalsa_mailbox_local_set_path(LIBBALSA_MAILBOX_LOCAL(mailbox),
                                         path, create) != 0) {
@@ -286,8 +265,7 @@ libbalsa_mailbox_maildir_finalize(GObject * object)
     g_free(mdir->tmpdir);
     mdir->tmpdir = NULL;
 
-    if (G_OBJECT_CLASS(parent_class)->finalize)
-	G_OBJECT_CLASS(parent_class)->finalize(object);
+    G_OBJECT_CLASS(libbalsa_mailbox_maildir_parent_class)->finalize(object);
 }
 
 static void
@@ -301,7 +279,7 @@ libbalsa_mailbox_maildir_load_config(LibBalsaMailbox * mailbox,
     lbm_maildir_set_subdirs(mdir, path);
     g_free(path);
 
-    LIBBALSA_MAILBOX_CLASS(parent_class)->load_config(mailbox, prefix);
+    LIBBALSA_MAILBOX_CLASS(libbalsa_mailbox_maildir_parent_class)->load_config(mailbox, prefix);
 }
 
 static GMimeStream *
@@ -326,8 +304,10 @@ static void
 lbm_maildir_remove_files(LibBalsaMailboxLocal *mailbox)
 {
     const gchar* path;
+
     g_return_if_fail(LIBBALSA_IS_MAILBOX_MAILDIR(mailbox));
-    path = libbalsa_mailbox_local_get_path(mailbox);
+
+    path = libbalsa_mailbox_local_get_path((LibBalsaMailboxLocal *) mailbox);
     g_print("DELETE MAILDIR\n");
 
     if (!libbalsa_delete_directory_contents(path)) {
@@ -341,7 +321,7 @@ lbm_maildir_remove_files(LibBalsaMailboxLocal *mailbox)
 			     _("Could not remove %s:\n%s"),
 			     path, strerror(errno));
     }
-    LIBBALSA_MAILBOX_LOCAL_CLASS(parent_class)->remove_files(mailbox);
+    LIBBALSA_MAILBOX_LOCAL_CLASS(libbalsa_mailbox_maildir_parent_class)->remove_files(mailbox);
 }
 
 static LibBalsaMessageFlag parse_filename(const gchar *subdir,
@@ -386,6 +366,7 @@ static LibBalsaMessageFlag parse_filename(const gchar *subdir,
 static void lbm_maildir_parse(LibBalsaMailboxMaildir * mdir,
                               const gchar *subdir, guint * fileno)
 {
+    const gchar *local_path;
     gchar *path;
     GDir *dir;
     GHashTable *messages_info;
@@ -396,8 +377,8 @@ static void lbm_maildir_parse(LibBalsaMailboxMaildir * mdir,
     LibBalsaMessageFlag flags;
     LibBalsaMailbox *mailbox = (LibBalsaMailbox *) mdir;
 
-    path = g_build_filename(libbalsa_mailbox_local_get_path(mailbox),
-			    subdir, NULL);
+    local_path = libbalsa_mailbox_local_get_path((LibBalsaMailboxLocal *) mailbox);
+    path = g_build_filename(local_path, subdir, NULL);
     dir = g_dir_open(path, 0, NULL);
     g_free(path);
     if (dir == NULL)
@@ -477,7 +458,7 @@ libbalsa_mailbox_maildir_open(LibBalsaMailbox * mailbox, GError **err)
     g_return_val_if_fail(LIBBALSA_IS_MAILBOX_MAILDIR(mailbox), FALSE);
 
     mdir = LIBBALSA_MAILBOX_MAILDIR(mailbox);
-    path = libbalsa_mailbox_local_get_path(mailbox);
+    path = libbalsa_mailbox_local_get_path((LibBalsaMailboxLocal *) mailbox);
 
     if (stat(path, &st) == -1) {
 	g_set_error(err, LIBBALSA_MAILBOX_ERROR, LIBBALSA_MAILBOX_OPEN_ERROR,
@@ -492,16 +473,17 @@ libbalsa_mailbox_maildir_open(LibBalsaMailbox * mailbox, GError **err)
     if (stat(mdir->tmpdir, &st) != -1)
 	libbalsa_mailbox_set_mtime(mailbox, st.st_mtime);
 
-    mailbox->readonly = 
+    libbalsa_mailbox_set_readonly(mailbox,
 	!(access(mdir->curdir, W_OK) == 0 &&
           access(mdir->newdir, W_OK) == 0 &&
-          access(mdir->tmpdir, W_OK) == 0);
+          access(mdir->tmpdir, W_OK) == 0));
 
-    mailbox->unread_messages = 0;
+    libbalsa_mailbox_clear_unread_messages(mailbox);
     lbm_maildir_parse_subdirs(mdir);
 #ifdef DEBUG
     g_print(_("%s: Opening %s Refcount: %d\n"),
-	    "LibBalsaMailboxMaildir", mailbox->name, mailbox->open_ref);
+	    "LibBalsaMailboxMaildir", libbalsa_mailbox_get_name(mailbox),
+            libbalsa_mailbox_get_open_ref(mailbox));
 #endif
     return TRUE;
 }
@@ -570,7 +552,7 @@ libbalsa_mailbox_maildir_check(LibBalsaMailbox * mailbox)
     }
 
     /* Was any message removed? */
-    path = libbalsa_mailbox_local_get_path(mailbox);
+    path = libbalsa_mailbox_local_get_path((LibBalsaMailboxLocal *) mailbox);
     renumber = mdir->msgno_2_msg_info->len + 1;
     for (msgno = 1; msgno <= mdir->msgno_2_msg_info->len; ) {
 	gchar *filename;
@@ -592,8 +574,8 @@ libbalsa_mailbox_maildir_check(LibBalsaMailbox * mailbox)
     }
     for (msgno = renumber; msgno <= mdir->msgno_2_msg_info->len; msgno++) {
 	msg_info = message_info_from_msgno(mdir, msgno);
-	if (msg_info->local_info.message)
-	    msg_info->local_info.message->msgno = msgno;
+	if (msg_info->local_info.message != NULL)
+            libbalsa_message_set_msgno(msg_info->local_info.message, msgno);
     }
 
     msgno = mdir->msgno_2_msg_info->len;
@@ -608,9 +590,9 @@ free_message_info(struct message_info *msg_info)
 	return;
     g_free(msg_info->key);
     g_free(msg_info->filename);
-    if (msg_info->local_info.message) {
-	msg_info->local_info.message->mailbox = NULL;
-	msg_info->local_info.message->msgno   = 0;
+    if (msg_info->local_info.message != NULL) {
+        libbalsa_message_set_mailbox(msg_info->local_info.message, NULL);
+        libbalsa_message_set_msgno(msg_info->local_info.message, 0);
 	g_object_remove_weak_pointer(G_OBJECT(msg_info->local_info.message),
 				     (gpointer *) & msg_info->local_info.message);
     }
@@ -629,8 +611,8 @@ libbalsa_mailbox_maildir_close_mailbox(LibBalsaMailbox * mailbox,
     if (mdir->msgno_2_msg_info->len != len)
         libbalsa_mailbox_changed(mailbox);
 
-    if (LIBBALSA_MAILBOX_CLASS(parent_class)->close_mailbox)
-        LIBBALSA_MAILBOX_CLASS(parent_class)->close_mailbox(mailbox,
+    if (LIBBALSA_MAILBOX_CLASS(libbalsa_mailbox_maildir_parent_class)->close_mailbox)
+        LIBBALSA_MAILBOX_CLASS(libbalsa_mailbox_maildir_parent_class)->close_mailbox(mailbox,
                                                             expunge);
 
     /* Now it's safe to free the message info. */
@@ -748,7 +730,7 @@ libbalsa_mailbox_maildir_sync(LibBalsaMailbox * mailbox, gboolean expunge)
      * record mtime of dirs
      */
     LibBalsaMailboxMaildir *mdir = LIBBALSA_MAILBOX_MAILDIR(mailbox);
-    const gchar *path = libbalsa_mailbox_local_get_path(mailbox);
+    const gchar *path = libbalsa_mailbox_local_get_path((LibBalsaMailboxLocal *) mailbox);
     GSList *removed_list = NULL;
     gboolean ok = TRUE;
     GSList *l;
@@ -771,7 +753,7 @@ libbalsa_mailbox_maildir_sync(LibBalsaMailbox * mailbox, gboolean expunge)
 	    continue;
 	}
 
-	if (mailbox->state == LB_MAILBOX_STATE_CLOSING)
+	if (libbalsa_mailbox_get_state(mailbox) == LB_MAILBOX_STATE_CLOSING)
 	    msg_info->local_info.flags &= ~LIBBALSA_MESSAGE_FLAG_RECENT;
 	if (((msg_info->local_info.flags & LIBBALSA_MESSAGE_FLAG_RECENT)
              && strcmp(msg_info->subdir, "new") != 0)
@@ -804,8 +786,8 @@ libbalsa_mailbox_maildir_sync(LibBalsaMailbox * mailbox, gboolean expunge)
     g_slist_free(removed_list);
     for (msgno = renumber; msgno <= mdir->msgno_2_msg_info->len; msgno++) {
 	msg_info = message_info_from_msgno(mdir, msgno);
-	if (msg_info->local_info.message)
-	    msg_info->local_info.message->msgno = msgno;
+	if (msg_info->local_info.message != NULL)
+	    libbalsa_message_set_msgno(msg_info->local_info.message, msgno);
     }
 
     if (changes) {              /* Record mtime of dir. */
@@ -813,7 +795,7 @@ libbalsa_mailbox_maildir_sync(LibBalsaMailbox * mailbox, gboolean expunge)
 
         /* Reparse, to get the fileno entries right. */
         lbm_maildir_parse_subdirs(mdir);
-        mailbox->msg_tree_changed = TRUE;
+        libbalsa_mailbox_set_msg_tree_changed(mailbox, TRUE);
 
         if (stat(mdir->tmpdir, &st) == 0)
             libbalsa_mailbox_set_mtime(mailbox, st.st_mtime);
@@ -838,17 +820,24 @@ libbalsa_mailbox_maildir_fetch_message_structure(LibBalsaMailbox * mailbox,
 						 LibBalsaMessage * message,
 						 LibBalsaFetchFlag flags)
 {
-    if (!message->mime_msg) {
+    GMimeMessage *mime_message;
+
+    mime_message = libbalsa_message_get_mime_message(message);
+
+    if (mime_message == NULL) {
 	struct message_info *msg_info =
             message_info_from_msgno((LibBalsaMailboxMaildir *) mailbox,
-                                    message->msgno);
-	message->mime_msg =
+                                    libbalsa_message_get_msgno(message));
+        mime_message =
 	    libbalsa_mailbox_local_get_mime_message(mailbox,
 						    msg_info->subdir,
 						    msg_info->filename);
+        libbalsa_message_set_mime_message(message, mime_message);
+        if (mime_message != NULL)
+            g_object_unref(mime_message);
     }
 
-    return LIBBALSA_MAILBOX_CLASS(parent_class)->
+    return LIBBALSA_MAILBOX_CLASS(libbalsa_mailbox_maildir_parent_class)->
         fetch_message_structure(mailbox, message, flags);
 }
 
