@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2016 Stuart Parmenter and others,
+ * Copyright (C) 1997-2019 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -61,10 +61,6 @@ struct message_info {
 #define FLAGS_REALLY_DIFFER(orig_flags, flags) \
     ((((orig_flags) ^ (flags)) & LIBBALSA_MESSAGE_FLAGS_REAL) != 0)
 
-static LibBalsaMailboxLocalClass *parent_class = NULL;
-
-static void libbalsa_mailbox_mbox_class_init(LibBalsaMailboxMboxClass *klass);
-static void libbalsa_mailbox_mbox_init(LibBalsaMailboxMbox * mailbox);
 static void libbalsa_mailbox_mbox_dispose(GObject * object);
 
 static GMimeStream *libbalsa_mailbox_mbox_get_message_stream(LibBalsaMailbox *
@@ -96,10 +92,6 @@ libbalsa_mailbox_mbox_total_messages(LibBalsaMailbox * mailbox);
 static void libbalsa_mailbox_mbox_lock_store(LibBalsaMailbox * mailbox,
                                              gboolean lock);
 
-struct _LibBalsaMailboxMboxClass {
-    LibBalsaMailboxLocalClass klass;
-};
-
 struct _LibBalsaMailboxMbox {
     LibBalsaMailboxLocal parent;
 
@@ -109,31 +101,9 @@ struct _LibBalsaMailboxMbox {
     gboolean messages_info_changed;
 };
 
-GType libbalsa_mailbox_mbox_get_type(void)
-{
-    static GType mailbox_type = 0;
-
-    if (!mailbox_type) {
-	static const GTypeInfo mailbox_info = {
-	    sizeof(LibBalsaMailboxMboxClass),
-            NULL,               /* base_init */
-            NULL,               /* base_finalize */
-	    (GClassInitFunc) libbalsa_mailbox_mbox_class_init,
-            NULL,               /* class_finalize */
-            NULL,               /* class_data */
-	    sizeof(LibBalsaMailboxMbox),
-            0,                  /* n_preallocs */
-	    (GInstanceInitFunc) libbalsa_mailbox_mbox_init
-	};
-
-	mailbox_type =
-	    g_type_register_static(LIBBALSA_TYPE_MAILBOX_LOCAL,
-	                           "LibBalsaMailboxMbox",
-                                   &mailbox_info, 0);
-    }
-
-    return mailbox_type;
-}
+G_DEFINE_TYPE(LibBalsaMailboxMbox,
+              libbalsa_mailbox_mbox,
+              LIBBALSA_TYPE_MAILBOX_LOCAL)
 
 static void
 libbalsa_mailbox_mbox_class_init(LibBalsaMailboxMboxClass * klass)
@@ -146,8 +116,6 @@ libbalsa_mailbox_mbox_class_init(LibBalsaMailboxMboxClass * klass)
     libbalsa_mailbox_class = LIBBALSA_MAILBOX_CLASS(klass);
     libbalsa_mailbox_local_class = LIBBALSA_MAILBOX_LOCAL_CLASS(klass);
 
-    parent_class = g_type_class_peek_parent(klass);
-
     libbalsa_mailbox_class->get_message_stream =
 	libbalsa_mailbox_mbox_get_message_stream;
 
@@ -182,15 +150,16 @@ libbalsa_mailbox_mbox_init(LibBalsaMailboxMbox * mbox)
 static void
 libbalsa_mailbox_mbox_dispose(GObject * object)
 {
-    if(MAILBOX_OPEN(LIBBALSA_MAILBOX(object)))
-	libbalsa_mailbox_mbox_close_mailbox(LIBBALSA_MAILBOX(object), FALSE);
+    LibBalsaMailbox *mailbox = LIBBALSA_MAILBOX(object);
+
+    if (MAILBOX_OPEN(mailbox))
+	libbalsa_mailbox_mbox_close_mailbox(mailbox, FALSE);
+    G_OBJECT_CLASS(libbalsa_mailbox_mbox_parent_class)->dispose(object);
 }
 
 static gint
 lbm_mbox_check_files(const gchar * path, gboolean create)
 {
-    g_return_val_if_fail(path != NULL, -1);
-
     if (access(path, F_OK) == 0) {
         /* File exists. Check if it is an mbox... */
         if (libbalsa_mailbox_type_from_path(path) !=
@@ -223,7 +192,7 @@ libbalsa_mailbox_mbox_new(const gchar * path, gboolean create)
 
     mailbox = g_object_new(LIBBALSA_TYPE_MAILBOX_MBOX, NULL);
 
-    mailbox->is_directory = FALSE;
+    libbalsa_mailbox_set_is_directory(mailbox, FALSE);
 
     if (libbalsa_mailbox_local_set_path(LIBBALSA_MAILBOX_LOCAL(mailbox),
                                         path, create) != 0) {
@@ -291,8 +260,6 @@ libbalsa_mailbox_mbox_get_message_stream(LibBalsaMailbox * mailbox,
     LibBalsaMailboxMbox *mbox;
     struct message_info *msg_info;
 
-    g_return_val_if_fail(LIBBALSA_IS_MAILBOX_MBOX(mailbox), NULL);
-
     mbox = LIBBALSA_MAILBOX_MBOX(mailbox);
     msg_info = message_info_from_msgno(mbox, msgno);
 
@@ -307,26 +274,26 @@ libbalsa_mailbox_mbox_get_message_stream(LibBalsaMailbox * mailbox,
 static void
 libbalsa_mailbox_mbox_remove_files(LibBalsaMailboxLocal *mailbox)
 {
-    g_return_if_fail (LIBBALSA_IS_MAILBOX_MBOX(mailbox));
-
     if ( unlink(libbalsa_mailbox_local_get_path(mailbox)) == -1 )
 	libbalsa_information(LIBBALSA_INFORMATION_ERROR, 
 			     _("Could not remove %s:\n%s"), 
 			     libbalsa_mailbox_local_get_path(mailbox), 
 			     strerror(errno));
-    LIBBALSA_MAILBOX_LOCAL_CLASS(parent_class)->remove_files(mailbox);
+    LIBBALSA_MAILBOX_LOCAL_CLASS(libbalsa_mailbox_mbox_parent_class)->remove_files(mailbox);
 }
 
 static int mbox_lock(LibBalsaMailbox * mailbox, GMimeStream *stream)
 {
-    const gchar *path = libbalsa_mailbox_local_get_path(mailbox);
+    const gchar *path =
+        libbalsa_mailbox_local_get_path(LIBBALSA_MAILBOX_LOCAL(mailbox));
     int fd = GMIME_STREAM_FS(stream)->fd;
     return libbalsa_lock_file(path, fd, FALSE, TRUE, 1);
 }
 
 static void mbox_unlock(LibBalsaMailbox * mailbox, GMimeStream *stream)
 {
-    const gchar *path = libbalsa_mailbox_local_get_path(mailbox);
+    const gchar *path =
+        libbalsa_mailbox_local_get_path(LIBBALSA_MAILBOX_LOCAL(mailbox));
     int fd = GMIME_STREAM_FS(stream)->fd;
     libbalsa_unlock_file(path, fd, 1);
 }
@@ -537,10 +504,10 @@ parse_mailbox(LibBalsaMailboxMbox * mbox)
                         g_memdup(&msg_info, sizeof(msg_info)));
         mbox->messages_info_changed = TRUE;
 
-        msg->flags = msg_info.orig_flags;
-        msg->length = msg_info.end - (msg_info.start + msg_info.from_len);
-        msg->mailbox = LIBBALSA_MAILBOX(mbox);
-        msg->msgno = ++msgno;
+        libbalsa_message_set_flags(msg, msg_info.orig_flags);
+        libbalsa_message_set_length(msg, msg_info.end - (msg_info.start + msg_info.from_len));
+        libbalsa_message_set_mailbox(msg, LIBBALSA_MAILBOX(mbox));
+        libbalsa_message_set_msgno(msg, ++msgno);
 	/* We must drop the mime-stream lock to call
          * libbalsa_mailbox_local_cache_message, which calls
 	 * libbalsa_mailbox_cache_message(), as it may grab the
@@ -664,12 +631,13 @@ free_message_info(struct message_info *msg_info)
     LibBalsaMessage *message;
 
     if ((message = msg_info->local_info.message) != NULL) {
-        message->mailbox = NULL;
-        message->msgno   = 0;
+        libbalsa_message_set_mailbox(message, NULL);
+        libbalsa_message_set_msgno(message, 0);
         g_object_remove_weak_pointer(G_OBJECT(message),
                                      (gpointer *) &msg_info->local_info.message);
         msg_info->local_info.message = NULL;
     }
+
     g_free(msg_info);
 }
 
@@ -679,11 +647,12 @@ libbalsa_mailbox_mbox_open(LibBalsaMailbox * mailbox, GError **err)
     LibBalsaMailboxMbox *mbox = LIBBALSA_MAILBOX_MBOX(mailbox);
     struct stat st;
     const gchar* path;
+    gboolean readonly;
     int fd;
     GMimeStream *gmime_stream;
     time_t t0;
 
-    path = libbalsa_mailbox_local_get_path(mailbox);
+    path = libbalsa_mailbox_local_get_path(LIBBALSA_MAILBOX_LOCAL(mailbox));
 
     if (stat(path, &st) == -1) {
 	g_set_error(err, LIBBALSA_MAILBOX_ERROR, LIBBALSA_MAILBOX_OPEN_ERROR,
@@ -691,8 +660,9 @@ libbalsa_mailbox_mbox_open(LibBalsaMailbox * mailbox, GError **err)
 	return FALSE;
     }
 
-    mailbox->readonly = access (path, W_OK);
-    fd = open(path, mailbox->readonly ? O_RDONLY : O_RDWR);
+    readonly = access (path, W_OK);
+    libbalsa_mailbox_set_readonly(mailbox, readonly);
+    fd = open(path, readonly ? O_RDONLY : O_RDWR);
     if (fd == -1) {
 	g_set_error(err, LIBBALSA_MAILBOX_ERROR, LIBBALSA_MAILBOX_OPEN_ERROR,
 		    _("Cannot open mailbox."));
@@ -730,7 +700,7 @@ libbalsa_mailbox_mbox_open(LibBalsaMailbox * mailbox, GError **err)
     mbox->msgno_2_msg_info =
         g_ptr_array_new_with_free_func((GDestroyNotify) free_message_info);
 
-    mailbox->unread_messages = 0;
+    libbalsa_mailbox_clear_unread_messages(mailbox);
     time(&t0);
 
     if (st.st_size > 0) {
@@ -979,7 +949,7 @@ libbalsa_mailbox_mbox_check(LibBalsaMailbox * mailbox)
     g_assert(LIBBALSA_IS_MAILBOX_MBOX(mailbox));
 
     mbox = LIBBALSA_MAILBOX_MBOX(mailbox);
-    path = libbalsa_mailbox_local_get_path(mailbox);
+    path = libbalsa_mailbox_local_get_path(LIBBALSA_MAILBOX_LOCAL(mailbox));
     if (mbox->gmime_stream ?
         fstat(GMIME_STREAM_FS(mbox->gmime_stream)->fd, &st) :
         stat(path, &st)) {
@@ -1076,7 +1046,7 @@ libbalsa_mailbox_mbox_check(LibBalsaMailbox * mailbox)
 
         if ((msg_info->local_info.flags & LIBBALSA_MESSAGE_FLAG_NEW)
             && !(msg_info->local_info.flags & LIBBALSA_MESSAGE_FLAG_DELETED))
-            --mailbox->unread_messages;
+            libbalsa_mailbox_add_to_unread_messages(mailbox, -1);
 
 	/* We must drop the mime-stream lock to call
 	 * libbalsa_mailbox_local_msgno_removed(), as it will grab the
@@ -1116,13 +1086,16 @@ libbalsa_mailbox_mbox_close_mailbox(LibBalsaMailbox * mailbox,
     LibBalsaMailboxMbox *mbox = LIBBALSA_MAILBOX_MBOX(mailbox);
     guint len;
 
+    if (mbox->msgno_2_msg_info == NULL)
+        return;
+
     len = mbox->msgno_2_msg_info->len;
     libbalsa_mailbox_mbox_sync(mailbox, expunge);
     if (mbox->msgno_2_msg_info->len != len)
         libbalsa_mailbox_changed(mailbox);
 
-    if (LIBBALSA_MAILBOX_CLASS(parent_class)->close_mailbox)
-        LIBBALSA_MAILBOX_CLASS(parent_class)->close_mailbox(mailbox,
+    if (LIBBALSA_MAILBOX_CLASS(libbalsa_mailbox_mbox_parent_class)->close_mailbox)
+        LIBBALSA_MAILBOX_CLASS(libbalsa_mailbox_mbox_parent_class)->close_mailbox(mailbox,
                                                             expunge);
 
     /* Now it's safe to close the stream and free the message info. */
@@ -1432,7 +1405,7 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
 	return TRUE;
     mbox_stream = mbox->gmime_stream;
 
-    path = libbalsa_mailbox_local_get_path(mailbox);
+    path = libbalsa_mailbox_local_get_path(LIBBALSA_MAILBOX_LOCAL(mailbox));
 
     /* lock mailbox file */
     if (mbox_lock(mailbox, mbox_stream) != 0)
@@ -1457,7 +1430,7 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
     for (i = j = 0; i < messages; i++)
     {
 	msg_info = message_info_from_msgno(mbox, i + 1);
-	if (mailbox->state == LB_MAILBOX_STATE_CLOSING)
+	if (libbalsa_mailbox_get_state(mailbox) == LB_MAILBOX_STATE_CLOSING)
 	    msg_info->local_info.flags &= ~LIBBALSA_MESSAGE_FLAG_RECENT;
 	if (expunge && (msg_info->local_info.flags & LIBBALSA_MESSAGE_FLAG_DELETED))
 	    break;
@@ -1643,7 +1616,7 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
     unlink(tempfile); /* remove partial copy of the mailbox */
     g_free(tempfile);
 
-    if (mailbox->state == LB_MAILBOX_STATE_CLOSING) {
+    if (libbalsa_mailbox_get_state(mailbox) == LB_MAILBOX_STATE_CLOSING) {
 	/* Just shorten the msg_info array. */
 	for (j = first; j < mbox->msgno_2_msg_info->len; ) {
 	    msg_info = message_info_from_msgno(mbox, j + 1);
@@ -1693,8 +1666,8 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
             mbox->messages_info_changed = TRUE;
 	    continue;
 	}
-	if (msg_info->local_info.message)
-	    msg_info->local_info.message->msgno = j + 1;
+	if (msg_info->local_info.message != NULL)
+	    libbalsa_message_set_msgno(msg_info->local_info.message, j + 1);
 
 	msg_info->status = msg_info->x_status = msg_info->mime_version = -1;
 	mime_msg = g_mime_parser_construct_message(gmime_parser);
@@ -1723,18 +1696,19 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
 	msg_info->end = g_mime_parser_tell(gmime_parser);
 	msg_info->orig_flags = REAL_FLAGS(msg_info->local_info.flags);
 	g_assert(mime_msg->mime_part != NULL);
-	if (!msg_info->local_info.message || !msg_info->local_info.message->mime_msg)
-	    g_object_unref(mime_msg);
-	else {
-	    g_object_unref(msg_info->local_info.message->mime_msg);
-	    msg_info->local_info.message->mime_msg = mime_msg;
+
+	if (msg_info->local_info.message != NULL &&
+            libbalsa_message_get_mime_message(msg_info->local_info.message) != NULL) {
+            libbalsa_message_set_mime_message(msg_info->local_info.message, mime_msg);
 	    /*
 	     * reinit the message parts info
 	     */
-	    libbalsa_message_body_set_mime_body(msg_info->local_info.message->body_list,
+	    libbalsa_message_body_set_mime_body(libbalsa_message_get_body_list(msg_info->local_info.message),
 						mime_msg->mime_part);
 	}
 
+        g_object_unref(mime_msg);
+
 	j++;
     }
     libbalsa_mime_stream_shared_unlock(mbox_stream);
@@ -1759,10 +1733,14 @@ libbalsa_mailbox_mbox_fetch_message_structure(LibBalsaMailbox * mailbox,
 					      LibBalsaMessage * message,
 					      LibBalsaFetchFlag flags)
 {
-    if (!message->mime_msg)
-	message->mime_msg = lbm_mbox_get_mime_message(mailbox, message->msgno);
+    if (libbalsa_message_get_mime_message(message) == NULL) {
+        GMimeMessage *mime_msg =
+            lbm_mbox_get_mime_message(mailbox, libbalsa_message_get_msgno(message));
+        libbalsa_message_set_mime_message(message, mime_msg);
+        g_object_unref(mime_msg);
+    }
 
-    return LIBBALSA_MAILBOX_CLASS(parent_class)->
+    return LIBBALSA_MAILBOX_CLASS(libbalsa_mailbox_mbox_parent_class)->
         fetch_message_structure(mailbox, message, flags);
 }
 
@@ -1881,6 +1859,7 @@ lbm_mbox_add_message(LibBalsaMailboxLocal * local,
 {
     LibBalsaMailbox *mailbox = (LibBalsaMailbox *) local;
     LibBalsaMessage *message;
+    LibBalsaMessageHeaders *headers;
     gchar date_string[27];
     gchar *sender;
     gchar *address;
@@ -1897,10 +1876,11 @@ lbm_mbox_add_message(LibBalsaMailboxLocal * local,
     message = libbalsa_message_new();
     libbalsa_message_load_envelope_from_stream(message, stream);
 
-    ctime_r(&(message->headers->date), date_string);
+    headers = libbalsa_message_get_headers(message);
+    ctime_r(&(headers->date), date_string);
 
-    sender = message->headers->from ?
-	internet_address_list_to_string(message->headers->from, FALSE) :
+    sender = headers->from != NULL ?
+        internet_address_list_to_string(headers->from, FALSE) :
 	g_strdup("none");
 
     g_object_unref(message);
