@@ -48,10 +48,6 @@ struct message_info {
 
 #define REAL_FLAGS(flags) (flags & LIBBALSA_MESSAGE_FLAGS_REAL)
 
-static LibBalsaMailboxLocalClass *parent_class = NULL;
-
-static void libbalsa_mailbox_mh_class_init(LibBalsaMailboxMhClass *klass);
-static void libbalsa_mailbox_mh_init(LibBalsaMailboxMh * mailbox);
 static void libbalsa_mailbox_mh_finalize(GObject * object);
 static void libbalsa_mailbox_mh_load_config(LibBalsaMailbox * mailbox,
                                             const gchar * prefix);
@@ -86,33 +82,19 @@ static gboolean libbalsa_mailbox_mh_fetch_message_structure(LibBalsaMailbox
                                                             flags);
 static guint libbalsa_mailbox_mh_total_messages(LibBalsaMailbox * mailbox);
 
+struct _LibBalsaMailboxMh {
+    LibBalsaMailboxLocal parent;
 
-GType
-libbalsa_mailbox_mh_get_type(void)
-{
-    static GType mailbox_type = 0;
+    GHashTable* messages_info;
+    GPtrArray* msgno_2_msg_info;
+    gchar* sequences_filename;
+    time_t mtime_sequences;
+    guint last_fileno;
+};
 
-    if (!mailbox_type) {
-	static const GTypeInfo mailbox_info = {
-	    sizeof(LibBalsaMailboxMhClass),
-            NULL,               /* base_init */
-            NULL,               /* base_finalize */
-	    (GClassInitFunc) libbalsa_mailbox_mh_class_init,
-            NULL,               /* class_finalize */
-            NULL,               /* class_data */
-	    sizeof(LibBalsaMailboxMh),
-            0,                  /* n_preallocs */
-	    (GInstanceInitFunc) libbalsa_mailbox_mh_init
-	};
-
-	mailbox_type =
-	    g_type_register_static(LIBBALSA_TYPE_MAILBOX_LOCAL,
-	                           "LibBalsaMailboxMh",
-                                   &mailbox_info, 0);
-    }
-
-    return mailbox_type;
-}
+G_DEFINE_TYPE(LibBalsaMailboxMh,
+              libbalsa_mailbox_mh,
+              LIBBALSA_TYPE_MAILBOX_LOCAL)
 
 static void
 libbalsa_mailbox_mh_class_init(LibBalsaMailboxMhClass * klass)
@@ -125,8 +107,6 @@ libbalsa_mailbox_mh_class_init(LibBalsaMailboxMhClass * klass)
     libbalsa_mailbox_class = LIBBALSA_MAILBOX_CLASS(klass);
     libbalsa_mailbox_local_class = LIBBALSA_MAILBOX_LOCAL_CLASS(klass);
 
-    parent_class = g_type_class_peek_parent(klass);
-
     object_class->finalize = libbalsa_mailbox_mh_finalize;
 
     libbalsa_mailbox_class->load_config = libbalsa_mailbox_mh_load_config;
@@ -221,7 +201,7 @@ libbalsa_mailbox_mh_new(const gchar * path, gboolean create)
 
     mailbox = g_object_new(LIBBALSA_TYPE_MAILBOX_MH, NULL);
 
-    mailbox->is_directory = TRUE;
+    libbalsa_mailbox_set_is_directory(mailbox, TRUE);
 
     if (libbalsa_mailbox_local_set_path(LIBBALSA_MAILBOX_LOCAL(mailbox),
                                         path, create) != 0) {
@@ -237,7 +217,7 @@ libbalsa_mailbox_mh_finalize(GObject * object)
 {
     LibBalsaMailboxMh *mh = LIBBALSA_MAILBOX_MH(object);
     g_free(mh->sequences_filename);
-    G_OBJECT_CLASS(parent_class)->finalize(object);
+    G_OBJECT_CLASS(libbalsa_mailbox_mh_parent_class)->finalize(object);
 }
 
 static void
@@ -251,7 +231,7 @@ libbalsa_mailbox_mh_load_config(LibBalsaMailbox * mailbox,
     lbm_mh_set_sequences_filename(mh, path);
     g_free(path);
 
-    LIBBALSA_MAILBOX_CLASS(parent_class)->load_config(mailbox, prefix);
+    LIBBALSA_MAILBOX_CLASS(libbalsa_mailbox_mh_parent_class)->load_config(mailbox, prefix);
 }
 
 #define MH_BASENAME(msg_info) \
@@ -295,7 +275,7 @@ lbm_mh_remove_files(LibBalsaMailboxLocal *mailbox)
 			     _("Could not remove %s:\n%s"),
 			     path, strerror(errno));
     }
-    LIBBALSA_MAILBOX_LOCAL_CLASS(parent_class)->remove_files(mailbox);
+    LIBBALSA_MAILBOX_LOCAL_CLASS(libbalsa_mailbox_mh_parent_class)->remove_files(mailbox);
 }
 
 
@@ -340,7 +320,7 @@ lbm_mh_parse_mailbox(LibBalsaMailboxMh * mh, gboolean add_msg_info)
     GDir *dir;
     const gchar *filename;
 
-    path = libbalsa_mailbox_local_get_path(mh);
+    path = libbalsa_mailbox_local_get_path((LibBalsaMailboxLocal *) mh);
 
     if ((dir = g_dir_open(path, 0, NULL)) == NULL)
 	return;
@@ -497,14 +477,16 @@ lbm_mh_parse_both(LibBalsaMailboxMh * mh)
 static void
 lbm_mh_free_message_info(struct message_info *msg_info)
 {
-    if (!msg_info)
+    if (msg_info == NULL)
 	return;
-    if (msg_info->local_info.message) {
-	msg_info->local_info.message->mailbox = NULL;
-	msg_info->local_info.message->msgno   = 0;
+
+    if (msg_info->local_info.message != NULL) {
+        libbalsa_message_set_mailbox(msg_info->local_info.message, NULL);
+        libbalsa_message_set_msgno(msg_info->local_info.message, 0);
 	g_object_remove_weak_pointer(G_OBJECT(msg_info->local_info.message),
 				     (gpointer *) &msg_info->local_info.message);
     }
+
     g_free(msg_info);
 }
 
@@ -515,7 +497,7 @@ libbalsa_mailbox_mh_open(LibBalsaMailbox * mailbox, GError **err)
     struct stat st;
     const gchar* path;
    
-    path = libbalsa_mailbox_local_get_path(mailbox);
+    path = libbalsa_mailbox_local_get_path((LibBalsaMailboxLocal *) mailbox);
 
     if (stat(path, &st) == -1) {
 	g_set_error(err, LIBBALSA_MAILBOX_ERROR, LIBBALSA_MAILBOX_OPEN_ERROR,
@@ -531,8 +513,8 @@ libbalsa_mailbox_mh_open(LibBalsaMailbox * mailbox, GError **err)
     mh->msgno_2_msg_info = g_ptr_array_new();
     mh->last_fileno = 0;
     
-    mailbox->readonly = access (path, W_OK);
-    mailbox->unread_messages = 0;
+    libbalsa_mailbox_set_readonly(mailbox, access(path, W_OK));
+    libbalsa_mailbox_clear_unread_messages(mailbox);
     lbm_mh_parse_both(mh);
 
 #ifdef DEBUG
@@ -613,7 +595,8 @@ libbalsa_mailbox_mh_check(LibBalsaMailbox * mailbox)
 {
     struct stat st, st_sequences;
     LibBalsaMailboxMh *mh = LIBBALSA_MAILBOX_MH(mailbox);
-    const gchar *path = libbalsa_mailbox_local_get_path(mailbox);
+    const gchar *path =
+        libbalsa_mailbox_local_get_path((LibBalsaMailboxLocal *) mailbox);
     int modified = 0;
     guint renumber, msgno;
     struct message_info *msg_info;
@@ -689,8 +672,8 @@ libbalsa_mailbox_mh_check(LibBalsaMailbox * mailbox)
     }
     for (msgno = renumber; msgno <= mh->msgno_2_msg_info->len; msgno++) {
 	msg_info = lbm_mh_message_info_from_msgno(mh, msgno);
-	if (msg_info->local_info.message)
-	    msg_info->local_info.message->msgno = msgno;
+	if (msg_info->local_info.message != NULL)
+	    libbalsa_message_set_msgno(msg_info->local_info.message, msgno);
     }
 
     msgno = mh->msgno_2_msg_info->len;
@@ -713,8 +696,8 @@ libbalsa_mailbox_mh_close_mailbox(LibBalsaMailbox * mailbox,
     g_hash_table_destroy(mh->messages_info);
     mh->messages_info = NULL;
 
-    if (LIBBALSA_MAILBOX_CLASS(parent_class)->close_mailbox)
-        LIBBALSA_MAILBOX_CLASS(parent_class)->close_mailbox(mailbox,
+    if (LIBBALSA_MAILBOX_CLASS(libbalsa_mailbox_mh_parent_class)->close_mailbox)
+        LIBBALSA_MAILBOX_CLASS(libbalsa_mailbox_mh_parent_class)->close_mailbox(mailbox,
                                                             expunge);
 
     /* Now it's safe to free the message info. */
@@ -819,14 +802,14 @@ libbalsa_mailbox_mh_sync(LibBalsaMailbox * mailbox, gboolean expunge)
     recent.first  = recent.last = -1;
     recent.line   = g_mime_stream_mem_new();
 
-    path = libbalsa_mailbox_local_get_path(mailbox);
+    path = libbalsa_mailbox_local_get_path((LibBalsaMailboxLocal *) mailbox);
 
     msgno = 1;
     while (msgno <= mh->msgno_2_msg_info->len) {
 	msg_info = lbm_mh_message_info_from_msgno(mh, msgno);
 	if (msg_info->local_info.flags == LIBBALSA_MESSAGE_FLAG_INVALID)
 	    msg_info->local_info.flags = msg_info->orig_flags;
-	if (mailbox->state == LB_MAILBOX_STATE_CLOSING)
+	if (libbalsa_mailbox_get_state(mailbox) == LB_MAILBOX_STATE_CLOSING)
 	    msg_info->local_info.flags &= ~LIBBALSA_MESSAGE_FLAG_RECENT;
 
 	if (expunge && (msg_info->local_info.flags
@@ -886,12 +869,12 @@ libbalsa_mailbox_mh_sync(LibBalsaMailbox * mailbox, gboolean expunge)
     /* Renumber */
     for (msgno = 1; msgno <= mh->msgno_2_msg_info->len; msgno++) {
 	msg_info = lbm_mh_message_info_from_msgno(mh, msgno);
-	if (msg_info->local_info.message)
-	    msg_info->local_info.message->msgno = msgno;
+	if (msg_info->local_info.message != NULL)
+	    libbalsa_message_set_msgno(msg_info->local_info.message, msgno);
     }
 
     /* open tempfile */
-    path = libbalsa_mailbox_local_get_path(mailbox);
+    path = libbalsa_mailbox_local_get_path((LibBalsaMailboxLocal *) mailbox);
     fd = libbalsa_mailbox_mh_open_temp(path, &name_used);
     if (fd == -1)
     {
@@ -1002,28 +985,34 @@ libbalsa_mailbox_mh_fetch_message_structure(LibBalsaMailbox * mailbox,
 					    LibBalsaMessage * message,
 					    LibBalsaFetchFlag flags)
 {
-    if (!message->mime_msg) {
+    GMimeMessage *mime_msg;
+
+    mime_msg = libbalsa_message_get_mime_message(message);
+
+    if (mime_msg == NULL) {
 	struct message_info *msg_info;
 	gchar *base_name;
 
 	msg_info =
 	    lbm_mh_message_info_from_msgno(LIBBALSA_MAILBOX_MH(mailbox),
-					   message->msgno);
+					   libbalsa_message_get_msgno(message));
 
 	base_name = MH_BASENAME(msg_info);
-	message->mime_msg =
-	    libbalsa_mailbox_local_get_mime_message(mailbox, base_name, NULL);
+	mime_msg = libbalsa_mailbox_local_get_mime_message(mailbox, base_name, NULL);
 	g_free(base_name);
 
-	if (message->mime_msg) {
-	    g_mime_object_remove_header(GMIME_OBJECT(message->mime_msg),
+        libbalsa_message_set_mime_message(message, mime_msg);
+
+	if (mime_msg != NULL) {
+	    g_mime_object_remove_header(GMIME_OBJECT(mime_msg),
 				        "Status");
-	    g_mime_object_remove_header(GMIME_OBJECT(message->mime_msg),
+	    g_mime_object_remove_header(GMIME_OBJECT(mime_msg),
 				        "X-Status");
+            g_object_unref(mime_msg);
 	}
     }
 
-    return LIBBALSA_MAILBOX_CLASS(parent_class)->
+    return LIBBALSA_MAILBOX_CLASS(libbalsa_mailbox_mh_parent_class)->
         fetch_message_structure(mailbox, message, flags);
 }
 
