@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2016 Stuart Parmenter and others,
+ * Copyright (C) 1997-2019 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -40,23 +40,26 @@
 enum {
     LAST_SIGNAL
 };
-static LibBalsaMailboxClass *parent_class = NULL;
 
-struct _LibBalsaMailboxPop3Class {
-    LibBalsaMailboxRemoteClass klass;
+struct _LibBalsaMailboxPOP3 {
+    LibBalsaMailboxRemote mailbox;
 
-    void (*config_changed) (LibBalsaMailboxPop3* mailbox);
+    gboolean check;
+    gboolean delete_from_server;
+    gchar *filter_cmd;
+    LibBalsaMailbox *inbox;
+    gint msg_size_limit;
+    gboolean filter; /* filter through procmail/filter_cmd? */
+    gboolean disable_apop; /* Some servers claim to support it but
+                              * they do not. */
+    gboolean enable_pipe;  /* ditto */
 };
 
 static void libbalsa_mailbox_pop3_finalize(GObject * object);
-static void libbalsa_mailbox_pop3_class_init(LibBalsaMailboxPop3Class *
-					     klass);
-static void libbalsa_mailbox_pop3_init(LibBalsaMailboxPop3 * mailbox);
 
 static gboolean libbalsa_mailbox_pop3_open(LibBalsaMailbox * mailbox,
 					   GError **err);
 static void libbalsa_mailbox_pop3_check(LibBalsaMailbox * mailbox);
-
 static void libbalsa_mailbox_pop3_save_config(LibBalsaMailbox * mailbox,
 					      const gchar * prefix);
 static void libbalsa_mailbox_pop3_load_config(LibBalsaMailbox * mailbox,
@@ -65,36 +68,10 @@ static void libbalsa_mailbox_pop3_load_config(LibBalsaMailbox * mailbox,
 
 #define MBOX_POP3_ERROR 	(g_quark_from_static_string("mailbox-pop3"))
 
-
-GType
-libbalsa_mailbox_pop3_get_type(void)
-{
-    static GType mailbox_type = 0;
-
-    if (!mailbox_type) {
-	static const GTypeInfo mailbox_info = {
-	    sizeof(LibBalsaMailboxPop3Class),
-            NULL,               /* base_init */
-            NULL,               /* base_finalize */
-	    (GClassInitFunc) libbalsa_mailbox_pop3_class_init,
-            NULL,               /* class_finalize */
-            NULL,               /* class_data */
-	    sizeof(LibBalsaMailboxPop3),
-            0,                  /* n_preallocs */
-	    (GInstanceInitFunc) libbalsa_mailbox_pop3_init
-	};
-
-	mailbox_type =
-	    g_type_register_static(LIBBALSA_TYPE_MAILBOX_REMOTE,
-                                   "LibBalsaMailboxPOP3",
-			           &mailbox_info, 0);
-    }
-
-    return mailbox_type;
-}
+G_DEFINE_TYPE(LibBalsaMailboxPOP3, libbalsa_mailbox_pop3, LIBBALSA_TYPE_MAILBOX_REMOTE)
 
 static void
-libbalsa_mailbox_pop3_class_init(LibBalsaMailboxPop3Class * klass)
+libbalsa_mailbox_pop3_class_init(LibBalsaMailboxPOP3Class * klass)
 {
     GObjectClass *object_class;
     LibBalsaMailboxClass *libbalsa_mailbox_class;
@@ -102,8 +79,6 @@ libbalsa_mailbox_pop3_class_init(LibBalsaMailboxPop3Class * klass)
     object_class = G_OBJECT_CLASS(klass);
     libbalsa_mailbox_class = LIBBALSA_MAILBOX_CLASS(klass);
 
-    parent_class = g_type_class_peek_parent(klass);
-
     object_class->finalize = libbalsa_mailbox_pop3_finalize;
 
     libbalsa_mailbox_class->open_mailbox = libbalsa_mailbox_pop3_open;
@@ -117,49 +92,48 @@ libbalsa_mailbox_pop3_class_init(LibBalsaMailboxPop3Class * klass)
 }
 
 static void
-libbalsa_mailbox_pop3_init(LibBalsaMailboxPop3 * mailbox)
+libbalsa_mailbox_pop3_init(LibBalsaMailboxPOP3 * mailbox_pop3)
 {
-    LibBalsaMailboxRemote *remote;
-    mailbox->check = FALSE;
-    mailbox->delete_from_server = FALSE;
-    mailbox->inbox = NULL;
-    mailbox->msg_size_limit = -1;
+    LibBalsaMailboxRemote *remote = LIBBALSA_MAILBOX_REMOTE(mailbox_pop3);
+    LibBalsaServer *server;
 
-    mailbox->filter = FALSE;
-    mailbox->filter_cmd = g_strdup("procmail -f -");
-    remote = LIBBALSA_MAILBOX_REMOTE(mailbox);
-    remote->server = libbalsa_server_new();
+    mailbox_pop3->check = FALSE;
+    mailbox_pop3->delete_from_server = FALSE;
+    mailbox_pop3->inbox = NULL;
+    mailbox_pop3->msg_size_limit = -1;
+
+    mailbox_pop3->filter = FALSE;
+    mailbox_pop3->filter_cmd = g_strdup("procmail -f -");
+
+    server = libbalsa_server_new();
+    libbalsa_server_set_protocol(server, "pop3");
+    libbalsa_mailbox_remote_set_server(remote, server);
 }
 
 static void
 libbalsa_mailbox_pop3_finalize(GObject * object)
 {
-    LibBalsaMailboxRemote *remote = LIBBALSA_MAILBOX_REMOTE(object);
-    LibBalsaMailboxPop3 *mailbox = LIBBALSA_MAILBOX_POP3(object);
+    LibBalsaMailboxPOP3 *mailbox_pop3 = LIBBALSA_MAILBOX_POP3(object);
 
-    g_object_unref(G_OBJECT(remote->server));
-    g_free(mailbox->filter_cmd);
+    g_free(mailbox_pop3->filter_cmd);
 
-    if (G_OBJECT_CLASS(parent_class)->finalize)
-	G_OBJECT_CLASS(parent_class)->finalize(object);
+    G_OBJECT_CLASS(libbalsa_mailbox_pop3_parent_class)->finalize(object);
 }
 
-LibBalsaMailboxPop3*
+LibBalsaMailboxPOP3*
 libbalsa_mailbox_pop3_new(void)
 {
-    LibBalsaMailboxPop3 *mailbox;
+    LibBalsaMailboxPOP3 *mailbox_pop3;
 
-    mailbox = g_object_new(LIBBALSA_TYPE_MAILBOX_POP3, NULL);
+    mailbox_pop3 = g_object_new(LIBBALSA_TYPE_MAILBOX_POP3, NULL);
 
-    return mailbox;
+    return mailbox_pop3;
 }
 
 
 static gboolean
 libbalsa_mailbox_pop3_open(LibBalsaMailbox * mailbox, GError **err)
 {
-    g_return_val_if_fail(LIBBALSA_IS_MAILBOX_POP3(mailbox), FALSE);
-
     /* FIXME: it should never be called. */
 
     g_print("Opened a POP3 mailbox!\n");
@@ -350,7 +324,7 @@ pop_handler_close(pop_handler_t *handler,
 			result = FALSE;
 		}
 	} else {
-		g_object_unref(G_OBJECT(handler->mbx_stream));
+		g_object_unref(handler->mbx_stream);
 	}
 	g_free(handler->path);
 	g_free(handler);
@@ -430,16 +404,26 @@ message_cb(const gchar                    *buffer,
 
 		notify_progress(fd);
 		if (fd->filter_path == NULL) {
-			GError *add_err = NULL;
+                    GError *add_err = NULL;
+                    LibBalsaMailbox *mailbox;
+                    LibBalsaMailbox *inbox;
+
+                    mailbox = fd->mailbox;
+                    inbox = LIBBALSA_MAILBOX_POP3(mailbox)->inbox;
 
 		    g_mime_stream_reset(fd->handler->mbx_stream);
-		    result = libbalsa_mailbox_add_message(LIBBALSA_MAILBOX_POP3(fd->mailbox)->inbox, fd->handler->mbx_stream,
-		    	LIBBALSA_MESSAGE_FLAG_NEW | LIBBALSA_MESSAGE_FLAG_RECENT, &add_err);
+		    result =
+                        libbalsa_mailbox_add_message(inbox, fd->handler->mbx_stream,
+                                                     LIBBALSA_MESSAGE_FLAG_NEW |
+                                                     LIBBALSA_MESSAGE_FLAG_RECENT,
+                                                     &add_err);
 		    if (!result) {
 		        libbalsa_information(LIBBALSA_INFORMATION_WARNING, _("Error appending message %d from %s to %s: %s"),
-		        	info->id, fd->mailbox->name, LIBBALSA_MAILBOX_POP3(fd->mailbox)->inbox->name,
-					(add_err != NULL) ? add_err->message : "?");
-		        g_error_free(add_err);
+		        	info->id,
+                                libbalsa_mailbox_get_name(mailbox),
+                                libbalsa_mailbox_get_name(inbox),
+                                add_err != NULL ? add_err->message : "?");
+		        g_clear_error(&add_err);
 		    }
 		}
 
@@ -461,20 +445,25 @@ message_cb(const gchar                    *buffer,
 
 
 static NetClientPop *
-libbalsa_mailbox_pop3_startup(LibBalsaServer            *server,
-							  const LibBalsaMailboxPop3 *mbox,
-							  const gchar               *name,
-							  GList                    **msg_list)
+libbalsa_mailbox_pop3_startup(LibBalsaServer      *server,
+                              LibBalsaMailboxPOP3 *mailbox_pop3,
+                              const gchar         *name,
+                              GList              **msg_list)
 {
+        LibBalsaMailbox *mailbox = (LibBalsaMailbox *) mailbox_pop3;
+        NetClientCryptMode security;
+        const gchar *host;
 	NetClientPop *pop;
 	GError *error = NULL;
 	guint allow_auth;
 
 	/* create the mailbox connection */
-	if (server->security == NET_CLIENT_CRYPT_ENCRYPTED) {
-		pop = net_client_pop_new(server->host, 995U, server->security, mbox->enable_pipe);
+        security = libbalsa_server_get_security(server);
+        host = libbalsa_server_get_host(server);
+	if (security == NET_CLIENT_CRYPT_ENCRYPTED) {
+		pop = net_client_pop_new(host, 995U, security, mailbox_pop3->enable_pipe);
 	} else {
-		pop = net_client_pop_new(server->host, 110U, server->security, mbox->enable_pipe);
+		pop = net_client_pop_new(host, 110U, security, mailbox_pop3->enable_pipe);
 	}
 	if (pop == NULL) {
 		return NULL;
@@ -483,7 +472,7 @@ libbalsa_mailbox_pop3_startup(LibBalsaServer            *server,
 	/* configure the mailbox connection; allow all (including plain text) auth methods even for unencrypted connections so using
 	 * e.g. popfile on localhost is possible, i.e. the user is responsible for choosing a proper security mode */
 	allow_auth = NET_CLIENT_POP_AUTH_ALL;
-	if (mbox->disable_apop) {
+	if (mailbox_pop3->disable_apop) {
 		allow_auth &= ~NET_CLIENT_POP_AUTH_APOP;
 	}
 	net_client_pop_allow_auth(pop, TRUE, allow_auth);
@@ -491,29 +480,31 @@ libbalsa_mailbox_pop3_startup(LibBalsaServer            *server,
 	net_client_set_timeout(NET_CLIENT(pop), 60U);
 
 	/* load client certificate if configured */
-	if (server->client_cert) {
-		g_signal_connect(G_OBJECT(pop), "cert-pass", G_CALLBACK(libbalsa_server_get_cert_pass), server);
-		if (!net_client_set_cert_from_file(NET_CLIENT(pop), server->cert_file, &error)) {
-			libbalsa_information(LIBBALSA_INFORMATION_ERROR, _("Cannot load certificate file %s: %s"), server->cert_file,
+	if (libbalsa_server_get_client_cert(server)) {
+                const gchar *cert_file = libbalsa_server_get_cert_file(server);
+
+		g_signal_connect(pop, "cert-pass", G_CALLBACK(libbalsa_server_get_cert_pass), server);
+		if (!net_client_set_cert_from_file(NET_CLIENT(pop), cert_file, &error)) {
+			libbalsa_information(LIBBALSA_INFORMATION_ERROR, _("Cannot load certificate file %s: %s"), cert_file,
 				error->message);
 			/* bad certificate private key password: clear it */
 			if (error->code == NET_CLIENT_ERROR_CERT_KEY_PASS) {
 				libbalsa_server_set_password(server, NULL, TRUE);
 			}
 			g_error_free(error);
-			g_object_unref(G_OBJECT(pop));
+			g_object_unref(pop);
 			return NULL;
 		}
 	}
 
 	/* connect signals */
-	g_signal_connect(G_OBJECT(pop), "cert-check", G_CALLBACK(libbalsa_server_check_cert), pop);
-	g_signal_connect(G_OBJECT(pop), "auth", G_CALLBACK(libbalsa_server_get_auth), server);
+	g_signal_connect(pop, "cert-check", G_CALLBACK(libbalsa_server_check_cert), pop);
+	g_signal_connect(pop, "auth", G_CALLBACK(libbalsa_server_get_auth), server);
 
 	/* connect server */
-	libbalsa_mailbox_progress_notify(LIBBALSA_MAILBOX(mbox), LIBBALSA_NTFY_INIT, INFINITY, _("Connecting %s…"), server->host);
+	libbalsa_mailbox_progress_notify(mailbox, LIBBALSA_NTFY_INIT, INFINITY, _("Connecting %s…"), host);
 	if (!net_client_pop_connect(pop, NULL, &error)) {
-		libbalsa_information(LIBBALSA_INFORMATION_ERROR, _("POP3 mailbox %s: cannot connect %s: %s"), name, server->host,
+		libbalsa_information(LIBBALSA_INFORMATION_ERROR, _("POP3 mailbox %s: cannot connect %s: %s"), name, host,
 			error->message);
 		/* authentication failed: clear password */
 		if (error->code == NET_CLIENT_ERROR_POP_AUTHFAIL) {
@@ -521,17 +512,17 @@ libbalsa_mailbox_pop3_startup(LibBalsaServer            *server,
 		}
 		g_error_free(error);
 		net_client_shutdown(NET_CLIENT(pop));
-		g_object_unref(G_OBJECT(pop));
+		g_object_unref(pop);
 		return NULL;
 	}
 
 	/* load message list */
-	libbalsa_mailbox_progress_notify(LIBBALSA_MAILBOX(mbox), LIBBALSA_NTFY_UPDATE, INFINITY, _("List messages…"));
-	if (!net_client_pop_list(pop, msg_list, !mbox->delete_from_server, &error)) {
+	libbalsa_mailbox_progress_notify(mailbox, LIBBALSA_NTFY_UPDATE, INFINITY, _("List messages…"));
+	if (!net_client_pop_list(pop, msg_list, !mailbox_pop3->delete_from_server, &error)) {
 		libbalsa_information(LIBBALSA_INFORMATION_ERROR, _("POP3 mailbox %s error: %s"), name, error->message);
 		g_error_free(error);
 		net_client_shutdown(NET_CLIENT(pop));
-		g_object_unref(G_OBJECT(pop));
+		g_object_unref(pop);
 		pop = NULL;
 	}
 
@@ -541,10 +532,10 @@ libbalsa_mailbox_pop3_startup(LibBalsaServer            *server,
 
 static GList *
 update_msg_list(struct fetch_data         *fd,
-				const LibBalsaMailboxPop3 *mbox,
-				GHashTable               **current_uids,
-				LibBalsaServer            *server,
-				GList                     *msg_list)
+                const LibBalsaMailboxPOP3 *mailbox_pop3,
+                GHashTable               **current_uids,
+                LibBalsaServer            *server,
+                GList                     *msg_list)
 {
 	GHashTable *uids = NULL;
 	gchar *uid_prefix = NULL;
@@ -552,8 +543,9 @@ update_msg_list(struct fetch_data         *fd,
 	GList *p;
 
 	/* load uid's if messages shall be left on the server */
-	if (!mbox->delete_from_server) {
-		uid_prefix = g_strconcat(server->user, "@", server->host, NULL);
+	if (!mailbox_pop3->delete_from_server) {
+		uid_prefix = g_strconcat(libbalsa_server_get_user(server), "@",
+                                         libbalsa_server_get_host(server), NULL);
 		prefix_len = strlen(uid_prefix);
 		uids = mp_load_uids(uid_prefix);
 		*current_uids = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, NULL);
@@ -569,7 +561,8 @@ update_msg_list(struct fetch_data         *fd,
 		GList* next = p->next;
 
 		/* check for oversized message */
-		if ((mbox->msg_size_limit > 0) && (msg_info->size >= (gsize) mbox->msg_size_limit)) {
+		if ((mailbox_pop3->msg_size_limit > 0) &&
+                        (msg_info->size >= (gsize) mailbox_pop3->msg_size_limit)) {
 			gchar *size_str;
 
 			size_str = libbalsa_size_to_gchar(msg_info->size);
@@ -580,7 +573,7 @@ update_msg_list(struct fetch_data         *fd,
 		}
 
 		/* check if we already know this message */
-		if (!skip && !mbox->delete_from_server) {
+		if (!skip && !mailbox_pop3->delete_from_server) {
 			gchar *full_uid = g_strconcat(uid_prefix, " ", msg_info->uid, NULL);
 
 			g_hash_table_insert(*current_uids, full_uid, GINT_TO_POINTER(1));
@@ -601,7 +594,7 @@ update_msg_list(struct fetch_data         *fd,
 	}
 
 	/* copy all keys /not/ starting with the prefix from the old to the current hash table, and drop the old table */
-	if (!mbox->delete_from_server && (msg_list != NULL)) {
+	if (!mailbox_pop3->delete_from_server && (msg_list != NULL)) {
 		GHashTableIter iter;
 		gpointer key;
 
@@ -625,19 +618,20 @@ update_msg_list(struct fetch_data         *fd,
 static void
 libbalsa_mailbox_pop3_check(LibBalsaMailbox * mailbox)
 {
-	LibBalsaMailboxPop3 *mbox = LIBBALSA_MAILBOX_POP3(mailbox);
+	LibBalsaMailboxPOP3 *mailbox_pop3 = LIBBALSA_MAILBOX_POP3(mailbox);
 	LibBalsaServer *server;
 	NetClientPop *pop;
 	GList *msg_list;
 
-	if (!mbox->check || (mbox->inbox == NULL)) {
+	if (!mailbox_pop3->check || (mailbox_pop3->inbox == NULL)) {
 		return;
 	}
 
-	server = LIBBALSA_MAILBOX_REMOTE_SERVER(mbox);
+	server = LIBBALSA_MAILBOX_REMOTE_GET_SERVER(mailbox_pop3);
 
 	/* open the mailbox connection and get the messages list (note: initiates the progress dialogue) */
-	pop = libbalsa_mailbox_pop3_startup(server, mbox, mailbox->name, &msg_list);
+	pop = libbalsa_mailbox_pop3_startup(server, mailbox_pop3,
+                                            libbalsa_mailbox_get_name(mailbox), &msg_list);
 
 	/* proceed on success only */
 	if (pop != NULL) {
@@ -653,7 +647,8 @@ libbalsa_mailbox_pop3_check(LibBalsaMailbox * mailbox)
 		/* nothing to do if no messages are on the server */
 		if (msg_list != NULL) {
 			/* load uid's if messages shall be left on the server */
-			msg_list = update_msg_list(&fd, mbox, &current_uids, server, msg_list);
+			msg_list = update_msg_list(&fd, mailbox_pop3, &current_uids,
+                                                   server, msg_list);
 		}
 
 		/* download messages unless the list is empty */
@@ -665,13 +660,13 @@ libbalsa_mailbox_pop3_check(LibBalsaMailbox * mailbox)
 				ngettext("%lu new message (%s)", "%lu new messages (%s)", fd.total_messages),
 				(unsigned long) fd.total_messages, fd.total_size_msg);
 
-			if (mbox->filter) {
-				fd.filter_path = mbox->filter_cmd;
+			if (mailbox_pop3->filter) {
+				fd.filter_path = mailbox_pop3->filter_cmd;
 			}
 
 			if (result) {
 				result = net_client_pop_retr(pop, msg_list, message_cb, &fd, &err);
-				if (result && mbox->delete_from_server) {
+				if (result && mailbox_pop3->delete_from_server) {
 					libbalsa_mailbox_progress_notify(mailbox, LIBBALSA_NTFY_UPDATE, INFINITY,
 						_("Deleting messages on server…"));
 					result = net_client_pop_dele(pop, msg_list, &err);
@@ -684,8 +679,10 @@ libbalsa_mailbox_pop3_check(LibBalsaMailbox * mailbox)
 		}
 
 		/* store uid list */
-		if (result && !mbox->delete_from_server) {
-			gchar *uid_prefix = g_strconcat(server->user, "@", server->host, NULL);
+		if (result && !mailbox_pop3->delete_from_server) {
+			gchar *uid_prefix =
+                            g_strconcat(libbalsa_server_get_user(server), "@",
+                                        libbalsa_server_get_host(server), NULL);
 
 			mp_save_uids(current_uids, uid_prefix, &err);
 			g_free(uid_prefix);
@@ -703,7 +700,7 @@ libbalsa_mailbox_pop3_check(LibBalsaMailbox * mailbox)
 		}
 
 		/* done - clean up */
-		g_object_unref(G_OBJECT(pop));
+		g_object_unref(pop);
 	}
 
 	libbalsa_mailbox_progress_notify(mailbox, LIBBALSA_NTFY_FINISHED, 1.0, _("Finished"));
@@ -714,24 +711,22 @@ static void
 libbalsa_mailbox_pop3_save_config(LibBalsaMailbox * mailbox,
 				  const gchar * prefix)
 {
-    LibBalsaMailboxPop3 *pop;
+    LibBalsaMailboxPOP3 *mailbox_pop3;
 
-    g_return_if_fail(LIBBALSA_IS_MAILBOX_POP3(mailbox));
+    libbalsa_server_save_config(LIBBALSA_MAILBOX_REMOTE_GET_SERVER(mailbox));
 
-    pop = LIBBALSA_MAILBOX_POP3(mailbox);
+    mailbox_pop3 = LIBBALSA_MAILBOX_POP3(mailbox);
 
-    libbalsa_server_save_config(LIBBALSA_MAILBOX_REMOTE_SERVER(mailbox));
+    libbalsa_conf_set_bool("Check", mailbox_pop3->check);
+    libbalsa_conf_set_bool("Delete", mailbox_pop3->delete_from_server);
+    libbalsa_conf_set_bool("DisableApop", mailbox_pop3->disable_apop);
+    libbalsa_conf_set_bool("EnablePipe", mailbox_pop3->enable_pipe);
+    libbalsa_conf_set_bool("Filter", mailbox_pop3->filter);
+    if (mailbox_pop3->filter_cmd != NULL)
+        libbalsa_conf_set_string("FilterCmd", mailbox_pop3->filter_cmd);
 
-    libbalsa_conf_set_bool("Check", pop->check);
-    libbalsa_conf_set_bool("Delete", pop->delete_from_server);
-    libbalsa_conf_set_bool("DisableApop", pop->disable_apop);
-    libbalsa_conf_set_bool("EnablePipe", pop->enable_pipe);
-    libbalsa_conf_set_bool("Filter", pop->filter);
-    if(pop->filter_cmd)
-        libbalsa_conf_set_string("FilterCmd", pop->filter_cmd);
-
-    if (LIBBALSA_MAILBOX_CLASS(parent_class)->save_config)
-	LIBBALSA_MAILBOX_CLASS(parent_class)->save_config(mailbox, prefix);
+    if (LIBBALSA_MAILBOX_CLASS(libbalsa_mailbox_pop3_parent_class)->save_config != NULL)
+	LIBBALSA_MAILBOX_CLASS(libbalsa_mailbox_pop3_parent_class)->save_config(mailbox, prefix);
 
 }
 
@@ -739,45 +734,159 @@ static void
 libbalsa_mailbox_pop3_load_config(LibBalsaMailbox * mailbox,
 				  const gchar * prefix)
 {
-    LibBalsaMailboxPop3 *pop;
+    LibBalsaMailboxPOP3 *mailbox_pop3;
 
-    g_return_if_fail(LIBBALSA_IS_MAILBOX_POP3(mailbox));
+    mailbox_pop3 = LIBBALSA_MAILBOX_POP3(mailbox);
 
-    pop = LIBBALSA_MAILBOX_POP3(mailbox);
+    libbalsa_server_load_config(LIBBALSA_MAILBOX_REMOTE_GET_SERVER(mailbox));
 
-    libbalsa_server_load_config(LIBBALSA_MAILBOX_REMOTE_SERVER(mailbox));
+    mailbox_pop3->check = libbalsa_conf_get_bool("Check=false");
+    mailbox_pop3->delete_from_server = libbalsa_conf_get_bool("Delete=false");
+    mailbox_pop3->disable_apop = libbalsa_conf_get_bool("DisableApop=false");
+    mailbox_pop3->enable_pipe = libbalsa_conf_get_bool("EnablePipe=false");
+    mailbox_pop3->filter = libbalsa_conf_get_bool("Filter=false");
 
-    pop->check = libbalsa_conf_get_bool("Check=false");
-    pop->delete_from_server = libbalsa_conf_get_bool("Delete=false");
-    pop->disable_apop = libbalsa_conf_get_bool("DisableApop=false");
-    pop->enable_pipe = libbalsa_conf_get_bool("EnablePipe=false");
-    pop->filter = libbalsa_conf_get_bool("Filter=false");
-    g_free(pop->filter_cmd);
-    pop->filter_cmd = libbalsa_conf_get_string("FilterCmd");
-    if(pop->filter_cmd && *pop->filter_cmd == '\0') {
-	g_free(pop->filter_cmd); pop->filter_cmd = NULL;
+    g_free(mailbox_pop3->filter_cmd);
+    mailbox_pop3->filter_cmd = libbalsa_conf_get_string("FilterCmd");
+    if (mailbox_pop3->filter_cmd != NULL && mailbox_pop3->filter_cmd[0] == '\0') {
+	g_free(mailbox_pop3->filter_cmd);
+        mailbox_pop3->filter_cmd = NULL;
     }
 
-    if (LIBBALSA_MAILBOX_CLASS(parent_class)->load_config)
-	LIBBALSA_MAILBOX_CLASS(parent_class)->load_config(mailbox, prefix);
+    if (LIBBALSA_MAILBOX_CLASS(libbalsa_mailbox_pop3_parent_class)->load_config != NULL)
+	LIBBALSA_MAILBOX_CLASS(libbalsa_mailbox_pop3_parent_class)->load_config(mailbox, prefix);
 
 }
 void
 libbalsa_mailbox_pop3_set_inbox(LibBalsaMailbox *mailbox,
                                 LibBalsaMailbox *inbox)
 {
-    LibBalsaMailboxPop3 *pop;
+    LibBalsaMailboxPOP3 *mailbox_pop3;
 
     g_return_if_fail(LIBBALSA_IS_MAILBOX_POP3(mailbox));
+    g_return_if_fail(LIBBALSA_IS_MAILBOX(inbox));
 
-    pop = LIBBALSA_MAILBOX_POP3(mailbox);
+    mailbox_pop3 = LIBBALSA_MAILBOX_POP3(mailbox);
 
-	pop->inbox=inbox;
+    mailbox_pop3->inbox=inbox;
+}
+
+/*
+ * Getters
+ */
+
+gboolean
+libbalsa_mailbox_pop3_get_delete_from_server(LibBalsaMailboxPOP3 *mailbox_pop3)
+{
+    g_return_val_if_fail(LIBBALSA_IS_MAILBOX_POP3(mailbox_pop3), FALSE);
+
+    return mailbox_pop3->delete_from_server;
+}
+
+gboolean
+libbalsa_mailbox_pop3_get_check(LibBalsaMailboxPOP3 *mailbox_pop3)
+{
+    g_return_val_if_fail(LIBBALSA_IS_MAILBOX_POP3(mailbox_pop3), FALSE);
+
+    return mailbox_pop3->check;
+}
+
+gboolean
+libbalsa_mailbox_pop3_get_filter(LibBalsaMailboxPOP3 *mailbox_pop3)
+{
+    g_return_val_if_fail(LIBBALSA_IS_MAILBOX_POP3(mailbox_pop3), FALSE);
+
+    return mailbox_pop3->filter;
+}
+
+const gchar *
+libbalsa_mailbox_pop3_get_filter_cmd(LibBalsaMailboxPOP3 *mailbox_pop3)
+{
+    g_return_val_if_fail(LIBBALSA_IS_MAILBOX_POP3(mailbox_pop3), NULL);
+
+    return mailbox_pop3->filter_cmd;
+}
+
+gboolean
+libbalsa_mailbox_pop3_get_disable_apop(LibBalsaMailboxPOP3 *mailbox_pop3)
+{
+    g_return_val_if_fail(LIBBALSA_IS_MAILBOX_POP3(mailbox_pop3), FALSE);
+
+    return mailbox_pop3->disable_apop;
+}
+
+gboolean
+libbalsa_mailbox_pop3_get_enable_pipe(LibBalsaMailboxPOP3 *mailbox_pop3)
+{
+    g_return_val_if_fail(LIBBALSA_IS_MAILBOX_POP3(mailbox_pop3), FALSE);
+
+    return mailbox_pop3->enable_pipe;
+}
+
+/*
+ * Setters
+ */
+
+void
+libbalsa_mailbox_pop3_set_msg_size_limit(LibBalsaMailboxPOP3 *mailbox_pop3,
+                                         gint sz_limit)
+{
+    g_return_if_fail(LIBBALSA_IS_MAILBOX_POP3(mailbox_pop3));
+
+    mailbox_pop3->msg_size_limit = sz_limit;
 }
 
 void
-libbalsa_mailbox_pop3_set_msg_size_limit(LibBalsaMailboxPop3 *pop,
-                                         gint sz_limit)
+libbalsa_mailbox_pop3_set_check(LibBalsaMailboxPOP3 *mailbox_pop3,
+                                gboolean check)
 {
-    pop->msg_size_limit = sz_limit;
+    g_return_if_fail(LIBBALSA_IS_MAILBOX_POP3(mailbox_pop3));
+
+    mailbox_pop3->check = check;
+}
+
+void
+libbalsa_mailbox_pop3_set_disable_apop(LibBalsaMailboxPOP3 *mailbox_pop3,
+                                       gboolean disable_apop)
+{
+    g_return_if_fail(LIBBALSA_IS_MAILBOX_POP3(mailbox_pop3));
+
+    mailbox_pop3->disable_apop = disable_apop;
+}
+
+void
+libbalsa_mailbox_pop3_set_delete_from_server(LibBalsaMailboxPOP3 *mailbox_pop3,
+                                             gboolean delete_from_server)
+{
+    g_return_if_fail(LIBBALSA_IS_MAILBOX_POP3(mailbox_pop3));
+
+    mailbox_pop3->delete_from_server = delete_from_server;
+}
+
+void
+libbalsa_mailbox_pop3_set_filter(LibBalsaMailboxPOP3 *mailbox_pop3,
+                                 gboolean filter)
+{
+    g_return_if_fail(LIBBALSA_IS_MAILBOX_POP3(mailbox_pop3));
+
+    mailbox_pop3->filter = filter;
+}
+
+void
+libbalsa_mailbox_pop3_set_filter_cmd(LibBalsaMailboxPOP3 *mailbox_pop3,
+                                     const gchar * filter_cmd)
+{
+    g_return_if_fail(LIBBALSA_IS_MAILBOX_POP3(mailbox_pop3));
+
+    g_free(mailbox_pop3->filter_cmd);
+    mailbox_pop3->filter_cmd = g_strdup(filter_cmd);
+}
+
+void
+libbalsa_mailbox_pop3_set_enable_pipe(LibBalsaMailboxPOP3 *mailbox_pop3,
+                                      gboolean enable_pipe)
+{
+    g_return_if_fail(LIBBALSA_IS_MAILBOX_POP3(mailbox_pop3));
+
+    mailbox_pop3->enable_pipe = enable_pipe;
 }
