@@ -32,10 +32,8 @@ typedef LibBalsaMailbox* (*LibBalsaFccboxFinder)(const gchar *url);
 typedef enum _LibBalsaMsgCreateResult LibBalsaMsgCreateResult;
 enum _LibBalsaMsgCreateResult {
     LIBBALSA_MESSAGE_CREATE_OK,
-#ifdef HAVE_GPGME
     LIBBALSA_MESSAGE_SIGN_ERROR,
     LIBBALSA_MESSAGE_ENCRYPT_ERROR,
-#endif
     LIBBALSA_MESSAGE_CREATE_ERROR,
     LIBBALSA_MESSAGE_QUEUE_ERROR,
     LIBBALSA_MESSAGE_SAVE_ERROR,
@@ -50,9 +48,6 @@ gboolean libbalsa_message_postpone(LibBalsaMessage * message,
 				   gboolean flow, 
 				   GError **error);
 
-
-#if ENABLE_ESMTP
-
 LibBalsaMsgCreateResult libbalsa_message_queue(LibBalsaMessage* message, 
 					       LibBalsaMailbox* outbox,
                                                LibBalsaMailbox* fccbox,
@@ -60,68 +55,23 @@ LibBalsaMsgCreateResult libbalsa_message_queue(LibBalsaMessage* message,
                                                smtp_server,
 					       gboolean flow,
 					       GError ** error);
-LibBalsaMsgCreateResult libbalsa_message_send(LibBalsaMessage * message,
-                                              LibBalsaMailbox * outbox,
-                                              LibBalsaMailbox * fccbox,
+LibBalsaMsgCreateResult libbalsa_message_send(LibBalsaMessage     *message,
+                                              LibBalsaMailbox     *outbox,
+                                              LibBalsaMailbox     *fccbox,
                                               LibBalsaFccboxFinder finder,
-                                              LibBalsaSmtpServer *
-                                              smtp_server,
-                                              GtkWindow * parent,
-                                              gboolean flow,
-                                              gboolean debug,
-					      GError ** error);
-gboolean libbalsa_process_queue(LibBalsaMailbox * outbox,
-                                LibBalsaFccboxFinder finder,
-                                GSList * smtp_servers,
-                                GtkWindow * parent,
-                                gboolean debug);
-#else
+                                              LibBalsaSmtpServer  *smtp_server,
+											  gboolean			   show_progress,
+                                              GtkWindow           *parent,
+                                              gboolean             flow,
+					                          GError             **error);
+void libbalsa_process_queue(LibBalsaMailbox     *outbox,
+                            LibBalsaFccboxFinder finder,
+							GSList              *smtp_servers,
+							gboolean			 show_progress,
+							GtkWindow           *parent);
 
-LibBalsaMsgCreateResult libbalsa_message_queue(LibBalsaMessage* message, 
-					       LibBalsaMailbox* outbox,
-                                               LibBalsaMailbox* fccbox,
-					       gboolean flow,
-					       GError ** error);
-LibBalsaMsgCreateResult libbalsa_message_send(LibBalsaMessage* message,
-					      LibBalsaMailbox* outbox,  
-					      LibBalsaMailbox* fccbox,
-                                              LibBalsaFccboxFinder finder, 
-					      gboolean flow, gboolean debug,
-					      GError ** error);
-gboolean libbalsa_process_queue(LibBalsaMailbox* outbox, 
-                                LibBalsaFccboxFinder finder,
-                                gboolean debug);
-
-#endif
-
-extern GMutex send_messages_lock;
-extern int send_thread_pipes[2];
-
-typedef struct {
-    int message_type;
-    char message_string[256];
-    LibBalsaMessage *msg;
-    LibBalsaMailbox *mbox;
-    float of_total;
-} SendThreadMessage;
-
-#define  MSGSENDTHREAD(t_message, type, string, s_msg, s_mbox, messof) \
-  t_message = g_new(SendThreadMessage, 1); \
-  t_message->message_type = type; \
-  strncpy(t_message->message_string, string, sizeof(t_message->message_string)); \
-  t_message->msg = s_msg; \
-  t_message->mbox = s_mbox; \
-  t_message->of_total = messof; \
-  if (write( send_thread_pipes[1], (void *) &t_message, sizeof(void *) ) \
-      < (ssize_t) sizeof(void *)) \
-    g_warning("pipe error");
-
-enum {
-    MSGSENDTHREADERROR,
-    MSGSENDTHREADPROGRESS,
-    MSGSENDTHREADPOSTPONE,
-    MSGSENDTHREADDELETE,
-    MSGSENDTHREADFINISHED
-};
+void libbalsa_auto_send_init(GSourceFunc auto_send_cb);
+void libbalsa_auto_send_config(gboolean enable,
+					           guint    timeout_minutes);
 
 #endif /* __SEND_H__ */
