@@ -128,11 +128,11 @@ unconditional_mailbox(const gchar * path, const gchar * prettyname,
     }
 
     if (is_remote) {
-        libbalsa_server_set_host(LIBBALSA_MAILBOX_REMOTE_SERVER(*box),
+        libbalsa_server_set_host(LIBBALSA_MAILBOX_REMOTE_GET_SERVER(*box),
                                  url.host,
                                  ssl ? NET_CLIENT_CRYPT_ENCRYPTED :
                                        NET_CLIENT_CRYPT_NONE);
-        libbalsa_server_set_username(LIBBALSA_MAILBOX_REMOTE_SERVER(*box),
+        libbalsa_server_set_username(LIBBALSA_MAILBOX_REMOTE_GET_SERVER(*box),
                                      getenv("USER"));
     }
     g_free(dup);
@@ -141,14 +141,18 @@ unconditional_mailbox(const gchar * path, const gchar * prettyname,
     if (*box == NULL) {
         if (strcmp("/var/spool/mail/", path)) {
             char tmp[32] = "/tmp/balsa.XXXXXX";
+
 	    /* Don't fail if you can't create the spool mailbox. */
 	    close(mkstemp(tmp));
-		*box = libbalsa_mailbox_local_new(tmp, FALSE);
-		if (*box) {
-			free((*box)->url);
-			(*box)->url = g_strdup_printf("file://%s",path);
-		}
-		unlink(tmp);
+            *box = libbalsa_mailbox_local_new(tmp, FALSE);
+            if (*box != NULL) {
+                gchar *mailbox_url;
+
+                mailbox_url = g_strdup_printf("file://%s", path);
+                libbalsa_mailbox_set_url(*box, mailbox_url);
+                g_free(mailbox_url);
+            }
+            unlink(tmp);
 	}
     }
     if ( *box == NULL) {
@@ -159,11 +163,11 @@ unconditional_mailbox(const gchar * path, const gchar * prettyname,
         return;
     }
 
-    (*box)->name = g_strdup(gettext(prettyname));
+    libbalsa_mailbox_set_name(*box, gettext(prettyname));
 
     config_mailbox_add(*box, (char *) prettyname);
     if (box == &balsa_app.outbox)
-        (*box)->no_reassemble = TRUE;
+        libbalsa_mailbox_set_no_reassemble(*box, TRUE);
 }
 
 static void
@@ -303,7 +307,7 @@ balsa_druid_page_directory_init(BalsaDruidPageDirectory * dir,
 
     gtk_box_pack_end(GTK_BOX(page), verify_button(dir), FALSE, FALSE, 0);
 
-    g_signal_connect(G_OBJECT(druid), "prepare",
+    g_signal_connect(druid, "prepare",
                      G_CALLBACK(balsa_druid_page_directory_prepare),
                      dir);
     dir->my_num = 98765;
