@@ -166,7 +166,7 @@ balsa_druid_page_user(GtkAssistant * druid)
     gtk_assistant_set_page_title(druid, user->page, _("User Settings"));
     balsa_druid_page_user_init(user, user->page, druid);
 
-    g_signal_connect(G_OBJECT(druid), "prepare",
+    g_signal_connect(druid, "prepare",
                      G_CALLBACK(balsa_druid_page_user_prepare),
                      user);
     g_object_weak_ref(G_OBJECT(druid), (GWeakNotify)g_free, user);
@@ -197,23 +197,27 @@ create_pop3_mbx(const gchar *name, const gchar* host, gint security,
                 const gchar *login, const gchar *passwd,
                 gboolean remember)
 {
-    LibBalsaMailboxPop3 *pop = libbalsa_mailbox_pop3_new();
-    LibBalsaMailbox *mbx   = LIBBALSA_MAILBOX(pop);
-    LibBalsaServer *server = LIBBALSA_MAILBOX_REMOTE_SERVER(pop);
+    LibBalsaMailboxPOP3 *mailbox_pop3 = libbalsa_mailbox_pop3_new();
+    LibBalsaMailbox *mailbox          = LIBBALSA_MAILBOX(mailbox_pop3);
+    LibBalsaServer *server            = LIBBALSA_MAILBOX_REMOTE_GET_SERVER(mailbox);
+    gchar *mailbox_name;
 
     libbalsa_server_set_username(server, login);
     libbalsa_server_set_password(server, passwd, FALSE);
     libbalsa_server_set_host(server, host, security);
-    server->security        = security;
-    server->remember_passwd = remember;
-    mbx->name               = g_strdup(name && *name ? name : host);
-    pop->check              = TRUE;
-    pop->disable_apop       = FALSE;
-    pop->delete_from_server = TRUE;
-    pop->filter             = FALSE;
-    pop->filter_cmd         = g_strdup("procmail -f -");
-    
-    return mbx;
+    libbalsa_server_set_remember_password(server, remember);
+
+    mailbox_name = g_strdup(name != NULL && name[0] != '\0' ? name : host);
+    libbalsa_mailbox_set_name(mailbox, mailbox_name);
+    g_free(mailbox_name);
+
+    libbalsa_mailbox_pop3_set_check(mailbox_pop3, TRUE);
+    libbalsa_mailbox_pop3_set_disable_apop(mailbox_pop3, FALSE);
+    libbalsa_mailbox_pop3_set_delete_from_server(mailbox_pop3, TRUE);
+    libbalsa_mailbox_pop3_set_filter(mailbox_pop3, FALSE);
+    libbalsa_mailbox_pop3_set_filter_cmd(mailbox_pop3, "procmail -f -");
+
+    return mailbox;
 }
 
 static void
@@ -227,9 +231,9 @@ create_imap_mbx(const gchar *name, const gchar* host, NetClientCryptMode securit
     libbalsa_server_set_username(server, login);
     libbalsa_server_set_password(server, passwd, FALSE);
     libbalsa_server_set_host(server, host, security);
-    server->remember_passwd = remember;
+    libbalsa_server_set_remember_password(server, remember);
     mbnode = balsa_mailbox_node_new_imap_folder(server, NULL);
-    mbnode->name = g_strdup(name && *name ? name : host);
+    balsa_mailbox_node_set_name(mbnode, name != NULL && name[0] != '\0' ? name : host);
 
     config_folder_add(mbnode, NULL);
     /* memory leak? */
