@@ -23,18 +23,12 @@ G_BEGIN_DECLS
 
 
 #define NET_CLIENT_SMTP_TYPE				(net_client_smtp_get_type())
-#define NET_CLIENT_SMTP(obj)				(G_TYPE_CHECK_INSTANCE_CAST((obj), NET_CLIENT_SMTP_TYPE, NetClientSmtp))
-#define NET_IS_CLIENT_SMTP(obj)				(G_TYPE_CHECK_INSTANCE_TYPE((obj), NET_CLIENT_SMTP_TYPE))
-#define NET_CLIENT_SMTP_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST((klass), NET_CLIENT_SMTP_TYPE, NetClientSmtpClass))
-#define NET_IS_CLIENT_SMTP_CLASS(klass)		(G_TYPE_CHECK_CLASS_TYPE((klass), NET_CLIENT_SMTP_TYPE))
-#define NET_CLIENT_SMTP_GET_CLASS(obj)		(G_TYPE_INSTANCE_GET_CLASS((obj), NET_CLIENT_SMTP_TYPE, NetClientSmtpClass))
+G_DECLARE_FINAL_TYPE(NetClientSmtp, net_client_smtp, NET, CLIENT_SMTP, NetClient)
+
 
 #define NET_CLIENT_SMTP_ERROR_QUARK			(g_quark_from_static_string("net-client-smtp"))
 
 
-typedef struct _NetClientSmtp NetClientSmtp;
-typedef struct _NetClientSmtpClass NetClientSmtpClass;
-typedef struct _NetClientSmtpPrivate NetClientSmtpPrivate;
 typedef struct _NetClientSmtpMessage NetClientSmtpMessage;
 typedef enum _NetClientSmtpDsnMode NetClientSmtpDsnMode;
 
@@ -75,17 +69,6 @@ enum _NetClientSmtpError {
 /** @} */
 
 
-struct _NetClientSmtp {
-    NetClient parent;
-    NetClientSmtpPrivate *priv;
-};
-
-
-struct _NetClientSmtpClass {
-	NetClientClass parent;
-};
-
-
 /** @brief Delivery Status Notification mode
  *
  * See <a href="https://tools.ietf.org/html/rfc3461">RFC 3461</a> for a description of Delivery Status Notifications (DSNs).  The
@@ -116,10 +99,6 @@ enum _NetClientSmtpDsnMode {
 typedef gssize (*NetClientSmtpSendCb)(gchar *buffer, gsize count, gpointer user_data, GError **error);
 
 
-GType net_client_smtp_get_type(void)
-	G_GNUC_CONST;
-
-
 /** @brief Create a new SMTP network client
  *
  * @param host host name or IP address to connect
@@ -182,12 +161,13 @@ gboolean net_client_smtp_can_dsn(NetClientSmtp *client);
  *
  * @param client connected SMTP network client object
  * @param message message data
+ * @param server_stat filled with the final server response on success, may be NULL
  * @param error filled with error information if the connection fails
  * @return TRUE on success or FALSE if sending the message failed
  *
  * Send the passed SMTP message to the connected SMTP server.
  */
-gboolean net_client_smtp_send_msg(NetClientSmtp *client, const NetClientSmtpMessage *message, GError **error);
+gboolean net_client_smtp_send_msg(NetClientSmtp *client, const NetClientSmtpMessage *message, gchar **server_stat, GError **error);
 
 
 /** @brief Create a SMTP message
