@@ -68,12 +68,14 @@ struct ABMainWindow {
     LibBalsaAddress *displayed_address;
 
     GMenu *file_menu;
+    GMenu *books_menu;
 } contacts_app;
 
 
 static void
 bab_cleanup(void)
 {
+    g_object_unref(contacts_app.books_menu);
     gtk_main_quit();
 }
 
@@ -122,35 +124,40 @@ bab_load_cb(LibBalsaAddressBook *libbalsa_ab,
             LibBalsaAddress *address, GtkTreeModel *model)
 {
     GtkTreeIter iter;
+    guint n_addrs;
 
     g_return_if_fail(LIBBALSA_IS_ADDRESS_BOOK(libbalsa_ab));
 
     if (address == NULL)
 	return;
 
-    if (libbalsa_address_is_dist_list(libbalsa_ab, address)) {
+    n_addrs = libbalsa_address_get_n_addrs(address);
+    if (libbalsa_address_book_get_dist_list_mode(libbalsa_ab)
+        && n_addrs > 1) {
         gchar *address_string = libbalsa_address_to_gchar(address, -1);
 
         gtk_list_store_prepend(GTK_LIST_STORE(model), &iter);
         /* GtkListStore refs address, and unrefs it when cleared  */
         gtk_list_store_set(GTK_LIST_STORE(model), &iter,
-                           LIST_COLUMN_NAME, address->full_name,
+                           LIST_COLUMN_NAME, libbalsa_address_get_full_name(address),
                            LIST_COLUMN_ADDRSPEC, address_string,
                            LIST_COLUMN_ADDRESS, address,
                            -1);
 
 	g_free(address_string);
     } else {
-        GList *l;
+        guint n;
+
+	for (n = 0; n < n_addrs; ++n) {
+            const gchar *addr = libbalsa_address_get_nth_addr(address, n);
 
-	for (l = address->address_list; l; l = l->next) {
             gtk_list_store_prepend(GTK_LIST_STORE(model), &iter);
             /* GtkListStore refs address once for each address in
              * the list, and unrefs it the same number of times when
              * cleared */
             gtk_list_store_set(GTK_LIST_STORE(model), &iter,
-                               LIST_COLUMN_NAME, address->full_name,
-                               LIST_COLUMN_ADDRSPEC, l->data,
+                               LIST_COLUMN_NAME, libbalsa_address_get_full_name(address),
+                               LIST_COLUMN_ADDRSPEC, addr,
                                LIST_COLUMN_ADDRESS, address,
                                -1);
 	}
@@ -251,63 +258,47 @@ address_changed_cb(struct ABMainWindow *aw)
 static void
 set_address_book_menu_items(void)
 {
-    GString *string;
     GList *l;
     guint pos;
-    gchar *s;
-    GtkBuilder *builder;
-    GMenuModel *menu_model;
+    GMenu *menu = contacts_app.books_menu;
 
-    pos = g_menu_model_get_n_items(G_MENU_MODEL (contacts_app.file_menu));
-    g_menu_remove(contacts_app.file_menu, --pos);
+    if (menu == NULL) {
+        contacts_app.books_menu = menu = g_menu_new();
+        g_menu_append_section(contacts_app.file_menu, NULL, G_MENU_MODEL(menu));
+    } else {
+        g_menu_remove_all(menu);
+    }
 
     pos = 0;
-    string = g_string_new(NULL);
-    g_string_append(string, "<interface>");
-    g_string_append(string, "<menu id='address-book-menu'>");
-    g_string_append(string, "<section>");
-    for (l = contacts_app.address_book_list; l; l = l->next) {
+    for (l = contacts_app.address_book_list; l != NULL; l = l->next) {
         LibBalsaAddressBook *address_book = l->data;
+        const gchar *name;
+        gchar *label;
+        gchar *detailed_action;
+        gchar *accel;
+        GMenuItem *item;
 
-        if (!address_book)
+        if (address_book == NULL)
             continue;
 
-        g_string_append(string, "<item>");
+        name = libbalsa_address_book_get_name(address_book);
 
-        g_string_append(string, "<attribute name='label'>");
-        g_string_append_printf(string, "_%d:%s", ++pos,
-                               libbalsa_address_book_get_name(address_book));
-        g_string_append(string, "</attribute>");
+        label = g_strdup_printf("_%d:%s", ++pos, name);
+        detailed_action = g_strdup_printf("win.address-book::%s", name);
+        item = g_menu_item_new(label, detailed_action);
+        g_free(detailed_action);
+        g_free(label);
 
-        g_string_append(string, "<attribute name='action'>");
-        g_string_append(string, "win.address-book");
-        g_string_append(string, "</attribute>");
+        accel = g_strdup_printf("<Primary>%d", pos);
+        g_menu_item_set_attribute(item, "accel", "s", accel);
+        g_free(accel);
 
-        g_string_append(string, "<attribute name='target'>");
-        g_string_append(string, libbalsa_address_book_get_name(address_book));
-        g_string_append(string, "</attribute>");
-
-        g_string_append(string, "<attribute name='accel'>");
-        g_string_append_printf(string, "&lt;Primary&gt;%d", pos);
-        g_string_append(string, "</attribute>");
-
-        g_string_append(string, "</item>");
+        g_menu_append_item(menu, item);
+        g_object_unref(item);
     }
-    g_string_append(string, "</section>");
-    g_string_append(string, "</menu>");
-    g_string_append(string, "</interface>");
-    s = g_string_free(string, FALSE);
 
-    builder = gtk_builder_new_from_string(s, -1);
-    g_free(s);
-
-    menu_model =
-        G_MENU_MODEL(gtk_builder_get_object(builder, "address-book-menu"));
-    g_menu_append_section(contacts_app.file_menu, NULL, menu_model);
     libbalsa_window_set_accels(GTK_APPLICATION_WINDOW(contacts_app.window),
-                               menu_model);
-
-    g_object_unref(builder);
+                               G_MENU_MODEL(menu));
 }
 
 static gboolean
@@ -447,8 +438,8 @@ file_delete_activated(GSimpleAction * action,
     const gchar *config_prefix;
     GList *list;
 
-    if (!(address_book = contacts_app.address_book)
-        || !g_list_next(contacts_app.address_book_list))
+    if ((address_book = contacts_app.address_book) == NULL
+        || contacts_app.address_book_list->next == NULL)
         return;
 
     config_prefix = libbalsa_address_book_get_config_prefix(address_book);
@@ -735,9 +726,9 @@ bab_window_list_new(void)
     g_object_unref(store);
     selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree));
     gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
-    g_signal_connect(G_OBJECT(selection), "changed",
+    g_signal_connect(selection, "changed",
                      G_CALLBACK(list_selection_changed_cb), NULL);
-    g_signal_connect(G_OBJECT(tree), "row-activated",
+    g_signal_connect(tree, "row-activated",
                      G_CALLBACK(list_row_activated_cb), NULL);
 
     renderer = gtk_cell_renderer_text_new();
@@ -753,7 +744,7 @@ bab_window_list_new(void)
                         GDK_BUTTON1_MASK,
                         libbalsa_address_target_list, 2,
                         GDK_ACTION_COPY);
-    g_signal_connect(G_OBJECT(tree), "drag-data-get",
+    g_signal_connect(tree, "drag-data-get",
                      G_CALLBACK(addrlist_drag_get_cb), NULL);
 
     renderer = gtk_cell_renderer_text_new();
@@ -847,17 +838,17 @@ bab_get_edit_button_box(struct ABMainWindow *abmw)
     gtk_container_add(GTK_CONTAINER(box),
                       abmw->apply_button =
                       gtk_button_new_with_mnemonic(_("_Apply")));
-    g_signal_connect(G_OBJECT(abmw->apply_button), "clicked",
+    g_signal_connect(abmw->apply_button, "clicked",
                      G_CALLBACK(apply_button_cb), (gpointer) NULL);
     gtk_container_add(GTK_CONTAINER(box),
                       abmw->remove_button =
                       gtk_button_new_with_mnemonic(_("_Remove")));
-    g_signal_connect(G_OBJECT(abmw->remove_button), "clicked",
+    g_signal_connect(abmw->remove_button, "clicked",
                      G_CALLBACK(remove_button_cb), (gpointer) NULL);
     gtk_container_add(GTK_CONTAINER(box),
                       abmw->cancel_button =
                       gtk_button_new_with_mnemonic(_("_Cancel")));
-    g_signal_connect(G_OBJECT(abmw->cancel_button), "clicked",
+    g_signal_connect(abmw->cancel_button, "clicked",
                      G_CALLBACK(cancel_button_cb), abmw);
     return box;
 }
@@ -898,13 +889,13 @@ bab_get_filter_box(void)
                                                    GTK_ICON_SIZE_BUTTON));
     gtk_box_pack_start(GTK_BOX(search_hbox), button, FALSE, FALSE, 1);
 
-    g_signal_connect(G_OBJECT(find_entry), "activate",
+    g_signal_connect(find_entry, "activate",
                      G_CALLBACK(bab_filter_entry_activate),
                      button);
-    g_signal_connect_swapped(G_OBJECT(button), "clicked",
+    g_signal_connect_swapped(button, "clicked",
                              G_CALLBACK(bab_filter_entry_activate),
                              find_entry);
-    g_signal_connect(G_OBJECT(find_entry), "changed",
+    g_signal_connect(find_entry, "changed",
                              G_CALLBACK(bab_filter_entry_changed),
                              button);
     return search_hbox;
@@ -974,7 +965,7 @@ bab_window_new(GtkApplication * application)
 			     gtk_label_new(_("Edit")));
 
     /*
-    g_signal_connect(G_OBJECT(find_entry), "changed",
+    g_signal_connect(find_entry, "changed",
 		     G_CALLBACK(balsa_ab_window_find), ab);
     */
     g_signal_connect(wnd, "key-press-event",
@@ -1135,9 +1126,9 @@ main(int argc, char *argv[])
 
     ab_window = bab_window_new(application);
 
-    g_signal_connect(G_OBJECT(ab_window), "destroy",
+    g_signal_connect(ab_window, "destroy",
                      G_CALLBACK(bab_cleanup), NULL);
-    g_signal_connect(G_OBJECT(ab_window), "delete-event",
+    g_signal_connect(ab_window, "delete-event",
                      G_CALLBACK(bab_delete_ok), NULL);
     bab_set_intial_address_book(ab, ab_window);
 
