@@ -1,6 +1,6 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2016 Stuart Parmenter and others,
+ * Copyright (C) 1997-2019 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -38,10 +38,6 @@ enum {
     N_COLUMNS
 };
 
-/* Object system functions ... */
-static void balsa_ab_window_init(BalsaAbWindow *ab);
-static void balsa_ab_window_class_init(BalsaAbWindowClass *klass);
-
 /* Loading ... */
 static void balsa_ab_window_load_cb(LibBalsaAddressBook *libbalsa_ab,
 				       LibBalsaAddress *address,
@@ -81,35 +77,40 @@ static gint balsa_ab_window_compare_entries(GtkTreeModel * model,
                                                GtkTreeIter * iter2,
                                                gpointer data);
 
-struct _BalsaAbWindowClass
+struct _BalsaAbWindow
 {
-    GtkDialogClass parent_class;
+    GtkDialog parent;
+
+    /* Are we composing? */
+    gboolean composing;
+
+    /* The current address book */
+    LibBalsaAddressBook *current_address_book;
+
+    /* the filter entry */
+    GtkWidget *filter_entry;
+
+    /* The address list */
+    GtkWidget *address_list;
+
+    /* The send to list */
+    GtkWidget *recipient_list;
+
+    /* Radio buttons for dist list mode */
+    GtkWidget *single_address_mode_radio;
+    GtkWidget *dist_address_mode_radio;
+    guint      toggle_handler_id;
+
+    /* Stuff to hide when not in compose mode */
+    GtkWidget *send_to_label;
+    GtkWidget *send_to_list;
+    GtkWidget *arrow_box;
+
+    /* The address book list */
+    GtkWidget *combo_box;
 };
 
-GType
-balsa_ab_window_get_type(void)
-{
-    static GType ab_type = 0;
-
-    if ( !ab_type ) {
-	static const GTypeInfo ab_info = {
-	    sizeof(BalsaAbWindowClass),
-            NULL,               /* base_init */
-            NULL,               /* base_finalize */
-	    (GClassInitFunc) balsa_ab_window_class_init,
-            NULL,               /* class_finalize */
-            NULL,               /* class_data */
-	    sizeof(BalsaAbWindow),
-            0,                  /* n_preallocs */
-	    (GInstanceInitFunc) balsa_ab_window_init
-	};
-        ab_type =
-            g_type_register_static(GTK_TYPE_DIALOG, "BalsaAbWindow",
-                                   &ab_info, 0);
-    }
-    return ab_type;
-}
-
+G_DEFINE_TYPE(BalsaAbWindow, balsa_ab_window, GTK_TYPE_DIALOG)
 
 GtkWidget *
 balsa_ab_window_new(gboolean composing, GtkWindow* parent)
@@ -117,7 +118,7 @@ balsa_ab_window_new(gboolean composing, GtkWindow* parent)
     GtkWidget *ret;
     BalsaAbWindow *ab;
     gint use_headerbar = 1;
-	const gchar *dialog_env;
+    const gchar *dialog_env;
 
     /* header bar configuration */
 	dialog_env = g_getenv("BALSA_DIALOG_HEADERBAR");
@@ -199,7 +200,7 @@ balsa_ab_window_list(BalsaAbWindow * ab, GCallback row_activated_cb)
     gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
 
     gtk_widget_show(tree);
-    g_signal_connect(G_OBJECT(tree), "row-activated", row_activated_cb,
+    g_signal_connect(tree, "row-activated", row_activated_cb,
                      ab);
     return tree;
 }
@@ -241,7 +242,7 @@ balsa_ab_window_init(BalsaAbWindow *ab)
     g_return_if_fail(balsa_app.address_book_list);
     gtk_window_set_title(GTK_WINDOW(ab), _("Address Book"));
 
-    g_signal_connect(G_OBJECT(ab), "response",
+    g_signal_connect(ab, "response",
 		     G_CALLBACK(balsa_ab_window_response_cb), NULL);
 
     vbox = gtk_dialog_get_content_area(GTK_DIALOG(ab));
@@ -295,7 +296,7 @@ balsa_ab_window_init(BalsaAbWindow *ab)
     ab->filter_entry = gtk_entry_new();
     gtk_widget_show(ab->filter_entry);
     gtk_label_set_mnemonic_widget(GTK_LABEL(find_label), ab->filter_entry);
-    g_signal_connect(G_OBJECT(ab->filter_entry), "changed",
+    g_signal_connect(ab->filter_entry, "changed",
 		     G_CALLBACK(balsa_ab_window_find), ab);
 
     /* Pack the find stuff into the grid */
@@ -331,7 +332,7 @@ balsa_ab_window_init(BalsaAbWindow *ab)
     gtk_widget_show(GTK_WIDGET(w));
 
     w = gtk_button_new_with_mnemonic(_("_Re-import"));
-    g_signal_connect(G_OBJECT(w), "clicked",
+    g_signal_connect(w, "clicked",
                      G_CALLBACK(balsa_ab_window_reload),
 		       ab);
     gtk_container_add(GTK_CONTAINER(hbox), w);
@@ -351,7 +352,7 @@ balsa_ab_window_init(BalsaAbWindow *ab)
                                       GTK_ICON_SIZE_BUTTON);
     gtk_box_pack_start(GTK_BOX(ab->arrow_box), w, TRUE, FALSE, 0);
     gtk_widget_show(w);
-    g_signal_connect(G_OBJECT(w), "clicked",
+    g_signal_connect(w, "clicked",
 		     G_CALLBACK(balsa_ab_window_move_to_recipient_list),
 		       ab);
 
@@ -359,7 +360,7 @@ balsa_ab_window_init(BalsaAbWindow *ab)
                                       GTK_ICON_SIZE_BUTTON);
     gtk_box_pack_start(GTK_BOX(ab->arrow_box), w, TRUE, FALSE, 0);
     gtk_widget_show(w);
-    g_signal_connect(G_OBJECT(w), "clicked",
+    g_signal_connect(w, "clicked",
 		     G_CALLBACK(balsa_ab_window_remove_from_recipient_list),
 		       ab);
 
@@ -393,7 +394,7 @@ balsa_ab_window_init(BalsaAbWindow *ab)
 	 _("a distribution list"));
     gtk_widget_show(ab->dist_address_mode_radio);
     ab->toggle_handler_id =
-        g_signal_connect(G_OBJECT(ab->single_address_mode_radio),
+        g_signal_connect(ab->single_address_mode_radio,
                          "toggled",
                          G_CALLBACK(balsa_ab_window_dist_mode_toggled),
                          ab);
@@ -463,7 +464,7 @@ balsa_ab_window_get_recipients(BalsaAbWindow * ab)
                            LIST_COLUMN_ADDRESS, &address,
                            LIST_COLUMN_WHICH, &which_multiple, -1);
         text = libbalsa_address_to_gchar(address, which_multiple);
-	g_object_unref(G_OBJECT(address));
+	g_object_unref(address);
         if (text) {
             if (str->len > 0)
                 g_string_append(str, ", ");
@@ -532,7 +533,7 @@ balsa_ab_window_swap_real(GtkTreeModel * model, GtkTreePath * path,
      * already reffed it, so it won't be finalized, even if there
      * were no other outstanding refs. */
     gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
-    g_object_unref(G_OBJECT(address));
+    g_object_unref(address);
     g_free(name);
     g_free(address_string);
 }
@@ -610,7 +611,7 @@ balsa_ab_window_activate_address(GtkTreeView * view,
                            LIST_COLUMN_ADDRESS, &address,
                            LIST_COLUMN_WHICH, &which_multiple, -1);
         addr = libbalsa_address_to_gchar(address, which_multiple);
-	g_object_unref(G_OBJECT(address));
+	g_object_unref(address);
         snd = sendmsg_window_compose_with_address(addr);
         g_free(addr);
 
@@ -751,23 +752,27 @@ balsa_ab_window_load_cb(LibBalsaAddressBook *libbalsa_ab,
 {
     GtkTreeModel *model;
     GtkTreeIter iter;
-    GList *address_list;
-    gint count;
+    guint n_addrs;
+    const gchar *full_name;
 
-    g_return_if_fail ( BALSA_IS_AB_WINDOW(ab));
-    g_return_if_fail ( LIBBALSA_IS_ADDRESS_BOOK(libbalsa_ab) );
+    g_return_if_fail(BALSA_IS_AB_WINDOW(ab));
+    g_return_if_fail(LIBBALSA_IS_ADDRESS_BOOK(libbalsa_ab));
 
-    if ( address == NULL )
+    if (address == NULL)
 	return;
 
+    full_name = libbalsa_address_get_full_name(address);
     model = gtk_tree_view_get_model(GTK_TREE_VIEW(ab->address_list));
-    if ( libbalsa_address_is_dist_list(libbalsa_ab, address) ) {
+
+    n_addrs = libbalsa_address_get_n_addrs(address);
+    if (libbalsa_address_book_get_dist_list_mode(libbalsa_ab)
+        && n_addrs > 1) {
         gchar *address_string = libbalsa_address_to_gchar(address, -1);
 
         gtk_list_store_prepend(GTK_LIST_STORE(model), &iter);
         /* GtkListStore refs address, and unrefs it when cleared  */
         gtk_list_store_set(GTK_LIST_STORE(model), &iter,
-                           LIST_COLUMN_NAME, address->full_name,
+                           LIST_COLUMN_NAME, full_name,
                            LIST_COLUMN_ADDRESS_STRING, address_string,
                            LIST_COLUMN_ADDRESS, address,
                            LIST_COLUMN_WHICH, -1,
@@ -775,23 +780,21 @@ balsa_ab_window_load_cb(LibBalsaAddressBook *libbalsa_ab,
 
 	g_free(address_string);
     } else {
-	address_list = address->address_list;
-	count = 0;
-	while ( address_list ) {
+        guint n;
+
+	for (n = 0; n < n_addrs; ++n) {
+            const gchar *addr = libbalsa_address_get_nth_addr(address, n);
+
             gtk_list_store_prepend(GTK_LIST_STORE(model), &iter);
             /* GtkListStore refs address once for each address in
              * the list, and unrefs it the same number of times when
              * cleared */
             gtk_list_store_set(GTK_LIST_STORE(model), &iter,
-                               LIST_COLUMN_NAME, address->full_name,
-                               LIST_COLUMN_ADDRESS_STRING,
-                               address_list->data,
+                               LIST_COLUMN_NAME, full_name,
+                               LIST_COLUMN_ADDRESS_STRING, addr,
                                LIST_COLUMN_ADDRESS, address,
-                               LIST_COLUMN_WHICH, count,
+                               LIST_COLUMN_WHICH, n,
                                -1);
-
-	    address_list = g_list_next(address_list);
-	    count++;
 	}
     }
 }
@@ -881,7 +884,7 @@ balsa_ab_window_menu_changed(GtkWidget * widget, BalsaAbWindow *ab)
 
     ab->current_address_book = addr;
 
-    g_signal_handler_block(G_OBJECT(ab->single_address_mode_radio),
+    g_signal_handler_block(ab->single_address_mode_radio,
 			   ab->toggle_handler_id);
     if (libbalsa_address_book_get_dist_list_mode(ab->current_address_book))
 	gtk_toggle_button_set_active(
@@ -889,7 +892,7 @@ balsa_ab_window_menu_changed(GtkWidget * widget, BalsaAbWindow *ab)
     else
 	gtk_toggle_button_set_active(
 	    GTK_TOGGLE_BUTTON(ab->single_address_mode_radio), TRUE);
-    g_signal_handler_unblock(G_OBJECT(ab->single_address_mode_radio),
+    g_signal_handler_unblock(ab->single_address_mode_radio,
 			     ab->toggle_handler_id);
 
     balsa_ab_window_load(ab);
