@@ -64,18 +64,26 @@ ask_password_real(LibBalsaServer *server, const gchar *cert_subject)
 #endif                          /* defined(HAVE_LIBSECRET) */
 	GcrPromptDialog *dialog;
 	gchar *prompt;
-	gboolean *remember;
+	gboolean remember;
 	gchar *passwd;
+        const gchar *user;
+        const gchar *host;
+        const gchar *protocol;
 
     g_return_val_if_fail(server != NULL, NULL);
 
+    user = libbalsa_server_get_user(server);
+    host = libbalsa_server_get_host(server);
+    protocol = libbalsa_server_get_protocol(server);
     if (cert_subject != NULL) {
     	prompt = g_strdup_printf(_("Password to unlock the user certificate\n%s\nfor %s@%s (%s)"),
-    		cert_subject, server->user, server->host, server->protocol);
-    	remember = &server->remember_cert_passphrase;
+    		cert_subject,
+                user, host, protocol);
+    	remember = libbalsa_server_get_remember_cert_passphrase(server);
     } else {
-    	prompt = g_strdup_printf(_("Password for %s@%s (%s)"), server->user, server->host, server->protocol);
-    	remember = &server->remember_passwd;
+    	prompt = g_strdup_printf(_("Password for %s@%s (%s)"),
+                user, host, protocol);
+    	remember = libbalsa_server_get_remember_password(server);
     }
 	dialog = g_object_new(GCR_TYPE_PROMPT_DIALOG,
 						  "use-header-bar", libbalsa_use_headerbar(),
@@ -85,7 +93,7 @@ ask_password_real(LibBalsaServer *server, const gchar *cert_subject)
 						  "choice-label", remember_password_message,
 						  "cancel-label", _("_Cancel"),
 						  "continue-label", _("_OK"),
-						  "choice-chosen", *remember,
+						  "choice-chosen", remember,
 						  "destroy-with-parent", TRUE,
 						  "transient-for", GTK_WINDOW(balsa_app.main_window),
 						  "modal", TRUE,
@@ -95,10 +103,14 @@ ask_password_real(LibBalsaServer *server, const gchar *cert_subject)
 	if (passwd != NULL) {
 		gboolean old_remember;
 
-		old_remember = *remember;
-		*remember = gcr_prompt_get_choice_chosen(GCR_PROMPT(dialog));
+		old_remember = remember;
+		remember = gcr_prompt_get_choice_chosen(GCR_PROMPT(dialog));
+                if (cert_subject != NULL)
+                    libbalsa_server_set_remember_cert_passphrase(server, remember);
+                else
+                    libbalsa_server_set_remember_password(server, remember);
 		libbalsa_server_set_password(server, passwd, cert_subject != NULL);
-		if (*remember || old_remember) {
+		if (remember || old_remember) {
 			libbalsa_server_config_changed(server);
 		}
 	}
@@ -124,7 +136,7 @@ ask_password_real(LibBalsaServer * server, const gchar *cert_subject)
     GtkWidget *rememb_check;
     gchar *prompt;
     gchar *passwd;
-	gboolean *remember;
+	gboolean remember;
 #if defined(HAVE_LIBSECRET)
     static const gchar *remember_password_message =
         N_("_Remember password in Secret Service");
@@ -136,11 +148,11 @@ ask_password_real(LibBalsaServer * server, const gchar *cert_subject)
     g_return_val_if_fail(server != NULL, NULL);
     if (cert_subject != NULL) {
     	prompt = g_strdup_printf(_("Password to unlock the user certificate\n%s\nfor %s@%s (%s)"),
-    		cert_subject, server->user, server->host, server->protocol);
-    	remember = &server->remember_cert_passphrase;
+    		cert_subject, libbalsa_server_get_user(server), libbalsa_server_get_host(server), libbalsa_server_get_protocol(server));
+    	remember = libbalsa_server_get_remember_cert_passphrase(server);
     } else {
-    	prompt = g_strdup_printf(_("Password for %s@%s (%s)"), server->user, server->host, server->protocol);
-       	remember = &server->remember_passwd;
+    	prompt = g_strdup_printf(_("Password for %s@%s (%s)"), libbalsa_server_get_user(server), libbalsa_server_get_host(server), libbalsa_server_get_protocol(server));
+       	remember = libbalsa_server_get_remember_password(server);
     }
 
     dialog = gtk_dialog_new_with_buttons(_("Password needed"),
@@ -165,12 +177,12 @@ ask_password_real(LibBalsaServer * server, const gchar *cert_subject)
 
     label = libbalsa_create_grid_label(_("Password:"), grid, 1);
     entry = libbalsa_create_grid_entry(grid, NULL, NULL, 1, NULL, label);
-    g_object_set(G_OBJECT(entry), "input-purpose", GTK_INPUT_PURPOSE_PASSWORD, NULL);
+    g_object_set(entry, "input-purpose", GTK_INPUT_PURPOSE_PASSWORD, NULL);
     gtk_entry_set_visibility(GTK_ENTRY(entry), FALSE);
     gtk_entry_set_activates_default(GTK_ENTRY(entry), TRUE);
     gtk_widget_grab_focus(entry);
 
-    rememb_check = libbalsa_create_grid_check(remember_password_message, grid, 2, *remember);
+    rememb_check = libbalsa_create_grid_check(remember_password_message, grid, 2, remember);
 
     gtk_widget_show_all(grid);
     gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
@@ -178,11 +190,15 @@ ask_password_real(LibBalsaServer * server, const gchar *cert_subject)
     if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK) {
 		gboolean old_remember;
 
-		old_remember = *remember;
+		old_remember = remember;
 		passwd = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
-		*remember = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rememb_check));
+		remember = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rememb_check));
+                if (cert_subject != NULL)
+                    libbalsa_server_set_remember_cert_passphrase(server, remember);
+                else
+                    libbalsa_server_set_remember_password(server, remember);
 		libbalsa_server_set_password(server, passwd, cert_subject != NULL);
-		if (*remember || old_remember) {
+		if (remember || old_remember) {
 			libbalsa_server_config_changed(server);
 		}
     } else {
@@ -407,11 +423,9 @@ balsa_app_init(void)
     balsa_app.check_imap_inbox = 0;
     balsa_app.imap_scan_depth = 1;
 
-#ifdef HAVE_GPGME
     /* gpgme stuff */
     balsa_app.has_openpgp = FALSE;
     balsa_app.has_smime = FALSE;
-#endif
 
     /* Message filing */
     balsa_app.folder_mru=NULL;
@@ -425,8 +439,7 @@ balsa_app_destroy(void)
 {
     config_save();
 
-    g_list_foreach(balsa_app.address_book_list, (GFunc)g_object_unref, NULL);
-    g_list_free(balsa_app.address_book_list);
+    g_list_free_full(balsa_app.address_book_list, g_object_unref);
     balsa_app.address_book_list = NULL;
 
     /* now free filters */
@@ -435,21 +448,17 @@ balsa_app_destroy(void)
     g_slist_free(balsa_app.filters);
     balsa_app.filters = NULL;
 
-    g_list_foreach(balsa_app.identities, (GFunc)g_object_unref, NULL);
-    g_list_free(balsa_app.identities);
+    g_list_free_full(balsa_app.identities, g_object_unref);
     balsa_app.identities = NULL;
 
-    g_list_foreach(balsa_app.inbox_input, (GFunc)g_object_unref, NULL);
-    g_list_free(balsa_app.inbox_input);
+    g_list_free_full(balsa_app.inbox_input, g_object_unref);
     balsa_app.inbox_input = NULL;
 
 
-    g_list_foreach(balsa_app.folder_mru, (GFunc)g_free, NULL);
-    g_list_free(balsa_app.folder_mru);
+    g_list_free_full(balsa_app.folder_mru, g_free);
     balsa_app.folder_mru = NULL;
 
-    g_list_foreach(balsa_app.fcc_mru, (GFunc)g_free, NULL);
-    g_list_free(balsa_app.fcc_mru);
+    g_list_free_full(balsa_app.fcc_mru, g_free);
     balsa_app.fcc_mru = NULL;
 
 
@@ -601,7 +610,7 @@ find_mailbox(GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter,
     BalsaMailboxNode *mbnode;
 
     gtk_tree_model_get(model, iter, 0, &mbnode, -1);
-    if (mbnode->mailbox == bf->data) {
+    if (balsa_mailbox_node_get_mailbox(mbnode) == bf->data) {
 	bf->mbnode = mbnode;
 	return TRUE;
     }
@@ -639,8 +648,8 @@ find_path(GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter,
     BalsaMailboxNode *mbnode;
 
     gtk_tree_model_get(model, iter, 0, &mbnode, -1);
-    if (mbnode->server == bf->server &&
-        g_strcmp0(mbnode->dir, bf->data) == 0) {
+    if (balsa_mailbox_node_get_server(mbnode) == bf->server &&
+        g_strcmp0(balsa_mailbox_node_get_dir(mbnode), bf->data) == 0) {
 	bf->mbnode = mbnode;
 	return TRUE;
     }
@@ -671,7 +680,8 @@ find_url(GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter,
     LibBalsaMailbox *mailbox;
 
     gtk_tree_model_get(model, iter, 0, &mbnode, -1);
-    if ((mailbox = mbnode->mailbox) && !strcmp(mailbox->url, bf->data)) {
+    if ((mailbox = balsa_mailbox_node_get_mailbox(mbnode)) != NULL &&
+        strcmp(libbalsa_mailbox_get_url(mailbox), bf->data) == 0) {
         bf->mbnode = mbnode;
         return TRUE;
     }
@@ -720,7 +730,7 @@ balsa_find_mailbox_by_url(const gchar * url)
     LibBalsaMailbox *mailbox = NULL;
 
     if ((mbnode = balsa_find_url(url))) {
-	mailbox = mbnode->mailbox;
+	mailbox = balsa_mailbox_node_get_mailbox(mbnode);
 	g_object_unref(mbnode);
     }
     return mailbox;
@@ -737,16 +747,28 @@ gchar*
 balsa_get_short_mailbox_name(const gchar *url)
 {
     BalsaMailboxNode *mbnode;
+    gchar *short_name = NULL;
 
-    if ((mbnode = balsa_find_url(url)) && mbnode->mailbox) {
-        if (mbnode->server) {
-            return g_strconcat(mbnode->server->host, ":",
-                               mbnode->mailbox->name, NULL);
-        } else {
-            return g_strdup(mbnode->mailbox->name);
+    if ((mbnode = balsa_find_url(url)) != NULL) {
+        LibBalsaMailbox *mailbox = balsa_mailbox_node_get_mailbox(mbnode);
+
+        if (mailbox != NULL) {
+            const gchar *name = libbalsa_mailbox_get_name(mailbox);
+            LibBalsaServer *server = balsa_mailbox_node_get_server(mbnode);
+
+            if (server != NULL) {
+                short_name = g_strconcat(libbalsa_server_get_host(server), ":",
+                                     name, NULL);
+            } else {
+                short_name = g_strdup(name);
+            }
         }
     }
-    return g_strdup(url);
+
+    if (short_name == NULL)
+        short_name = g_strdup(url);
+
+    return short_name;
 }
 
 struct balsa_find_iter_by_data_info {
@@ -765,7 +787,7 @@ balsa_find_iter_by_data_func(GtkTreeModel * model, GtkTreePath * path,
     gtk_tree_model_get(model, iter, 0, &mbnode, -1);
     if(!mbnode)
         return FALSE;
-    if (mbnode == bf->data || mbnode->mailbox == bf->data) {
+    if (mbnode == bf->data || balsa_mailbox_node_get_mailbox(mbnode) == bf->data) {
 	*bf->iter = *iter;
 	bf->found = TRUE;
     }
@@ -812,9 +834,12 @@ ba_remove_children_mailbox_nodes(GtkTreeModel * model, GtkTreeIter * parent,
 	return;
 
     do {
+        LibBalsaMailbox *mailbox;
+
 	gtk_tree_model_get(model, &iter, 0, &mbnode, -1);
-	if (mbnode->parent) {
-	    LibBalsaMailbox *mailbox = mbnode->mailbox;
+	mailbox = balsa_mailbox_node_get_mailbox(mbnode);
+
+	if (balsa_mailbox_node_get_parent(mbnode) != NULL) {
 	    if (mailbox == balsa_app.inbox
 		|| mailbox == balsa_app.outbox
 		|| mailbox == balsa_app.sentbox
@@ -828,11 +853,13 @@ ba_remove_children_mailbox_nodes(GtkTreeModel * model, GtkTreeIter * parent,
 		gtk_tree_store_remove(balsa_app.mblist_tree_store, &iter);
 	} else {
 	    printf("sparing %s %s\n",
-		   mbnode->mailbox ? "mailbox" : "folder ",
-		   mbnode->mailbox ? mbnode->mailbox->name : mbnode->name);
+		   mailbox != NULL ? "mailbox" : "folder ",
+		   mailbox != NULL ?
+                   libbalsa_mailbox_get_name(mailbox) :
+                   balsa_mailbox_node_get_name(mbnode));
 	    valid = gtk_tree_model_iter_next(model, &iter);
 	}
-	g_object_unref(mbnode); 
+	g_object_unref(mbnode);
     } while (valid);
 }
 
@@ -844,9 +871,13 @@ balsa_remove_children_mailbox_nodes(BalsaMailboxNode * mbnode)
     GtkTreeIter *iter = NULL;
     GSList *specials = NULL, *l;
 
-    if (balsa_app.debug)
-	printf("Destroying children of %p %s\n",
-	       mbnode, mbnode && mbnode->name ? mbnode->name : "");
+    if (balsa_app.debug) {
+        const gchar *name;
+
+	printf("Destroying children of %p %s\n", mbnode,
+               (mbnode != NULL &&
+                (name = balsa_mailbox_node_get_name(mbnode)) != NULL) ? name : "");
+    }
 
     if (mbnode && balsa_find_iter_by_data(&parent, mbnode))
 	iter = &parent;
@@ -868,18 +899,23 @@ BalsaIndex*
 balsa_find_index_by_mailbox(LibBalsaMailbox * mailbox)
 {
     GtkWidget *page;
-    GtkWidget *index;
+    GtkWidget *child;
     guint i;
-    g_return_val_if_fail(balsa_app.notebook, NULL);
+
+    g_return_val_if_fail(LIBBALSA_IS_MAILBOX(mailbox), NULL);
+    g_return_val_if_fail(GTK_IS_NOTEBOOK(balsa_app.notebook), NULL);
 
     for (i = 0;
-	 (page =
-	  gtk_notebook_get_nth_page(GTK_NOTEBOOK(balsa_app.notebook), i));
+	 (page = gtk_notebook_get_nth_page((GtkNotebook *) balsa_app.notebook, i)) != NULL;
 	 i++) {
-        index = gtk_bin_get_child(GTK_BIN(page));
-	if (index && BALSA_INDEX(index)->mailbox_node
-            && BALSA_INDEX(index)->mailbox_node->mailbox == mailbox)
-	    return BALSA_INDEX(index);
+        child = gtk_bin_get_child(GTK_BIN(page));
+	if (child != NULL) {
+            BalsaIndex *bindex = BALSA_INDEX(child);
+            LibBalsaMailbox *this_mailbox = balsa_index_get_mailbox(bindex);
+
+            if (this_mailbox == mailbox)
+                return bindex;
+        }
     }
 
     /* didn't find a matching mailbox */
