@@ -1,6 +1,6 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2016 Stuart Parmenter and others
+ * Copyright (C) 1997-2019 Stuart Parmenter and others
  * Written by (C) Albrecht Dre <albrecht.dress@arcor.de> 2007
  *
  * This program is free software; you can redistribute it and/or modify
@@ -31,10 +31,6 @@ struct _BalsaCiteBar {
     gint space;
 };
 
-struct _BalsaCiteBarClass {
-    GtkWidgetClass parent_class;
-};
-
 static void balsa_cite_bar_get_preferred_width (GtkWidget * widget,
                                                 gint      * minimum_width,
                                                 gint      * natural_width);
@@ -46,8 +42,6 @@ static gboolean balsa_cite_bar_draw            (GtkWidget * widget,
 
 G_DEFINE_TYPE(BalsaCiteBar, balsa_cite_bar, GTK_TYPE_WIDGET)
 
-static GtkWidgetClass *parent_class = NULL;
-
 static void
 balsa_cite_bar_class_init(BalsaCiteBarClass * class)
 {
@@ -55,8 +49,6 @@ balsa_cite_bar_class_init(BalsaCiteBarClass * class)
 
     widget_class = (GtkWidgetClass *) class;
 
-    parent_class = g_type_class_peek_parent(class);
-
     widget_class->get_preferred_width  = balsa_cite_bar_get_preferred_width;
     widget_class->get_preferred_height = balsa_cite_bar_get_preferred_height;
     widget_class->draw                 = balsa_cite_bar_draw;
