@@ -60,8 +60,6 @@
 
 
 /* gtk widget */
-static void bndx_class_init(BalsaIndexClass * klass);
-static void bndx_instance_init(BalsaIndex * index);
 static void bndx_destroy(GObject * obj);
 static gboolean bndx_popup_menu(GtkWidget * widget);
 
@@ -142,38 +140,44 @@ static void bndx_select_row(BalsaIndex * index, GtkTreePath * path);
 /* Other callbacks. */
 static void bndx_store_address(gpointer data);
 
-static GtkTreeViewClass *parent_class = NULL;
+struct _BalsaIndex {
+    GtkTreeView tree_view;
+
+    /* the popup menu and some items we need to refer to */
+    GtkWidget *popup_menu;
+    GtkWidget *delete_item;
+    GtkWidget *undelete_item;
+    GtkWidget *move_to_trash_item;
+    GtkWidget *toggle_item;
+    GtkWidget *move_to_item;
+
+    BalsaMailboxNode* mailbox_node;
+    guint current_msgno;
+    guint next_msgno;
+	gboolean current_message_is_deleted:1;
+    gboolean prev_message:1;
+    gboolean next_message:1;
+    gboolean has_selection_changed_idle:1;
+    gboolean has_mailbox_changed_idle:1;
+    gboolean collapsing:1;
+    int    filter_no;
+    gchar *filter_string; /* Quick view filter string, if any */
+
+    /* signal handler ids */
+    gulong row_expanded_id;
+    gulong row_collapsed_id;
+    gulong selection_changed_id;
+
+	LibBalsaMailboxSearchIter *search_iter;
+    BalsaIndexWidthPreference width_preference;
+};
 
 /* Class type. */
-GType
-balsa_index_get_type(void)
-{
-    static GType balsa_index_type = 0;
-
-    if (!balsa_index_type) {
-        static const GTypeInfo balsa_index_info = {
-            sizeof(BalsaIndexClass),
-            NULL,               /* base_init */
-            NULL,               /* base_finalize */
-            (GClassInitFunc) bndx_class_init,
-            NULL,               /* class_finalize */
-            NULL,               /* class_data */
-            sizeof(BalsaIndex),
-            0,                  /* n_preallocs */
-            (GInstanceInitFunc) bndx_instance_init
-        };
-
-        balsa_index_type =
-            g_type_register_static(GTK_TYPE_TREE_VIEW,
-                                   "BalsaIndex", &balsa_index_info, 0);
-    }
-
-    return balsa_index_type;
-}
+G_DEFINE_TYPE(BalsaIndex, balsa_index, GTK_TYPE_TREE_VIEW)
 
 /* BalsaIndex class init method. */
 static void
-bndx_class_init(BalsaIndexClass * klass)
+balsa_index_class_init(BalsaIndexClass * klass)
 {
     GObjectClass *object_class;
     GtkWidgetClass *widget_class;
@@ -181,21 +185,17 @@ bndx_class_init(BalsaIndexClass * klass)
     object_class = (GObjectClass *) klass;
     widget_class = (GtkWidgetClass *) klass;
 
-    parent_class = g_type_class_peek_parent(klass);
-
     balsa_index_signals[INDEX_CHANGED] =
         g_signal_new("index-changed",
                      G_TYPE_FROM_CLASS(object_class),
 		     G_SIGNAL_RUN_FIRST,
-                     G_STRUCT_OFFSET(BalsaIndexClass,
-                                     index_changed),
+                     0,
                      NULL, NULL,
 		     NULL,
                      G_TYPE_NONE, 0);
 
     object_class->dispose = bndx_destroy;
     widget_class->popup_menu = bndx_popup_menu;
-    klass->index_changed = NULL;
 }
 
 /* Object class destroy method. */
@@ -218,7 +218,7 @@ bndx_destroy(GObject * obj)
     if (index->mailbox_node) {
 	LibBalsaMailbox* mailbox;
 
-	if ((mailbox = index->mailbox_node->mailbox)) {
+	if ((mailbox = balsa_mailbox_node_get_mailbox(index->mailbox_node)) != NULL) {
 	    g_signal_handlers_disconnect_matched(mailbox,
 						 G_SIGNAL_MATCH_DATA,
 						 0, 0, NULL, NULL, index);
@@ -246,8 +246,7 @@ bndx_destroy(GObject * obj)
 
     g_free(index->filter_string); index->filter_string = NULL;
 
-    if (G_OBJECT_CLASS(parent_class)->dispose)
-        (*G_OBJECT_CLASS(parent_class)->dispose) (obj);
+    G_OBJECT_CLASS(balsa_index_parent_class)->dispose(obj);
 }
 
 /* Widget class popup menu method. */
@@ -296,7 +295,7 @@ bndx_string_width(const gchar * text)
 /* BalsaIndex instance init method; no tree store is set on the tree
  * view--that's handled later, when the view is populated. */
 static void
-bndx_instance_init(BalsaIndex * index)
+balsa_index_init(BalsaIndex * index)
 {
     GtkTreeView *tree_view = GTK_TREE_VIEW(index);
     GtkTreeSelection *selection = gtk_tree_view_get_selection(tree_view);
@@ -506,7 +505,7 @@ bndx_selection_changed_idle(BalsaIndex * index)
 
     if (!index->mailbox_node)
         return FALSE;
-    mailbox = index->mailbox_node->mailbox;
+    mailbox = balsa_mailbox_node_get_mailbox(index->mailbox_node);
     selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(index));
 
     /* Save next_msgno, because changing flags may zero it. */
@@ -570,7 +569,7 @@ bndx_selection_changed(GtkTreeSelection * selection, BalsaIndex * index)
     if (index->current_msgno) {
         GtkTreePath *path;
 
-        if (libbalsa_mailbox_msgno_find(index->mailbox_node->mailbox,
+        if (libbalsa_mailbox_msgno_find(balsa_mailbox_node_get_mailbox(index->mailbox_node),
                                         index->current_msgno,
                                         &path, NULL)) {
             gboolean update_preview = TRUE;
@@ -594,7 +593,7 @@ bndx_selection_changed(GtkTreeSelection * selection, BalsaIndex * index)
             gtk_tree_path_free(path);
 
             if (!update_preview) {
-                g_signal_emit(G_OBJECT(index), balsa_index_signals[INDEX_CHANGED], 0);
+                g_signal_emit(index, balsa_index_signals[INDEX_CHANGED], 0);
                 return;
             }
         }
@@ -666,7 +665,7 @@ bndx_row_activated(GtkTreeView * tree_view, GtkTreePath * path,
         BalsaSendmsg *sm =
             sendmsg_window_continue(mailbox, msgno);
         if (sm)
-            g_signal_connect(G_OBJECT(sm->window), "destroy",
+            g_signal_connect(sm->window, "destroy",
                              G_CALLBACK(sendmsg_window_destroy_cb), NULL);
     } else
         message_window_new(mailbox, msgno);
@@ -677,7 +676,7 @@ bndx_find_current_msgno(BalsaIndex * bindex,
                         GtkTreePath ** path , GtkTreeIter * iter)
 {
     return bindex->current_msgno > 0
-        && libbalsa_mailbox_msgno_find(bindex->mailbox_node->mailbox,
+        && libbalsa_mailbox_msgno_find(balsa_mailbox_node_get_mailbox(bindex->mailbox_node),
                                        bindex->current_msgno, path, iter);
 }
 
@@ -808,6 +807,7 @@ static gboolean
 bndx_scroll_on_open_idle(BalsaIndex *index)
 {
     LibBalsaMailbox *mailbox;
+    guint first_unread;
     GtkTreeView *tree_view = GTK_TREE_VIEW(index);
     GtkTreePath *path;
     gpointer view_on_open;
@@ -816,11 +816,12 @@ bndx_scroll_on_open_idle(BalsaIndex *index)
         return FALSE;
 
     balsa_index_update_tree(index, balsa_app.expand_tree);
-    mailbox = index->mailbox_node->mailbox;
-    if (mailbox->first_unread) {
-	unsigned msgno = mailbox->first_unread;
-	mailbox->first_unread = 0;
-        if(!libbalsa_mailbox_msgno_find(mailbox, msgno, &path, NULL))
+    mailbox = balsa_mailbox_node_get_mailbox(index->mailbox_node);
+    first_unread = libbalsa_mailbox_get_first_unread(mailbox);
+    if (first_unread > 0) {
+	unsigned msgno = first_unread;
+        libbalsa_mailbox_set_first_unread(mailbox, 0);
+        if (!libbalsa_mailbox_msgno_find(mailbox, msgno, &path, NULL))
             return FALSE; /* Oops! */
     } else {
         /* we want to scroll to the last one in current order. The
@@ -901,7 +902,7 @@ bndx_mailbox_row_inserted_cb(LibBalsaMailbox * mailbox, GtkTreePath * path,
 {
     guint msgno;
 
-    if (mailbox->state != LB_MAILBOX_STATE_OPEN)
+    if (libbalsa_mailbox_get_state(mailbox) != LB_MAILBOX_STATE_OPEN)
         return;
 
     gtk_tree_model_get(GTK_TREE_MODEL(mailbox), iter,
@@ -946,7 +947,7 @@ bndx_mailbox_message_expunged_cb(LibBalsaMailbox * mailbox, guint msgno,
 
 /* balsa_index_load_mailbox_node:
  *
- * mbnode->mailbox is already open
+ * balsa_mailbox_node_get_mailbox(mbnode) is already open
  */
 
 gboolean
@@ -959,9 +960,9 @@ balsa_index_load_mailbox_node(BalsaIndex * index,
     g_return_val_if_fail(BALSA_IS_INDEX(index), TRUE);
     g_return_val_if_fail(index->mailbox_node == NULL, TRUE);
     g_return_val_if_fail(BALSA_IS_MAILBOX_NODE(mbnode), TRUE);
-    g_return_val_if_fail(LIBBALSA_IS_MAILBOX(mbnode->mailbox), TRUE);
 
-    mailbox = mbnode->mailbox;
+    mailbox = balsa_mailbox_node_get_mailbox(mbnode);
+    g_return_val_if_fail(LIBBALSA_IS_MAILBOX(mailbox), TRUE);
 
     /*
      * set the new mailbox
@@ -1000,7 +1001,7 @@ balsa_index_load_mailbox_node(BalsaIndex * index,
                                              LIBBALSA_MESSAGE_FLAG_DELETED);
     index->search_iter = libbalsa_mailbox_search_iter_new(cond_undeleted);
     /* Note when this mailbox was opened, for use in auto-closing. */
-    time(&index->mailbox_node->last_use);
+    balsa_mailbox_node_set_last_use_time(index->mailbox_node);
 
     return FALSE;
 }
@@ -1109,14 +1110,14 @@ bndx_search_iter(BalsaIndex * index,
 		 BndxSearchViewable viewable,
 		 guint stop_msgno)
 {
+    LibBalsaMailbox *mailbox = balsa_mailbox_node_get_mailbox(index->mailbox_node);
     gboolean found;
 
     do {
 	GtkTreePath *path;
 
 	found =
-	    libbalsa_mailbox_search_iter_step(index->mailbox_node->mailbox,
-					      search_iter, iter,
+	    libbalsa_mailbox_search_iter_step(mailbox, search_iter, iter,
 					      direction ==
 					      BNDX_SEARCH_DIRECTION_NEXT,
 					      stop_msgno);
@@ -1125,9 +1126,7 @@ bndx_search_iter(BalsaIndex * index,
 	if (viewable == BNDX_SEARCH_VIEWABLE_ANY)
 	    break;
 
-	path = gtk_tree_model_get_path(GTK_TREE_MODEL
-				       (index->mailbox_node->mailbox),
-				       iter);
+	path = gtk_tree_model_get_path(GTK_TREE_MODEL(mailbox), iter);
 	found = bndx_row_is_viewable(index, path);
 	gtk_tree_path_free(path);
     } while (!found);
@@ -1147,7 +1146,7 @@ bndx_search_iter_and_select(BalsaIndex * index,
     guint stop_msgno;
 
     if (!((index->next_msgno > 0
-           && libbalsa_mailbox_msgno_find(index->mailbox_node->mailbox,
+           && libbalsa_mailbox_msgno_find(balsa_mailbox_node_get_mailbox(index->mailbox_node),
                                           index->next_msgno, NULL, &iter))
           || (start == BNDX_SEARCH_START_ANY
               && bndx_find_root(index, &iter))))
@@ -1361,6 +1360,7 @@ static gboolean
 bndx_mailbox_changed_idle(BalsaIndex * bindex)
 {
     LibBalsaMailbox *mailbox;
+    guint first_unread;
     GtkTreePath *path;
 
     if (bndx_clear_if_last_ref(&bindex))
@@ -1368,13 +1368,14 @@ bndx_mailbox_changed_idle(BalsaIndex * bindex)
 
     bindex->has_mailbox_changed_idle = FALSE;
 
-    mailbox = bindex->mailbox_node->mailbox;
-    if (mailbox->first_unread
-        && libbalsa_mailbox_msgno_find(mailbox, mailbox->first_unread,
+    mailbox = balsa_mailbox_node_get_mailbox(bindex->mailbox_node);
+    first_unread = libbalsa_mailbox_get_first_unread(mailbox);
+    if (first_unread > 0
+        && libbalsa_mailbox_msgno_find(mailbox, first_unread,
                                        &path, NULL)) {
         bndx_expand_to_row(bindex, path);
         gtk_tree_path_free(path);
-        mailbox->first_unread = 0;
+        libbalsa_mailbox_set_first_unread(mailbox, 0);
     }
 
     if (bndx_find_current_msgno(bindex, &path, NULL)) {
@@ -1437,14 +1438,14 @@ balsa_index_selected_msgnos_new(BalsaIndex * index)
     gtk_tree_selection_selected_foreach(selection,
                                         (GtkTreeSelectionForeachFunc)
                                         bndx_selected_msgnos_func, msgnos);
-    libbalsa_mailbox_register_msgnos(index->mailbox_node->mailbox, msgnos);
+    libbalsa_mailbox_register_msgnos(balsa_mailbox_node_get_mailbox(index->mailbox_node), msgnos);
     return msgnos;
 }
 
 void
 balsa_index_selected_msgnos_free(BalsaIndex * index, GArray * msgnos)
 {
-    libbalsa_mailbox_unregister_msgnos(index->mailbox_node->mailbox,
+    libbalsa_mailbox_unregister_msgnos(balsa_mailbox_node_get_mailbox(index->mailbox_node),
                                        msgnos);
     g_array_free(msgnos, TRUE);
 }
@@ -1453,7 +1454,7 @@ static void
 bndx_view_source(gpointer data)
 {
     BalsaIndex *index = BALSA_INDEX(data);
-    LibBalsaMailbox *mailbox = index->mailbox_node->mailbox;
+    LibBalsaMailbox *mailbox = balsa_mailbox_node_get_mailbox(index->mailbox_node);
     guint i;
     GArray *selected = balsa_index_selected_msgnos_new(index);
 
@@ -1478,8 +1479,7 @@ bndx_store_address(gpointer data)
     GList *messages = balsa_index_selected_list(BALSA_INDEX(data));
 
     balsa_store_address_from_messages(messages);
-    g_list_foreach(messages, (GFunc)g_object_unref, NULL);
-    g_list_free(messages);
+    g_list_free_full(messages, g_object_unref);
 }
 
 static void
@@ -1523,7 +1523,7 @@ balsa_index_selected_list(BalsaIndex * index)
 static void
 bndx_compose_foreach(BalsaIndex * index, SendType send_type)
 {
-    LibBalsaMailbox *mailbox = index->mailbox_node->mailbox;
+    LibBalsaMailbox *mailbox = balsa_mailbox_node_get_mailbox(index->mailbox_node);
     GArray *selected;
     guint i;
     guint skipped = 0U;
@@ -1548,7 +1548,7 @@ bndx_compose_foreach(BalsaIndex * index, SendType send_type)
         }
 
         if (sm != NULL) {
-            g_signal_connect(G_OBJECT(sm->window), "destroy",
+            g_signal_connect(sm->window, "destroy",
                              G_CALLBACK(sendmsg_window_destroy_cb), NULL);
         } else if (send_type == SEND_REPLY_GROUP) {
             ++skipped;
@@ -1603,11 +1603,11 @@ bndx_compose_from_list(BalsaIndex * index, SendType send_type)
 {
     GArray *selected = balsa_index_selected_msgnos_new(index);
     BalsaSendmsg *sm =
-        sendmsg_window_new_from_list(index->mailbox_node->mailbox,
+        sendmsg_window_new_from_list(balsa_mailbox_node_get_mailbox(index->mailbox_node),
                                      selected, send_type);
 
     balsa_index_selected_msgnos_free(index, selected);
-    g_signal_connect(G_OBJECT(sm->window), "destroy",
+    g_signal_connect(sm->window, "destroy",
                      G_CALLBACK(sendmsg_window_destroy_cb), NULL);
 }
 
@@ -1643,7 +1643,7 @@ bndx_do_delete(BalsaIndex* index, gboolean move_to_trash)
     BalsaIndex *trash = balsa_find_index_by_mailbox(balsa_app.trash);
     GArray *selected = balsa_index_selected_msgnos_new(index);
     GArray *messages;
-    LibBalsaMailbox *mailbox = index->mailbox_node->mailbox;
+    LibBalsaMailbox *mailbox = balsa_mailbox_node_get_mailbox(index->mailbox_node);
     guint i;
 
     messages = g_array_new(FALSE, FALSE, sizeof(guint));
@@ -1691,7 +1691,7 @@ balsa_message_move_to_trash(gpointer user_data)
     bndx_do_delete(index, TRUE);
     /* Note when message was flagged as deleted, for use in
      * auto-expunge. */
-    time(&index->mailbox_node->last_use);
+    balsa_mailbox_node_set_last_use_time(index->mailbox_node);
 }
 
 gint
@@ -1708,9 +1708,11 @@ balsa_find_notebook_page_num(LibBalsaMailbox * mailbox)
           gtk_notebook_get_nth_page(GTK_NOTEBOOK(balsa_app.notebook), i));
          i++) {
         GtkWidget *index = gtk_bin_get_child(GTK_BIN(page));
+        BalsaMailboxNode *mbnode;
 
-        if (index && BALSA_INDEX(index)->mailbox_node
-            && BALSA_INDEX(index)->mailbox_node->mailbox == mailbox)
+        if (index != NULL &&
+            (mbnode = BALSA_INDEX(index)->mailbox_node) != NULL &&
+            balsa_mailbox_node_get_mailbox(mbnode) == mailbox)
             return i;
     }
 
@@ -1724,7 +1726,7 @@ balsa_find_notebook_page_num(LibBalsaMailbox * mailbox)
 void
 balsa_index_toggle_flag(BalsaIndex* index, LibBalsaMessageFlag flag)
 {
-    LibBalsaMailbox *mailbox = index->mailbox_node->mailbox;
+    LibBalsaMailbox *mailbox = balsa_mailbox_node_get_mailbox(index->mailbox_node);
     int is_all_flagged = TRUE;
     GArray *selected = balsa_index_selected_msgnos_new(index);
     guint i;
@@ -1746,7 +1748,7 @@ balsa_index_toggle_flag(BalsaIndex* index, LibBalsaMessageFlag flag)
     if (flag == LIBBALSA_MESSAGE_FLAG_DELETED)
 	/* Note when deleted flag was changed, for use in
 	 * auto-expunge. */
-	time(&index->mailbox_node->last_use);
+	balsa_mailbox_node_set_last_use_time(index->mailbox_node);
 }
 
 static void
@@ -1762,7 +1764,7 @@ bi_toggle_deleted_cb(gpointer user_data, GtkWidget * widget)
 
     selected = balsa_index_selected_msgnos_new(index);
     if (widget == index->undelete_item && selected->len > 0) {
-	LibBalsaMailbox *mailbox = index->mailbox_node->mailbox;
+	LibBalsaMailbox *mailbox = balsa_mailbox_node_get_mailbox(index->mailbox_node);
         guint msgno = g_array_index(selected, guint, 0);
         if (libbalsa_mailbox_msgno_has_flags(mailbox, msgno,
                                              LIBBALSA_MESSAGE_FLAG_DELETED,
@@ -1801,7 +1803,7 @@ mru_menu_cb(const gchar * url, BalsaIndex * index)
 
     g_return_if_fail(mailbox != NULL);
 
-    if (index->mailbox_node->mailbox != mailbox) {
+    if (balsa_mailbox_node_get_mailbox(index->mailbox_node) != mailbox) {
         GArray *selected = balsa_index_selected_msgnos_new(index);
         balsa_index_transfer(index, selected, mailbox, FALSE);
         balsa_index_selected_msgnos_free(index, selected);
@@ -1940,10 +1942,11 @@ bndx_do_popup(BalsaIndex * index, GdkEventButton * event)
     gboolean any_not_deleted = FALSE;
     GArray *selected = balsa_index_selected_msgnos_new(index);
     guint i;
+    gboolean readonly;
 
     BALSA_DEBUG();
 
-    mailbox = index->mailbox_node->mailbox;
+    mailbox = balsa_mailbox_node_get_mailbox(index->mailbox_node);
     for (i = 0; i < selected->len; i++) {
         guint msgno = g_array_index(selected, guint, i);
         if (libbalsa_mailbox_msgno_has_flags(mailbox, msgno,
@@ -1961,18 +1964,18 @@ bndx_do_popup(BalsaIndex * index, GdkEventButton * event)
         gtk_widget_set_sensitive(GTK_WIDGET(list->data), any);
     g_list_free(l);
 
-    mailbox = index->mailbox_node->mailbox;
+    readonly = libbalsa_mailbox_get_readonly(mailbox);
     gtk_widget_set_sensitive(index->delete_item,
-                             any_not_deleted && !mailbox->readonly);
+                             any_not_deleted && !readonly);
     gtk_widget_set_sensitive(index->undelete_item,
-                             any_deleted && !mailbox->readonly);
+                             any_deleted && !readonly);
     gtk_widget_set_sensitive(index->move_to_trash_item,
                              any && mailbox != balsa_app.trash
-                             && !mailbox->readonly);
+                             && !readonly);
     gtk_widget_set_sensitive(index->toggle_item,
-                             any && !mailbox->readonly);
+                             any && !readonly);
     gtk_widget_set_sensitive(index->move_to_item,
-                             any && !mailbox->readonly);
+                             any && !readonly);
 
     submenu =
         balsa_mblist_mru_menu(GTK_WINDOW
@@ -2008,7 +2011,7 @@ create_stock_menu_item(GtkWidget * menu, const gchar * label, GCallback cb,
 {
     GtkWidget *menuitem = gtk_menu_item_new_with_mnemonic(label);
 
-    g_signal_connect_swapped(G_OBJECT(menuitem), "activate",
+    g_signal_connect_swapped(menuitem, "activate",
                              G_CALLBACK(cb), data);
 
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
@@ -2064,7 +2067,7 @@ balsa_index_set_thread_messages(BalsaIndex * index,
 
     g_return_if_fail(index != NULL);
     g_return_if_fail(index->mailbox_node != NULL);
-    mailbox = index->mailbox_node->mailbox;
+    mailbox = balsa_mailbox_node_get_mailbox(index->mailbox_node);
     g_return_if_fail(mailbox != NULL);
 
     if (thread_messages) {
@@ -2095,7 +2098,7 @@ balsa_index_set_view_filter(BalsaIndex * bindex, int filter_no,
     LibBalsaMailbox *mailbox;
 
     g_return_if_fail(BALSA_IS_INDEX(bindex));
-    mailbox = bindex->mailbox_node->mailbox;
+    mailbox = balsa_mailbox_node_get_mailbox(bindex->mailbox_node);
 
     g_free(bindex->filter_string);
     bindex->filter_no = filter_no;
@@ -2125,22 +2128,24 @@ balsa_index_transfer(BalsaIndex *index, GArray * msgnos,
     gboolean success;
     LibBalsaMailbox *from_mailbox;
     GError *e = NULL;
+    const gchar *to_name;
 
     if (msgnos->len == 0)
         return;
 
-    from_mailbox = index->mailbox_node->mailbox;
+    from_mailbox = balsa_mailbox_node_get_mailbox(index->mailbox_node);
     success = copy ?
         libbalsa_mailbox_messages_copy(from_mailbox, msgnos, to_mailbox, &e) :
         libbalsa_mailbox_messages_move(from_mailbox, msgnos, to_mailbox, &e);
 
+    to_name = libbalsa_mailbox_get_name(to_mailbox);
     if (!success) {
 	balsa_information
             (LIBBALSA_INFORMATION_WARNING,
              ngettext("Failed to copy %d message to mailbox “%s”: %s",
                       "Failed to copy %d messages to mailbox “%s”: %s",
                       msgnos->len),
-             msgnos->len, to_mailbox->name, e ? e->message : "?");
+             msgnos->len, to_name, e ? e->message : "?");
 	return;
     }
 
@@ -2150,11 +2155,11 @@ balsa_index_transfer(BalsaIndex *index, GArray * msgnos,
         enable_empty_trash(balsa_app.main_window, TRASH_FULL);
     balsa_information(LIBBALSA_INFORMATION_MESSAGE,
                       copy ? _("Copied to “%s”.")
-                      : _("Moved to “%s”."), to_mailbox->name);
+                      : _("Moved to “%s”."), to_name);
     if (!copy)
 	/* Note when message was flagged as deleted, for use in
 	 * auto-expunge. */
-	time(&index->mailbox_node->last_use);
+	balsa_mailbox_node_set_last_use_time(index->mailbox_node);
 }
 
 /* General helpers. */
@@ -2205,7 +2210,7 @@ bndx_changed_find_row(BalsaIndex * index)
         index->prev_message = FALSE;
     }
 
-    g_signal_emit(G_OBJECT(index), balsa_index_signals[INDEX_CHANGED], 0);
+    g_signal_emit(index, balsa_index_signals[INDEX_CHANGED], 0);
 }
 
 /* Make the actual selection,
@@ -2245,15 +2250,15 @@ balsa_index_expunge(BalsaIndex * index)
 
     g_return_if_fail(index != NULL);
 
-    mailbox = index->mailbox_node->mailbox;
-    if (mailbox->readonly)
+    mailbox = balsa_mailbox_node_get_mailbox(index->mailbox_node);
+    if (libbalsa_mailbox_get_readonly(mailbox))
 	return;
 
     rc = libbalsa_mailbox_sync_storage(mailbox, TRUE);
     if (!rc)
 	balsa_information(LIBBALSA_INFORMATION_WARNING,
 			  _("Committing mailbox %s failed."),
-			  mailbox->name);
+			  libbalsa_mailbox_get_name(mailbox));
 }
 
 /* Message window */
@@ -2262,7 +2267,7 @@ bndx_next_msgno(BalsaIndex * index, guint current_msgno,
                 LibBalsaMailboxSearchIter * search_iter,
                 BndxSearchDirection direction)
 {
-    LibBalsaMailbox *mailbox = index->mailbox_node->mailbox;
+    LibBalsaMailbox *mailbox = balsa_mailbox_node_get_mailbox(index->mailbox_node);
     GtkTreeModel *model = GTK_TREE_MODEL(mailbox);
     GtkTreeIter iter;
     guint msgno = 0;
@@ -2645,6 +2650,7 @@ bndx_pipe_response(GtkWidget * dialog, gint response,
 void
 balsa_index_pipe(BalsaIndex * index)
 {
+    LibBalsaMailbox *mailbox;
     struct bndx_mailbox_info *info;
     GtkWidget *label, *entry;
     GtkWidget *dialog;
@@ -2653,22 +2659,23 @@ balsa_index_pipe(BalsaIndex * index)
 
     g_return_if_fail(BALSA_IS_INDEX(index));
     g_return_if_fail(BALSA_IS_MAILBOX_NODE(index->mailbox_node));
-    g_return_if_fail(LIBBALSA_IS_MAILBOX(index->mailbox_node->mailbox));
 
-    info =
-        g_object_get_data(G_OBJECT(index->mailbox_node->mailbox),
-                          BALSA_INDEX_PIPE_INFO);
+    mailbox = balsa_mailbox_node_get_mailbox(index->mailbox_node);
+    g_return_if_fail(LIBBALSA_IS_MAILBOX(mailbox));
+
+    info = g_object_get_data(G_OBJECT(mailbox), BALSA_INDEX_PIPE_INFO);
     if (info) {
         gtk_window_present_with_time(GTK_WINDOW(info->dialog),
                                      gtk_get_current_event_time());
         return;
     }
 
-    if(!libbalsa_mailbox_open(index->mailbox_node->mailbox, NULL))
+    if (!libbalsa_mailbox_open(mailbox, NULL))
 	return;
+
     info = g_new(struct bndx_mailbox_info, 1);
     info->bindex = index;
-    info->mailbox = index->mailbox_node->mailbox;
+    info->mailbox = mailbox;
     g_object_set_data_full(G_OBJECT(info->mailbox), BALSA_INDEX_PIPE_INFO,
                            info, bndx_mailbox_notify);
 
@@ -2785,16 +2792,17 @@ balsa_index_count_selected_messages(BalsaIndex * bindex)
 void
 balsa_index_select_thread(BalsaIndex * bindex)
 {
+    LibBalsaMailbox *mailbox = balsa_mailbox_node_get_mailbox(bindex->mailbox_node);
+    GtkTreeModel *model = GTK_TREE_MODEL(mailbox);
     GtkTreeIter iter;
-    GtkTreeModel *model = GTK_TREE_MODEL(bindex->mailbox_node->mailbox);
     GtkTreeIter next_iter;
     GtkTreePath *path;
     GtkTreeSelection *selection =
         gtk_tree_view_get_selection(GTK_TREE_VIEW(bindex));
     gboolean valid;
 
-    if (!bindex->current_msgno
-        || !libbalsa_mailbox_msgno_find(bindex->mailbox_node->mailbox,
+    if (bindex->current_msgno == 0
+        || !libbalsa_mailbox_msgno_find(mailbox,
                                         bindex->current_msgno, NULL,
                                         &iter))
         return;
@@ -2826,3 +2834,85 @@ balsa_index_select_thread(BalsaIndex * bindex)
     } while (valid
              && gtk_tree_model_iter_parent(model, &next_iter, &iter));
 }
+
+/*
+ * Getters
+ */
+
+BalsaMailboxNode *
+balsa_index_get_mailbox_node(BalsaIndex *bindex)
+{
+    g_return_val_if_fail(BALSA_IS_INDEX(bindex), NULL);
+
+    return bindex->mailbox_node;
+}
+
+guint
+balsa_index_get_current_msgno(BalsaIndex *bindex)
+{
+    g_return_val_if_fail(BALSA_IS_INDEX(bindex), 0);
+
+    return bindex->current_msgno;
+}
+
+gint
+balsa_index_get_filter_no(BalsaIndex *bindex)
+{
+    g_return_val_if_fail(BALSA_IS_INDEX(bindex), 0);
+
+    return bindex->filter_no;
+}
+
+gboolean
+balsa_index_get_next_message(BalsaIndex *bindex)
+{
+    g_return_val_if_fail(BALSA_IS_INDEX(bindex), 0);
+
+    return bindex->next_message;
+}
+
+gboolean
+balsa_index_get_prev_message(BalsaIndex *bindex)
+{
+    g_return_val_if_fail(BALSA_IS_INDEX(bindex), 0);
+
+    return bindex->prev_message;
+}
+
+const gchar *
+balsa_index_get_filter_string(BalsaIndex *bindex)
+{
+    g_return_val_if_fail(BALSA_IS_INDEX(bindex), NULL);
+
+    return bindex->filter_string;
+}
+
+/*
+ * Convenience
+ */
+
+void
+balsa_index_set_last_use_time(BalsaIndex *bindex)
+{
+    g_return_if_fail(BALSA_IS_INDEX(bindex));
+
+    if (bindex->mailbox_node != NULL)
+        balsa_mailbox_node_set_last_use_time(bindex->mailbox_node);
+}
+
+time_t
+balsa_index_get_last_use_time(BalsaIndex *bindex)
+{
+    g_return_val_if_fail(BALSA_IS_INDEX(bindex), 0);
+
+    return bindex->mailbox_node != NULL ?
+        balsa_mailbox_node_get_last_use_time(bindex->mailbox_node) : 0;
+}
+
+LibBalsaMailbox *
+balsa_index_get_mailbox(BalsaIndex *bindex)
+{
+    g_return_val_if_fail(BALSA_IS_INDEX(bindex), NULL);
+
+    return bindex->mailbox_node != NULL ? balsa_mailbox_node_get_mailbox(bindex->mailbox_node) : NULL;
+}
