@@ -79,10 +79,7 @@ static GtkTargetEntry bmbl_drop_types[] = {
     {"x-application/x-message-list", GTK_TARGET_SAME_APP, TARGET_MESSAGES}
 };
 
-static GtkTreeViewClass *parent_class = NULL;
-
 /* class methods */
-static void bmbl_class_init(BalsaMBListClass * klass);
 static void bmbl_set_property(GObject * object, guint prop_id,
                               const GValue * value, GParamSpec * pspec);
 static void bmbl_get_property(GObject * object, guint prop_id,
@@ -93,7 +90,6 @@ static gboolean bmbl_drag_motion(GtkWidget * mblist,
 static gboolean bmbl_popup_menu(GtkWidget * widget);
 static void bmbl_select_mailbox(GtkTreeSelection * selection,
                                 gpointer data);
-static void bmbl_init(BalsaMBList * mblist);
 static gboolean bmbl_selection_func(GtkTreeSelection * selection,
                                     GtkTreeModel * model,
                                     GtkTreePath * path,
@@ -141,42 +137,27 @@ static void bmbl_expand_to_row(BalsaMBList * mblist, GtkTreePath * path);
 
 /* class methods */
 
-GType
-balsa_mblist_get_type(void)
-{
-    static GType mblist_type = 0;
+struct _BalsaMBList {
+    GtkTreeView tree_view;
 
-    if (!mblist_type) {
-	static const GTypeInfo mblist_info = {
-	    sizeof(BalsaMBListClass),
-            NULL,               /* base_init */
-            NULL,               /* base_finalize */
-	    (GClassInitFunc) bmbl_class_init,
-            NULL,               /* class_finalize */
-            NULL,               /* class_data */
-	    sizeof(BalsaMBList),
-            0,                  /* n_preallocs */
-	    (GInstanceInitFunc) bmbl_init
-	};
+    /* shall the number of messages be displayed ? */
+    gboolean display_info;
+    /* signal handler id */
+    gulong toggled_handler_id;
 
-	mblist_type =
-            g_type_register_static(GTK_TYPE_TREE_VIEW,
-	                           "BalsaMBList",
-                                   &mblist_info, 0);
-    }
-
-    return mblist_type;
-}
+    /* to set sort order in an idle callback */
+    gint  sort_column_id;
+    guint sort_idle_id;
+};
 
+G_DEFINE_TYPE(BalsaMBList, balsa_mblist, GTK_TYPE_TREE_VIEW)
 
 static void
-bmbl_class_init(BalsaMBListClass * klass)
+balsa_mblist_class_init(BalsaMBListClass * klass)
 {
     GObjectClass *object_class;
     GtkWidgetClass *widget_class;
 
-    parent_class = g_type_class_peek_parent(klass);
-
     object_class = (GObjectClass *) klass;
     widget_class = (GtkWidgetClass *) klass;
 
@@ -186,7 +167,7 @@ bmbl_class_init(BalsaMBListClass * klass)
         g_signal_new("has-unread-mailbox",
                      G_TYPE_FROM_CLASS(object_class),
                      G_SIGNAL_RUN_FIRST,
-                     G_STRUCT_OFFSET(BalsaMBListClass, has_unread_mailbox),
+                     0,
                      NULL, NULL,
                      NULL,
                      G_TYPE_NONE,
@@ -200,9 +181,6 @@ bmbl_class_init(BalsaMBListClass * klass)
     widget_class->drag_motion = bmbl_drag_motion;
     widget_class->popup_menu = bmbl_popup_menu;
 
-    /* Signals */
-    klass->has_unread_mailbox = NULL;
-
     /* Properties */
     g_object_class_install_property(object_class, PROP_SHOW_CONTENT_INFO,
                                     g_param_spec_boolean
@@ -310,7 +288,7 @@ bmbl_drag_motion(GtkWidget * mblist, GdkDragContext * context, gint x,
     gboolean can_drop;
 
     ret_val =
-        GTK_WIDGET_CLASS(parent_class)->drag_motion(mblist, context, x, y,
+        GTK_WIDGET_CLASS(balsa_mblist_parent_class)->drag_motion(mblist, context, x, y,
                                                     time);
 
     gtk_tree_view_get_drag_dest_row(tree_view, &path, NULL);
@@ -335,7 +313,7 @@ bmbl_drag_motion(GtkWidget * mblist, GdkDragContext * context, gint x,
  * callbacks
  */
 static void
-bmbl_init(BalsaMBList * mblist)
+balsa_mblist_init(BalsaMBList * mblist)
 {
     GtkTreeStore *store = balsa_mblist_get_store();
     GtkTreeView *tree_view = GTK_TREE_VIEW(mblist);
@@ -426,12 +404,12 @@ bmbl_init(BalsaMBList * mblist)
                                            bmbl_selection_func, NULL,
                                            NULL);
 
-    g_signal_connect_after(G_OBJECT(tree_view), "row-expanded",
+    g_signal_connect_after(tree_view, "row-expanded",
                            G_CALLBACK(bmbl_tree_expand), NULL);
-    g_signal_connect(G_OBJECT(tree_view), "row-collapsed",
+    g_signal_connect(tree_view, "row-collapsed",
                      G_CALLBACK(bmbl_tree_collapse), NULL);
 
-    g_object_set(G_OBJECT(mblist),
+    g_object_set(mblist,
                  "show_content_info",
                  balsa_app.mblist_show_mb_content_info,
                  NULL);
@@ -509,8 +487,12 @@ bmbl_selection_func(GtkTreeSelection * selection, GtkTreeModel * model,
     /* If the node is selected, allow it to be deselected, whether or
      * not it has a mailbox (if it doesn't, it shouldn't have been
      * selected in the first place, but you never know...). */
-    retval = (path_currently_selected || (mbnode && mbnode->mailbox));
+    retval = (path_currently_selected ||
+              (mbnode != NULL &&
+               balsa_mailbox_node_get_mailbox(mbnode) != NULL));
+
     g_object_unref(mbnode);
+
     return retval;
 }
 
@@ -539,7 +521,7 @@ bmbl_tree_expand(GtkTreeView * tree_view, GtkTreeIter * iter,
     gtk_tree_model_get(model, iter, MBNODE_COLUMN, &mbnode, -1);
     balsa_mailbox_node_scan_children(mbnode);
 
-    if (!mbnode->mailbox)
+    if (balsa_mailbox_node_get_mailbox(mbnode) == NULL)
         gtk_tree_store_set(GTK_TREE_STORE(model), iter,
                            ICON_COLUMN,
                            balsa_icon_id(BALSA_PIXMAP_MBOX_DIR_OPEN),
@@ -550,22 +532,25 @@ bmbl_tree_expand(GtkTreeView * tree_view, GtkTreeIter * iter,
 	GtkWidget *current_index =
 	    balsa_window_find_current_index(balsa_app.main_window);
 	LibBalsaMailbox *current_mailbox =
-	    current_index ?
-	    BALSA_INDEX(current_index)->mailbox_node->mailbox :
+	    current_index != NULL ?
+	    balsa_index_get_mailbox(BALSA_INDEX(current_index)):
 	    NULL;
 	gboolean first_mailbox = TRUE;
 
         do {
+            LibBalsaMailbox *mailbox;
+
             gtk_tree_model_get(model, &child_iter,
                                MBNODE_COLUMN, &mbnode, -1);
-            if (mbnode && mbnode->mailbox) {
+            if (mbnode != NULL &&
+                (mailbox = balsa_mailbox_node_get_mailbox(mbnode)) != NULL) {
 		/* Mark only one mailbox as exposed. */
 		if (first_mailbox) {
-		    libbalsa_mailbox_set_exposed(mbnode->mailbox, TRUE);
+		    libbalsa_mailbox_set_exposed(mailbox, TRUE);
 		    first_mailbox = FALSE;
 		} else
-		    libbalsa_mailbox_set_exposed(mbnode->mailbox, FALSE);
-		if (mbnode->mailbox == current_mailbox) {
+		    libbalsa_mailbox_set_exposed(mailbox, FALSE);
+		if (mailbox == current_mailbox) {
 		    GtkTreeSelection *selection =
 			gtk_tree_view_get_selection(tree_view);
 		    g_signal_handlers_block_by_func(selection,
@@ -591,11 +576,12 @@ bmbl_tree_collapse_helper(GtkTreeModel * model, GtkTreeIter * iter)
     if (gtk_tree_model_iter_children(model, &child_iter, iter)) {
         do {
             BalsaMailboxNode *mbnode;
+            LibBalsaMailbox *mailbox;
 
             gtk_tree_model_get(model, &child_iter,
                                MBNODE_COLUMN, &mbnode, -1);
-            if (mbnode->mailbox)
-		libbalsa_mailbox_set_exposed(mbnode->mailbox, FALSE);
+            if ((mailbox = balsa_mailbox_node_get_mailbox(mbnode)) != NULL)
+		libbalsa_mailbox_set_exposed(mailbox, FALSE);
 	    g_object_unref(mbnode);
             bmbl_tree_collapse_helper(model, &child_iter);
         } while (gtk_tree_model_iter_next(model, &child_iter));
@@ -611,7 +597,7 @@ bmbl_tree_collapse(GtkTreeView * tree_view, GtkTreeIter * iter,
 
     gtk_tree_model_get(model, iter, MBNODE_COLUMN, &mbnode, -1);
 
-    if (!mbnode->mailbox)
+    if (balsa_mailbox_node_get_mailbox(mbnode) == NULL)
         gtk_tree_store_set(GTK_TREE_STORE(model), iter,
                            ICON_COLUMN,
                            balsa_icon_id(BALSA_PIXMAP_MBOX_DIR_CLOSED),
@@ -644,12 +630,12 @@ bmbl_row_compare(GtkTreeModel * model, GtkTreeIter * iter1,
 
     gtk_tree_model_get(model, iter1,
                        MBNODE_COLUMN, &mbnode, NAME_COLUMN, &name1, -1);
-    m1 = mbnode->mailbox;
+    m1 = balsa_mailbox_node_get_mailbox(mbnode);
     g_object_unref(mbnode);
 
     gtk_tree_model_get(model, iter2,
                        MBNODE_COLUMN, &mbnode, NAME_COLUMN, &name2, -1);
-    m2 = mbnode->mailbox;
+    m2 = balsa_mailbox_node_get_mailbox(mbnode);
     g_object_unref(mbnode);
 
     switch (sort_column) {
@@ -840,7 +826,7 @@ bmbl_drag_cb(GtkWidget * widget, GdkDragContext * context,
         return;
     }
 
-    orig_mailbox = orig_index->mailbox_node->mailbox;
+    orig_mailbox = balsa_index_get_mailbox(orig_index);
 
     /* find the node and mailbox */
 
@@ -854,7 +840,7 @@ bmbl_drag_cb(GtkWidget * widget, GdkDragContext * context,
 
         gtk_tree_model_get_iter(model, &iter, path);
         gtk_tree_model_get(model, &iter, MBNODE_COLUMN, &mbnode, -1);
-        mailbox = mbnode->mailbox;
+        mailbox = balsa_mailbox_node_get_mailbox(mbnode);
 	g_object_unref(mbnode);
 
         /* cannot transfer to the originating mailbox */
@@ -893,7 +879,7 @@ bmbl_select_mailbox(GtkTreeSelection * selection, gpointer data)
 	    BalsaMailboxNode *mbnode;
 	    LibBalsaMailbox *mailbox;
 	    gtk_tree_model_get(model, &iter, MBNODE_COLUMN, &mbnode, -1);
-	    mailbox = mbnode->mailbox;
+	    mailbox = balsa_mailbox_node_get_mailbox(mbnode);
 	    g_object_unref(mbnode);
 	    if (MAILBOX_OPEN(mailbox))
 		/* Opening a mailbox under program control. */
@@ -931,15 +917,15 @@ bmbl_select_mailbox(GtkTreeSelection * selection, gpointer data)
     }
 
     if (gtk_tree_selection_path_is_selected(selection, path)) {
-        BalsaMailboxNode *mbnode;
         GtkTreeIter iter;
+        BalsaMailboxNode *mbnode;
+        LibBalsaMailbox *mailbox;
 
         gtk_tree_model_get_iter(model, &iter, path);
         gtk_tree_model_get(model, &iter, MBNODE_COLUMN, &mbnode, -1);
-        g_return_if_fail(mbnode != NULL);
 
-        if (mbnode->mailbox)
-            balsa_mblist_open_mailbox(mbnode->mailbox);
+        if ((mailbox = balsa_mailbox_node_get_mailbox(mbnode)) != NULL)
+            balsa_mblist_open_mailbox(mailbox);
 	g_object_unref(mbnode);
     }
     gtk_tree_path_free(path);
@@ -961,13 +947,14 @@ bmbl_row_activated_cb(GtkTreeView * tree_view, GtkTreePath * path,
     GtkTreeModel *model = gtk_tree_view_get_model(tree_view);
     GtkTreeIter iter;
     BalsaMailboxNode *mbnode;
+    LibBalsaMailbox *mailbox;
 
     gtk_tree_model_get_iter(model, &iter, path);
     gtk_tree_model_get(model, &iter, MBNODE_COLUMN, &mbnode, -1);
     g_return_if_fail(mbnode != NULL);
 
-    if (mbnode->mailbox)
-        balsa_mblist_open_mailbox(mbnode->mailbox);
+    if ((mailbox = balsa_mailbox_node_get_mailbox(mbnode)) != NULL)
+        balsa_mblist_open_mailbox(mailbox);
     g_object_unref(mbnode);
 }
 
@@ -1030,6 +1017,7 @@ static void
 bmbl_mailbox_changed_cb(LibBalsaMailbox * mailbox, gpointer data)
 {
     struct update_mbox_data *umd;
+    LibBalsaMailboxState state;
 
     g_return_if_fail(LIBBALSA_IS_MAILBOX(mailbox));
 
@@ -1046,8 +1034,9 @@ bmbl_mailbox_changed_cb(LibBalsaMailbox * mailbox, gpointer data)
         g_idle_add((GSourceFunc) update_mailbox_idle, umd);
     }
 
-    umd->notify = (mailbox->state == LB_MAILBOX_STATE_OPEN
-                   || mailbox->state == LB_MAILBOX_STATE_CLOSED);
+    state = libbalsa_mailbox_get_state(mailbox);
+    umd->notify = (state == LB_MAILBOX_STATE_OPEN
+                   || state == LB_MAILBOX_STATE_CLOSED);
 
     G_UNLOCK(mblist_update);
 }
@@ -1092,7 +1081,7 @@ bmbl_find_all_unread_mboxes_func(GtkTreeModel * model, GtkTreePath * path,
     gtk_tree_model_get(model, iter, MBNODE_COLUMN, &mbnode, -1);
     if(!mbnode) /* this node has no MBNODE associated at this time */
         return FALSE;
-    mailbox = mbnode->mailbox;
+    mailbox = balsa_mailbox_node_get_mailbox(mbnode);
     g_object_unref(mbnode);
 
     if (mailbox
@@ -1135,30 +1124,29 @@ static void
 bmbl_open_mailbox(LibBalsaMailbox * mailbox, gboolean set_current)
 {
     int i;
-    GtkWidget *index;
+    GtkWidget *bindex;
     BalsaMailboxNode *mbnode;
 
     mbnode = balsa_find_mailbox(mailbox);
-    if (!mbnode) {
+    if (mbnode == NULL) {
         g_warning(_("Failed to find mailbox"));
         return;
     }
 
-    index = balsa_window_find_current_index(balsa_app.main_window);
+    bindex = balsa_window_find_current_index(balsa_app.main_window);
 
     /* If we currently have a page open, update the time last visited */
-    if (index) {
-	time(&BALSA_INDEX(index)->mailbox_node->last_use);
-    }
+    if (bindex != NULL)
+        balsa_index_set_last_use_time(BALSA_INDEX(bindex));
 
     i = balsa_find_notebook_page_num(mailbox);
     if (i != -1) {
         if (set_current) {
             gtk_notebook_set_current_page(GTK_NOTEBOOK(balsa_app.notebook),
                                           i);
-            index = balsa_window_find_current_index(balsa_app.main_window);
-            time(&BALSA_INDEX(index)->mailbox_node->last_use);
-            balsa_index_set_column_widths(BALSA_INDEX(index));
+            bindex = balsa_window_find_current_index(balsa_app.main_window);
+            balsa_index_set_last_use_time(BALSA_INDEX(bindex));
+            balsa_index_set_column_widths(BALSA_INDEX(bindex));
         }
     } else { /* page with mailbox not found, open it */
         balsa_window_open_mbnode(balsa_app.main_window, mbnode,
@@ -1212,19 +1200,23 @@ get_lru_descendant(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter,
 {
     struct lru_data *dt  = (struct lru_data*)data;
     BalsaMailboxNode *mbnode;
+    LibBalsaMailbox *mailbox;
 
-    if(!gtk_tree_path_is_descendant(path, dt->ancestor_path))
+    if (!gtk_tree_path_is_descendant(path, dt->ancestor_path))
         return FALSE;
+
     gtk_tree_model_get(model, iter, MBNODE_COLUMN, &mbnode, -1);
-    if(mbnode->mailbox && libbalsa_mailbox_is_open(mbnode->mailbox) &&
-       (!dt->mbnode || (mbnode->last_use < dt->mbnode->last_use)) )
-    {
-        if (dt->mbnode)
-            g_object_unref(dt->mbnode);
-        dt->mbnode = mbnode;
+
+    if ((mailbox = balsa_mailbox_node_get_mailbox(mbnode)) != NULL &&
+        libbalsa_mailbox_is_open(mailbox) &&
+        (dt->mbnode == NULL ||
+         (balsa_mailbox_node_get_last_use_time(mbnode) <
+          balsa_mailbox_node_get_last_use_time(dt->mbnode)))) {
+        g_set_object(&dt->mbnode, mbnode);
     }
 
-    else g_object_unref(mbnode);
+    g_object_unref(mbnode);
+
     return FALSE;
 }
 
@@ -1263,9 +1255,9 @@ balsa_mblist_default_signal_bindings(BalsaMBList * mblist)
 {
     GtkTreeSelection *selection;
 
-    g_signal_connect(G_OBJECT(mblist), "button_press_event",
+    g_signal_connect(mblist, "button_press_event",
                      G_CALLBACK(bmbl_button_press_cb), NULL);
-    g_signal_connect_after(G_OBJECT(mblist), "size-allocate",
+    g_signal_connect_after(mblist, "size-allocate",
                            G_CALLBACK(bmbl_column_resize), NULL);
     gtk_tree_view_enable_model_drag_dest(GTK_TREE_VIEW(mblist),
                                          bmbl_drop_types,
@@ -1273,13 +1265,13 @@ balsa_mblist_default_signal_bindings(BalsaMBList * mblist)
                                          GDK_ACTION_DEFAULT |
                                          GDK_ACTION_COPY |
                                          GDK_ACTION_MOVE);
-    g_signal_connect(G_OBJECT(mblist), "drag-data-received",
+    g_signal_connect(mblist, "drag-data-received",
                      G_CALLBACK(bmbl_drag_cb), NULL);
 
     selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(mblist));
-    g_signal_connect(G_OBJECT(selection), "changed",
+    g_signal_connect(selection, "changed",
                      G_CALLBACK(bmbl_select_mailbox), NULL);
-    g_signal_connect(G_OBJECT(mblist), "row-activated",
+    g_signal_connect(mblist, "row-activated",
                      G_CALLBACK(bmbl_row_activated_cb), NULL);
 }
 
@@ -1291,8 +1283,10 @@ static void
 bmbl_real_disconnect_mbnode_signals(BalsaMailboxNode * mbnode,
 				    GtkTreeModel * model)
 {
-    if (mbnode->mailbox)
-        g_signal_handlers_disconnect_by_func(mbnode->mailbox,
+    LibBalsaMailbox *mailbox;
+
+    if ((mailbox = balsa_mailbox_node_get_mailbox(mbnode)) != NULL)
+        g_signal_handlers_disconnect_by_func(mailbox,
                                              G_CALLBACK
                                              (bmbl_mailbox_changed_cb),
                                              NULL);
@@ -1306,15 +1300,13 @@ bmbl_real_disconnect_mbnode_signals(BalsaMailboxNode * mbnode,
 static gboolean
 bmbl_store_redraw_mbnode(GtkTreeIter * iter, BalsaMailboxNode * mbnode)
 {
+    LibBalsaMailbox *mailbox;
     const gchar *icon;
     const gchar *name;
     gchar *tmp = NULL;
     gboolean expose = FALSE;
 
-    g_return_val_if_fail(mbnode, FALSE);
-
-    if (mbnode->mailbox) {
-        LibBalsaMailbox *mailbox = mbnode->mailbox;
+    if ((mailbox = balsa_mailbox_node_get_mailbox(mbnode)) != NULL) {
 	static guint mailbox_changed_signal = 0;
 
 	if (LIBBALSA_IS_MAILBOX_POP3(mailbox)) {
@@ -1337,7 +1329,7 @@ bmbl_store_redraw_mbnode(GtkTreeIter * iter, BalsaMailboxNode * mbnode)
 		? BALSA_PIXMAP_MBOX_TRAY_FULL
                 : BALSA_PIXMAP_MBOX_TRAY_EMPTY;
 
-            name = mailbox->name;
+            name = libbalsa_mailbox_get_name(mailbox);
 
             /* Make sure the show column is set before showing the
              * mailbox in the list. */
@@ -1361,10 +1353,10 @@ bmbl_store_redraw_mbnode(GtkTreeIter * iter, BalsaMailboxNode * mbnode)
 	if (!mailbox_changed_signal)
 	    mailbox_changed_signal =
 		g_signal_lookup("changed", LIBBALSA_TYPE_MAILBOX);
-	if (!g_signal_has_handler_pending(G_OBJECT(mbnode->mailbox),
+	if (!g_signal_has_handler_pending(mailbox,
                                           mailbox_changed_signal, 0, TRUE)) {
 	    /* Now we have a mailbox: */
-	    g_signal_connect(mbnode->mailbox, "changed",
+	    g_signal_connect(mailbox, "changed",
 			     G_CALLBACK(bmbl_mailbox_changed_cb),
 			     NULL);
             if (libbalsa_mailbox_get_unread(mailbox) > 0
@@ -1375,12 +1367,12 @@ bmbl_store_redraw_mbnode(GtkTreeIter * iter, BalsaMailboxNode * mbnode)
                               0, TRUE);
 	    /* If necessary, expand rows to expose this mailbox after
 	     * setting its mbnode in the tree-store. */
-	    expose = libbalsa_mailbox_get_exposed(mbnode->mailbox);
+	    expose = libbalsa_mailbox_get_exposed(mailbox);
 	}
     } else {
 	/* new directory, but not a mailbox */
 	icon = BALSA_PIXMAP_MBOX_DIR_CLOSED;
-        name = tmp = g_path_get_basename(mbnode->name);
+        name = tmp = g_path_get_basename(balsa_mailbox_node_get_name(mbnode));
     }
 
     gtk_tree_store_set(balsa_app.mblist_tree_store, iter,
@@ -1394,7 +1386,7 @@ bmbl_store_redraw_mbnode(GtkTreeIter * iter, BalsaMailboxNode * mbnode)
                        -1);
     g_free(tmp);
 
-    if (mbnode->mailbox) {
+    if (mailbox != NULL) {
 	GtkTreeModel *model = GTK_TREE_MODEL(balsa_app.mblist_tree_store);
 	if (expose) {
 	    GtkTreePath *path = gtk_tree_model_get_path(model, iter);
@@ -1432,7 +1424,8 @@ bmbl_update_mailbox(GtkTreeStore * store, LibBalsaMailbox * mailbox)
     bmbl_node_style(model, &iter);
 
     bindex = balsa_window_find_current_index(balsa_app.main_window);
-    if (!bindex || mailbox != BALSA_INDEX(bindex)->mailbox_node->mailbox)
+    if (bindex == NULL ||
+        mailbox != balsa_index_get_mailbox(BALSA_INDEX(bindex)))
         return;
 
     balsa_window_set_statusbar(balsa_app.main_window, mailbox);
@@ -1468,10 +1461,10 @@ bmbl_node_style(GtkTreeModel * model, GtkTreeIter * iter)
     gchar *text_total = NULL;
 
     gtk_tree_model_get(model, iter, MBNODE_COLUMN, &mbnode, -1);
-    if (!mbnode || !mbnode->mailbox)
+    if (mbnode == NULL ||
+        (mailbox = balsa_mailbox_node_get_mailbox(mbnode)) == NULL)
         return;
 
-    mailbox = mbnode->mailbox;
     unread_messages = libbalsa_mailbox_get_unread(mailbox);
     total_messages = libbalsa_mailbox_get_total(mailbox);
 
@@ -1480,12 +1473,14 @@ bmbl_node_style(GtkTreeModel * model, GtkTreeIter * iter)
           mailbox == balsa_app.draftbox || mailbox == balsa_app.trash)) {
         const gchar *icon;
         const gchar *name;
+        const gchar *mailbox_name;
         gchar *tmp = NULL;
         PangoWeight weight;
 
         /* Set the style appropriate for unread_messages; we do this
          * even if the state hasn't changed, because we might be
          * rerendering after hiding or showing the info columns. */
+        mailbox_name = libbalsa_mailbox_get_name(mailbox);
         if (unread_messages > 0) {
             gboolean display_info;
 
@@ -1495,18 +1490,18 @@ bmbl_node_style(GtkTreeModel * model, GtkTreeIter * iter)
                                            (G_OBJECT(model),
                                             BALSA_MBLIST_DISPLAY_INFO));
             name = (!display_info && total_messages >= 0) ?
-                (tmp = g_strdup_printf("%s (%d)", mailbox->name,
+                (tmp = g_strdup_printf("%s (%d)", mailbox_name,
                                       unread_messages))
-                : mailbox->name;
+                : mailbox_name;
 
             weight = PANGO_WEIGHT_BOLD;
-            mbnode->style |= MBNODE_STYLE_NEW_MAIL;
+            balsa_mailbox_node_change_style(mbnode, MBNODE_STYLE_NEW_MAIL, 0);
         } else {
             icon = (mailbox == balsa_app.inbox) ?
                 BALSA_PIXMAP_MBOX_IN : BALSA_PIXMAP_MBOX_TRAY_EMPTY;
-            name = mailbox->name;
+            name = mailbox_name;
             weight = PANGO_WEIGHT_NORMAL;
-            mbnode->style &= ~MBNODE_STYLE_NEW_MAIL;
+            balsa_mailbox_node_change_style(mbnode, 0, MBNODE_STYLE_NEW_MAIL);
         }
 
         gtk_tree_store_set(GTK_TREE_STORE(model), iter,
@@ -1546,18 +1541,18 @@ bmbl_node_style(GtkTreeModel * model, GtkTreeIter * iter)
 	    do {
 		BalsaMailboxNode *mn;
 		gtk_tree_model_get(model, &child, MBNODE_COLUMN, &mn, -1);
-		if (mn->style & (MBNODE_STYLE_NEW_MAIL |
-				 MBNODE_STYLE_UNREAD_CHILD))
+		if ((balsa_mailbox_node_get_style(mn) &
+                     (MBNODE_STYLE_NEW_MAIL | MBNODE_STYLE_UNREAD_CHILD)) != 0)
 		    has_unread_child = TRUE;
 		g_object_unref(mn);
 	    } while (!has_unread_child && gtk_tree_model_iter_next(model, &child));
 	}
 	if (has_unread_child) {
-	    mbnode->style |= MBNODE_STYLE_UNREAD_CHILD;
+	    balsa_mailbox_node_change_style(mbnode, MBNODE_STYLE_UNREAD_CHILD, 0);
 	    gtk_tree_store_set(GTK_TREE_STORE(model), &parent,
 			       STYLE_COLUMN, PANGO_STYLE_OBLIQUE, -1);
 	} else {
-	    mbnode->style &= ~MBNODE_STYLE_UNREAD_CHILD;
+	    balsa_mailbox_node_change_style(mbnode, 0, MBNODE_STYLE_UNREAD_CHILD);
 	    gtk_tree_store_set(GTK_TREE_STORE(model), &parent,
 			       STYLE_COLUMN, PANGO_STYLE_NORMAL, -1);
 	}
@@ -1751,7 +1746,7 @@ bmbl_mru_menu(GtkWindow * window, GList ** url_list,
         if (mailbox || (allow_empty && !*url)) {
             mru = bmbl_mru_new(url_list, user_func, user_data, url);
             item =
-                gtk_menu_item_new_with_label(mailbox ? mailbox->name : "");
+                gtk_menu_item_new_with_label(mailbox ? libbalsa_mailbox_get_name(mailbox) : "");
             gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
             g_signal_connect_data(item, "activate",
                                   G_CALLBACK(bmbl_mru_activate_cb), mru,
@@ -1948,7 +1943,8 @@ bmbl_mru_selected_cb(GtkTreeSelection * selection, gpointer data)
         model = gtk_tree_view_get_model(tree_view);
         gtk_tree_model_get_iter(model, &iter, path);
         gtk_tree_model_get(model, &iter, 0, &mbnode, -1);
-        ((BalsaMBListMRUEntry *) data)->url = g_strdup(mbnode->mailbox->url);
+        ((BalsaMBListMRUEntry *) data)->url =
+            g_strdup(libbalsa_mailbox_get_url(balsa_mailbox_node_get_mailbox(mbnode)));
 	g_object_unref(mbnode);
         bmbl_mru_activate_cb(NULL, data);
 
@@ -1975,14 +1971,15 @@ bmbl_mru_activated_cb(GtkTreeView * tree_view, GtkTreePath * path,
     GtkTreeModel *model = gtk_tree_view_get_model(tree_view);
     GtkTreeIter iter;
     BalsaMailboxNode *mbnode;
+    LibBalsaMailbox *mailbox;
 
     gtk_tree_model_get_iter(model, &iter, path);
     gtk_tree_model_get(model, &iter, MBNODE_COLUMN, &mbnode, -1);
     g_return_if_fail(mbnode != NULL);
 
-    if (mbnode->mailbox) {
+    if ((mailbox = balsa_mailbox_node_get_mailbox(mbnode)) != NULL) {
         ((BalsaMBListMRUEntry *) data)->url =
-            g_strdup(mbnode->mailbox->url);
+            g_strdup(libbalsa_mailbox_get_url(mailbox));
         bmbl_mru_activate_cb(NULL, data);
 
         gtk_dialog_response(GTK_DIALOG
@@ -2056,8 +2053,7 @@ bmbl_mru_combo_box_setup(GtkComboBox * combo_box)
     gtk_combo_box_set_active(combo_box, -1);
     store = GTK_LIST_STORE(gtk_combo_box_get_model(combo_box));
     gtk_list_store_clear(store);
-    g_slist_foreach(mro->real_urls, (GFunc) g_free, NULL);
-    g_slist_free(mro->real_urls);
+    g_slist_free_full(mro->real_urls, g_free);
     mro->real_urls = NULL;
 
     for (list = *mro->url_list; list; list = list->next) {
@@ -2114,8 +2110,7 @@ bmbl_mru_combo_box_changed(GtkComboBox * combo_box,
 static void
 bmbl_mru_combo_box_destroy_cb(BalsaMBListMRUOption * mro)
 {
-    g_slist_foreach(mro->real_urls, (GFunc) g_free, NULL);
-    g_slist_free(mro->real_urls);
+    g_slist_free_full(mro->real_urls, g_free);
     g_free(mro);
 }
 
@@ -2171,7 +2166,7 @@ balsa_mblist_mru_option_menu(GtkWindow * window, GList ** url_list)
     g_object_set_data_full(G_OBJECT(combo_box), "mro", mro,
                            (GDestroyNotify) bmbl_mru_combo_box_destroy_cb);
     bmbl_mru_combo_box_setup(GTK_COMBO_BOX(combo_box));
-    g_signal_connect(G_OBJECT(combo_box), "changed",
+    g_signal_connect(combo_box, "changed",
                      G_CALLBACK(bmbl_mru_combo_box_changed), mro);
 
     return combo_box;
@@ -2282,11 +2277,9 @@ balsa_mblist_mailbox_node_append(BalsaMailboxNode * root,
         GtkTreePath *parent_path =
             gtk_tree_model_get_path(model, parent_iter);
         if (gtk_tree_view_row_expanded(GTK_TREE_VIEW(balsa_app.mblist),
-                                       parent_path)
-            && !mbnode->scanned) {
+                                       parent_path)) {
             /* Check this node for children. */
             balsa_mailbox_node_append_subtree(mbnode);
-            mbnode->scanned = TRUE;
         }
         gtk_tree_path_free(parent_path);
     }
