@@ -29,15 +29,9 @@
 
 G_BEGIN_DECLS
 
+#define BALSA_TYPE_MESSAGE (balsa_message_get_type ())
 
-#define BALSA_TYPE_MESSAGE          (balsa_message_get_type ())
-#define BALSA_MESSAGE(obj)          G_TYPE_CHECK_INSTANCE_CAST (obj, BALSA_TYPE_MESSAGE, BalsaMessage)
-#define BALSA_MESSAGE_CLASS(klass)  G_TYPE_CHECK_CLASS_CAST (klass, BALSA_TYPE_MESSAGE, BalsaMessageClass)
-#define BALSA_IS_MESSAGE(obj)       G_TYPE_CHECK_INSTANCE_TYPE (obj, BALSA_TYPE_MESSAGE)
-
-
-typedef struct _BalsaMessage BalsaMessage;
-typedef struct _BalsaMessageClass BalsaMessageClass;
+G_DECLARE_FINAL_TYPE(BalsaMessage, balsa_message, BALSA, MESSAGE, GtkBox)
 
 typedef struct _BalsaPartInfo BalsaPartInfo;
 
@@ -49,65 +43,6 @@ typedef enum {
     BALSA_MESSAGE_FOCUS_STATE_HOLD
 } BalsaMessageFocusState;
 
-struct _BalsaMessage {
-        GtkBox parent;
-
-        GtkWidget *stack;
-        GtkWidget *switcher;
-
-        /* Top-level MIME widget */
-        BalsaMimeWidget *bm_widget;
-
-	/* header-related information */
-	ShownHeaders shown_headers;
-
-	/* Widgets to hold content */
-	GtkWidget *scroll;
-
-        /* Widget to hold structure tree */
-        GtkWidget *treeview;
-        gint info_count;
-        GList *save_all_list;
-        GtkWidget *save_all_popup;
-    
-	gboolean wrap_text;
-
-        BalsaPartInfo *current_part;
-        GtkWidget *parts_popup;
-        gboolean force_inline;
-
-	LibBalsaMessage *message;
-
-        BalsaMessageFocusState focus_state;
-
-        /* Find-in-message stuff */
-        GtkWidget  *find_bar;
-        GtkWidget  *find_entry;
-        GtkWidget  *find_next;
-        GtkWidget  *find_prev;
-        GtkWidget  *find_sep;
-        GtkWidget  *find_label;
-        GtkTextIter find_iter;
-        gboolean    find_forward;
-
-        /* Tab position for headers */
-        gint tab_position;
-
-        /* Widget to hold Faces */
-        GtkWidget *face_box;
-
-#ifdef HAVE_HTML_WIDGET
-        gpointer html_find_info;
-#endif				/* HAVE_HTML_WIDGET */
-};
-
-struct _BalsaMessageClass {
-	GtkNotebookClass parent_class;
-
-	void (*select_part) (BalsaMessage * message);
-};
-
-GType balsa_message_get_type(void);
 GtkWidget *balsa_message_new(void);
 
 gboolean balsa_message_set(BalsaMessage * bmessage,
@@ -136,15 +71,32 @@ gboolean balsa_message_can_zoom(BalsaMessage * bm);
 void balsa_message_zoom(BalsaMessage * bm, gint in_out);
 #endif				/* HAVE_HTML_WIDGET */
 
-#ifdef HAVE_GPGME
 void balsa_message_perform_crypto(LibBalsaMessage * message,
 				  LibBalsaChkCryptoMode chk_mode,
 				  gboolean no_mp_signed,
 				  guint max_ref);
-#endif
 
 void balsa_message_find_in_message (BalsaMessage * bm);
 
+/*
+ * Getters
+ */
+gboolean balsa_message_get_wrap_text(BalsaMessage *bm);
+BalsaMessageFocusState balsa_message_get_focus_state(BalsaMessage *bm);
+GtkScrolledWindow * balsa_message_get_scroll(BalsaMessage *bm);
+BalsaMimeWidget * balsa_message_get_bm_widget(BalsaMessage *bm);
+LibBalsaMessage * balsa_message_get_message(BalsaMessage *bm);
+ShownHeaders balsa_message_get_shown_headers(BalsaMessage *bm);
+GtkWidget * balsa_message_get_face_box(BalsaMessage *bm);
+GtkWidget * balsa_message_get_tree_view(BalsaMessage *bm);
+
+/*
+ * Setters
+ */
+void balsa_message_set_focus_state(BalsaMessage *bm,
+                                   BalsaMessageFocusState focus_state);
+void balsa_message_set_face_box(BalsaMessage *bm, GtkWidget *face_box);
+
 G_END_DECLS
 
 #endif				/* __BALSA_MESSAGE_H__ */
