@@ -267,7 +267,7 @@ balsa_information_list(GtkWindow *parent, LibBalsaInformationType type,
 	information_list = balsa_information_list_new();
 	gtk_container_add(GTK_CONTAINER(scrolled_window),
 			  information_list);
-        g_signal_connect(G_OBJECT(information_dialog), "response",
+        g_signal_connect(information_dialog, "response",
                          G_CALLBACK(balsa_information_list_response_cb),
                          information_list);
 
@@ -289,7 +289,7 @@ balsa_information_list(GtkWindow *parent, LibBalsaInformationType type,
         GtkStatusbar *statusbar;
         guint context_id;
 
-        statusbar = GTK_STATUSBAR(balsa_app.main_window->statusbar);
+        statusbar = balsa_window_get_statusbar(balsa_app.main_window);
         context_id = gtk_statusbar_get_context_id(statusbar, "Information list");
         gtk_statusbar_pop(statusbar, context_id);
 
@@ -308,7 +308,7 @@ status_bar_refresh(gpointer data)
         GtkStatusbar *statusbar;
         guint context_id;
 
-        statusbar = GTK_STATUSBAR(balsa_app.main_window->statusbar);
+        statusbar = balsa_window_get_statusbar(balsa_app.main_window);
         context_id = gtk_statusbar_get_context_id(statusbar, "Information bar");
         gtk_statusbar_pop(statusbar, context_id);
     }
@@ -329,7 +329,7 @@ balsa_information_bar(GtkWindow *parent, LibBalsaInformationType type,
     if (!balsa_app.main_window)
         return;
 
-    statusbar = GTK_STATUSBAR(balsa_app.main_window->statusbar);
+    statusbar = balsa_window_get_statusbar(balsa_app.main_window);
     context_id = gtk_statusbar_get_context_id(statusbar, "Information bar");
 
     /* First clear any current message. */
