@@ -41,7 +41,6 @@
    Folders can additionally scan associated directory or IMAP server to
    retrieve their tree of mailboxes.
 */
-static GObjectClass *parent_class = NULL;
 
 typedef struct imap_scan_item_ imap_scan_item;
 struct imap_scan_item_ {
@@ -50,9 +49,6 @@ struct imap_scan_item_ {
     unsigned scanned:1, selectable:1, marked:1;
 };
 
-static void balsa_mailbox_node_class_init(BalsaMailboxNodeClass *
-					     klass);
-static void balsa_mailbox_node_init(BalsaMailboxNode * mn);
 static void balsa_mailbox_node_dispose(GObject * object);
 static void balsa_mailbox_node_finalize(GObject * object);
 
@@ -77,15 +73,13 @@ static BalsaMailboxNode *add_local_folder(BalsaMailboxNode * root,
 					  const char *d_name,
 					  const char *fn);
 
-static void handle_imap_path(const char *fn, char delim, int noselect,
+static void handle_imap_path(const char *fn, int delim, int noselect,
 			     int noscan, int marked, void *data);
 static gint check_imap_path(const char *fn, LibBalsaServer * server,
 			    guint depth);
 static void mark_imap_path(const gchar * fn, gpointer data);
 
 enum {
-    SAVE_CONFIG,
-    LOAD_CONFIG,
     SHOW_PROP_DIALOG,
     APPEND_SUBTREE,
     LAST_SIGNAL
@@ -93,82 +87,47 @@ enum {
 
 static guint balsa_mailbox_node_signals[LAST_SIGNAL];
 
-GType
-balsa_mailbox_node_get_type(void)
-{
-    static GType mailbox_node_type = 0;
+struct _BalsaMailboxNode {
+    GObject object;
+    BalsaMailboxNode *parent; /* NULL for root-level folders & mailboxes */
+    LibBalsaMailbox *mailbox; /* != NULL for leaves only */
+    gchar *name;       /* used for folders, i.e. when mailbox == NULL */
+    time_t last_use;   /* for closing least recently used mailboxes */
+    BalsaMailboxNodeStyle style;
+    /* folder data */
+    gchar* config_prefix;
+    gchar* dir;
+    LibBalsaServer * server; /* Used only by remote; is referenced */
+    int delim; /* IMAP delimiter so that we do not need to check it
+		 * too often. */
 
-    if (!mailbox_node_type) {
-	static const GTypeInfo mailbox_node_info = {
-	    sizeof(BalsaMailboxNodeClass),
-            NULL,               /* base_init */
-            NULL,               /* base_finalize */
-	    (GClassInitFunc)  balsa_mailbox_node_class_init,
-            NULL,               /* class_finalize */
-            NULL,               /* class_data */
-	    sizeof(BalsaMailboxNode),
-            0,                  /* n_preallocs */
-	    (GInstanceInitFunc) balsa_mailbox_node_init
-	};
+    unsigned subscribed:1;     /* Used only by remote */
+    unsigned list_inbox:1;     /* Used only by remote */
+    unsigned scanned:1;        /* IMAP flag */
+};
 
-	mailbox_node_type =
-	    g_type_register_static(G_TYPE_OBJECT, "BalsaMailboxNode",
-                                   &mailbox_node_info, 0);
-    }
-    
-    return mailbox_node_type;
-}
+G_DEFINE_TYPE(BalsaMailboxNode, balsa_mailbox_node, G_TYPE_OBJECT)
 
 static void
 balsa_mailbox_node_class_init(BalsaMailboxNodeClass * klass)
 {
     GObjectClass *object_class;
 
-    parent_class = g_type_class_peek_parent(klass);
-
     object_class = G_OBJECT_CLASS(klass);
 
-    balsa_mailbox_node_signals[SAVE_CONFIG] =
-	g_signal_new("save-config",
-                     G_TYPE_FROM_CLASS(object_class),
-                     G_SIGNAL_RUN_FIRST,
-                     G_STRUCT_OFFSET(BalsaMailboxNodeClass, save_config),
-                     NULL, NULL,
-                     NULL,
-		     G_TYPE_NONE, 1,
-	             G_TYPE_POINTER);
-    balsa_mailbox_node_signals[LOAD_CONFIG] =
-	g_signal_new("load-config",
-                     G_TYPE_FROM_CLASS(object_class),
-                     G_SIGNAL_RUN_FIRST,
-                     G_STRUCT_OFFSET(BalsaMailboxNodeClass, load_config),
-                     NULL, NULL,
-                     NULL,
-		     G_TYPE_NONE, 1,
-	             G_TYPE_POINTER);
     balsa_mailbox_node_signals[SHOW_PROP_DIALOG] =
 	g_signal_new("show-prop-dialog",
                      G_TYPE_FROM_CLASS(object_class),
                      G_SIGNAL_RUN_LAST,
-                     G_STRUCT_OFFSET(BalsaMailboxNodeClass,
-                                     show_prop_dialog),
-                     NULL, NULL,
-                     NULL,
+                     0, NULL, NULL, NULL,
 		     G_TYPE_NONE, 0);
     balsa_mailbox_node_signals[APPEND_SUBTREE] =
 	g_signal_new("append-subtree",
                      G_TYPE_FROM_CLASS(object_class),
                      G_SIGNAL_RUN_FIRST,
-                     G_STRUCT_OFFSET(BalsaMailboxNodeClass, append_subtree),
-                     NULL, NULL,
-                     NULL,
+                     0, NULL, NULL, NULL,
 		     G_TYPE_NONE, 0);
 
-    klass->save_config = balsa_mailbox_node_real_save_config;
-    klass->load_config = balsa_mailbox_node_real_load_config;
-    klass->show_prop_dialog = NULL;
-    klass->append_subtree   = NULL;
-
     object_class->dispose = balsa_mailbox_node_dispose;
     object_class->finalize = balsa_mailbox_node_finalize;
 }
@@ -192,10 +151,11 @@ balsa_mailbox_node_dispose(GObject * object)
     BalsaMailboxNode *mn = BALSA_MAILBOX_NODE(object);
     LibBalsaMailbox *mailbox = mn->mailbox;
 
-    if (mailbox) {
+    if (mailbox != NULL) {
         if (!LIBBALSA_IS_MAILBOX_POP3(mailbox)) {
             libbalsa_mailbox_set_open(mailbox, libbalsa_mailbox_is_open(mailbox));
-            config_save_mailbox_view(mailbox->url, mailbox->view);
+            config_save_mailbox_view(libbalsa_mailbox_get_url(mailbox),
+                                     libbalsa_mailbox_get_view(mailbox));
             if (balsa_app.main_window != NULL)
                 balsa_window_close_mbnode(balsa_app.main_window, mn);
         }
@@ -203,7 +163,7 @@ balsa_mailbox_node_dispose(GObject * object)
 	mn->mailbox = NULL;
     }
 
-    G_OBJECT_CLASS(parent_class)->dispose(object);
+    G_OBJECT_CLASS(balsa_mailbox_node_parent_class)->dispose(object);
 }
 
 static void
@@ -226,7 +186,7 @@ balsa_mailbox_node_finalize(GObject * object)
 	mn->server = NULL;
     }
 
-    G_OBJECT_CLASS(parent_class)->finalize(G_OBJECT(object));
+    G_OBJECT_CLASS(balsa_mailbox_node_parent_class)->finalize(object);
 }
 
 static void
@@ -402,8 +362,10 @@ load_mailbox_view(BalsaMailboxNode * mbnode)
 {
     LibBalsaMailbox *mailbox = mbnode->mailbox;
 
-    if (!mailbox->view)
-        mailbox->view = config_load_mailbox_view(mailbox->url);
+    if (!libbalsa_mailbox_get_view(mailbox))
+        libbalsa_mailbox_set_view(mailbox,
+                                  config_load_mailbox_view
+                                  (libbalsa_mailbox_get_url(mailbox)));
 }
 
 static gboolean
@@ -414,10 +376,10 @@ imap_scan_attach_mailbox(BalsaMailboxNode * mbnode, imap_scan_item * isi)
 
     /* If the mailbox was added from the config file, it is already
      * connected to "show-prop-dialog". */
-    if (!g_signal_has_handler_pending(G_OBJECT(mbnode),
+    if (!g_signal_has_handler_pending(mbnode,
                                       balsa_mailbox_node_signals
                                       [SHOW_PROP_DIALOG], 0, FALSE))
-	g_signal_connect(G_OBJECT(mbnode), "show-prop-dialog",
+	g_signal_connect(mbnode, "show-prop-dialog",
                          G_CALLBACK(folder_conf_imap_sub_node), NULL);
     if (LIBBALSA_IS_MAILBOX_IMAP(mbnode->mailbox))
         /* it already has a mailbox */
@@ -428,9 +390,10 @@ imap_scan_attach_mailbox(BalsaMailboxNode * mbnode, imap_scan_item * isi)
     libbalsa_mailbox_remote_set_server(LIBBALSA_MAILBOX_REMOTE(m),
 				       mbnode->server);
     libbalsa_mailbox_imap_set_path(m, isi->fn);
-    g_debug("imap_scan_attach_mailbox: add mbox of name %s (full path %s)", isi->fn, mailbox->url);
-    /* avoid allocating the name again: */
-    mailbox->name = mbnode->name;
+    g_debug("imap_scan_attach_mailbox: add mbox of name %s (full path %s)", isi->fn,
+            libbalsa_mailbox_get_url(mailbox));
+    libbalsa_mailbox_set_name(mailbox, mbnode->name);
+    g_free(mbnode->name);
     mbnode->name = NULL;
     mbnode->mailbox = mailbox;
     load_mailbox_view(mbnode);
@@ -441,7 +404,7 @@ imap_scan_attach_mailbox(BalsaMailboxNode * mbnode, imap_scan_item * isi)
         *isi->special = mailbox;
 	g_object_add_weak_pointer(G_OBJECT(m), (gpointer *) isi->special);
         if (isi->special == &balsa_app.outbox)
-            mailbox->no_reassemble = TRUE;
+            libbalsa_mailbox_set_no_reassemble(mailbox, TRUE);
     }
 
     return TRUE;
@@ -463,7 +426,7 @@ imap_scan_attach_mailbox(BalsaMailboxNode * mbnode, imap_scan_item * isi)
 typedef struct imap_scan_tree_ imap_scan_tree;
 struct imap_scan_tree_ {
     GSList *list;               /* a list of imap_scan_items */
-    char delim;
+    int delim;
 };
 
 static void imap_scan_destroy_tree(imap_scan_tree * tree);
@@ -483,7 +446,7 @@ imap_dir_cb(BalsaMailboxNode* mb)
     if (!balsa_app.main_window)
         return;
 
-    statusbar = GTK_STATUSBAR(balsa_app.main_window->statusbar);
+    statusbar = balsa_window_get_statusbar(balsa_app.main_window);
     context_id = gtk_statusbar_get_context_id(statusbar, "MailboxNode imap");
 
     if(restore_children_from_cache(mb))
@@ -510,7 +473,7 @@ imap_dir_cb(BalsaMailboxNode* mb)
                              ? _("Scanning of %s failed: %s\n"
                                  "Check network connectivity.")
                              : _("Scanning of %s failed: %s"),
-                             mb->server->host,
+                             libbalsa_server_get_host(mb->server),
                              error->message);
         g_error_free(error);
         imap_scan_destroy_tree(&imap_tree);
@@ -555,14 +518,15 @@ balsa_mailbox_node_new_from_mailbox(LibBalsaMailbox * mb)
     mbn = BALSA_MAILBOX_NODE(balsa_mailbox_node_new());
     mbn->mailbox = mb;
     load_mailbox_view(mbn);
-    g_signal_connect(G_OBJECT(mbn), "show-prop-dialog", 
+    g_signal_connect(mbn, "show-prop-dialog", 
 		     G_CALLBACK(mailbox_conf_edit), NULL);
     if (LIBBALSA_IS_MAILBOX_MH(mb) || LIBBALSA_IS_MAILBOX_MAILDIR(mb)) {
 	/* Mh and Maildir mailboxes are directories, and may be nested,
 	 * so we need to be able to append a subtree. */
-	mbn->name = g_strdup(libbalsa_mailbox_local_get_path(mb));
+	mbn->name =
+            g_strdup(libbalsa_mailbox_local_get_path((LibBalsaMailboxLocal *) mb));
 	mbn->dir = g_strdup(mbn->name);
-	g_signal_connect(G_OBJECT(mbn), "append-subtree", 
+	g_signal_connect(mbn, "append-subtree", 
                          G_CALLBACK(read_dir_cb), NULL);
     }
     return mbn;
@@ -577,9 +541,9 @@ balsa_mailbox_node_new_from_dir(const gchar* dir)
 
     mbn->name = g_strdup(dir);
     mbn->dir  = g_strdup(dir);
-    g_signal_connect(G_OBJECT(mbn), "show-prop-dialog", 
+    g_signal_connect(mbn, "show-prop-dialog", 
 		     G_CALLBACK(dir_conf_edit), NULL);
-    g_signal_connect(G_OBJECT(mbn), "append-subtree", 
+    g_signal_connect(mbn, "append-subtree", 
 		     G_CALLBACK(read_dir_cb), NULL);
     return mbn;
 }
@@ -602,13 +566,13 @@ balsa_mailbox_node_new_from_config(const gchar* group)
 
     folder->server = LIBBALSA_SERVER(libbalsa_imap_server_new_from_config());
 
-	g_debug("Server loaded, host: %s, security %d.", folder->server->host,
-		folder->server->security);
-    g_signal_connect_swapped(G_OBJECT(folder->server), "config-changed", 
+	g_debug("Server loaded, host: %s, security %d.", libbalsa_server_get_host(folder->server),
+		libbalsa_server_get_security(folder->server));
+    g_signal_connect_swapped(folder->server, "config-changed", 
                              G_CALLBACK(config_folder_update), folder);
-    g_signal_connect(G_OBJECT(folder), "show-prop-dialog", 
+    g_signal_connect(folder, "show-prop-dialog", 
 		     G_CALLBACK(folder_conf_imap_node), NULL);
-    g_signal_connect(G_OBJECT(folder), "append-subtree", 
+    g_signal_connect(folder, "append-subtree", 
 		     G_CALLBACK(imap_dir_cb), NULL);
     libbalsa_server_connect_signals(folder->server,
                                     G_CALLBACK(ask_password), NULL);
@@ -642,7 +606,7 @@ balsa_mailbox_node_new_imap_node(LibBalsaServer* s, const char*p)
 
     folder->server = s;
     folder->dir = g_strdup(p);
-    g_signal_connect(G_OBJECT(folder), "append-subtree", 
+    g_signal_connect(folder, "append-subtree", 
 		     G_CALLBACK(imap_dir_cb), NULL);
 
     return folder;
@@ -654,7 +618,7 @@ balsa_mailbox_node_new_imap_folder(LibBalsaServer* s, const char*p)
     BalsaMailboxNode * folder = balsa_mailbox_node_new_imap_node(s, p);
     g_assert(s);
 
-    g_signal_connect(G_OBJECT(folder), "show-prop-dialog", 
+    g_signal_connect(folder, "show-prop-dialog", 
 		     G_CALLBACK(folder_conf_imap_node), NULL);
     return folder;
 }
@@ -662,19 +626,20 @@ balsa_mailbox_node_new_imap_folder(LibBalsaServer* s, const char*p)
 void
 balsa_mailbox_node_show_prop_dialog(BalsaMailboxNode* mn)
 {
-    if (mn)
-        g_signal_emit(G_OBJECT(mn),
-                      balsa_mailbox_node_signals[SHOW_PROP_DIALOG], 0);
+    if (mn != NULL)
+        g_signal_emit(mn, balsa_mailbox_node_signals[SHOW_PROP_DIALOG], 0);
 }
 
 void
 balsa_mailbox_node_append_subtree(BalsaMailboxNode * mn)
 {
-    g_signal_emit(G_OBJECT(mn),
-		  balsa_mailbox_node_signals[APPEND_SUBTREE], 0);
+    if (!mn->scanned) {
+        g_signal_emit(mn, balsa_mailbox_node_signals[APPEND_SUBTREE], 0);
+        mn->scanned = TRUE;
+    }
 }
 
-void 
+void
 balsa_mailbox_node_show_prop_dialog_cb(GtkWidget * widget, gpointer data)
 {
     balsa_mailbox_node_show_prop_dialog((BalsaMailboxNode*)data);
@@ -687,15 +652,13 @@ balsa_mailbox_node_show_prop_dialog_cb(GtkWidget * widget, gpointer data)
 void
 balsa_mailbox_node_load_config(BalsaMailboxNode* mn, const gchar* group)
 {
-    g_signal_emit(G_OBJECT(mn),
-		  balsa_mailbox_node_signals[LOAD_CONFIG], 0, group);
+    balsa_mailbox_node_real_load_config(mn, group);
 }
 
 void
 balsa_mailbox_node_save_config(BalsaMailboxNode* mn, const gchar* group)
 {
-    g_signal_emit(G_OBJECT(mn),
-		  balsa_mailbox_node_signals[SAVE_CONFIG], 0, group);
+    balsa_mailbox_node_real_save_config(mn, group);
 }
 
 /* ---------------------------------------------------------------------
@@ -718,7 +681,7 @@ balsa_mailbox_local_append(LibBalsaMailbox* mbx)
 
     g_return_if_fail(LIBBALSA_IS_MAILBOX_LOCAL(mbx));
 
-    dir = g_strdup(libbalsa_mailbox_local_get_path(mbx));
+    dir = g_strdup(libbalsa_mailbox_local_get_path((LibBalsaMailboxLocal *) mbx));
     while (dir[1] /* i.e. dir != "/" */
            && !(parent = find_dir(dir))) {
         gchar* tmp =  g_path_get_dirname(dir);
@@ -866,17 +829,19 @@ bmbn_scan_children_idle(BalsaMailboxNode ** mbnode)
             gboolean has_unread_messages = FALSE;
 
             if (mn->mailbox)
-                has_unread_messages = mn->mailbox->has_unread_messages;
+                has_unread_messages =
+                    libbalsa_mailbox_get_has_unread_messages(mn->mailbox);
             balsa_mailbox_node_rescan(mn);
             if (!l->data)
                 continue;
             if (mn->mailbox)
-                mn->mailbox->has_unread_messages = has_unread_messages;
+                libbalsa_mailbox_set_has_unread_messages
+                    (mn->mailbox, has_unread_messages);
             mn->scanned = TRUE;
         } else if (balsa_app.debug)
             g_print("%s: %s “%s” was already scanned\n", __func__,
                     mn->mailbox ? "mailbox" : "folder",
-                    mn->mailbox ? mn->mailbox->name : mn->name);
+                    mn->mailbox ? libbalsa_mailbox_get_name(mn->mailbox) : mn->name);
         g_object_remove_weak_pointer(G_OBJECT(mn), & l->data);
     }
     g_slist_free(list);
@@ -904,7 +869,7 @@ add_menu_entry(GtkWidget * menu, const gchar * label, GCallback cb,
 	: gtk_separator_menu_item_new();
 
     if (cb)
-	g_signal_connect(G_OBJECT(menuitem), "activate",
+	g_signal_connect(menuitem, "activate",
 			 G_CALLBACK(cb), mbnode);
 
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
@@ -1011,7 +976,7 @@ balsa_mailbox_node_is_imap(const BalsaMailboxNode *mbnode)
 {
 	return (mbnode != NULL) &&
 		(mbnode->server != NULL) &&
-		(strcmp(mbnode->server->protocol, "imap") == 0);
+		(strcmp(libbalsa_server_get_protocol(mbnode->server), "imap") == 0);
 }
 
 GtkWidget *
@@ -1027,7 +992,7 @@ balsa_mailbox_node_get_context_menu(BalsaMailboxNode * mbnode)
 
     menu = gtk_menu_new();
     /* it's a single-use menu, so we must destroy it when we're done */
-    g_signal_connect(G_OBJECT(menu), "selection-done",
+    g_signal_connect(menu, "selection-done",
                      G_CALLBACK(gtk_widget_destroy), NULL);
 
     submenu = gtk_menu_new();
@@ -1061,13 +1026,13 @@ balsa_mailbox_node_get_context_menu(BalsaMailboxNode * mbnode)
     /* If we didn't click on a mailbox node then there is only one option. */
     add_menu_entry(menu, NULL, NULL, NULL);
 
-    if (g_signal_has_handler_pending(G_OBJECT(mbnode),
+    if (g_signal_has_handler_pending(mbnode,
                                      balsa_mailbox_node_signals
                                      [SHOW_PROP_DIALOG], 0, FALSE))
         add_menu_entry(menu, _("_Properties…"),
                        G_CALLBACK(mb_conf_cb), mbnode);
 
-    if (g_signal_has_handler_pending(G_OBJECT(mbnode),
+    if (g_signal_has_handler_pending(mbnode,
 		                     balsa_mailbox_node_signals
 				     [APPEND_SUBTREE], 0, FALSE))
 	add_menu_entry(menu, _("_Rescan"),
@@ -1140,28 +1105,28 @@ remove_mailbox_from_nodes(LibBalsaMailbox* mailbox)
 	mbnode = balsa_mailbox_node_new_from_mailbox(mailbox);
     return mbnode;
 }
-    
-static BalsaMailboxNode * 
+
+static BalsaMailboxNode *
 remove_special_mailbox_by_url(const gchar* url, LibBalsaMailbox *** special)
 {
     LibBalsaMailbox **mailbox;
 
-    if (g_strcmp0(url, balsa_app.trash->url) == 0)
+    if (g_strcmp0(url, libbalsa_mailbox_get_url(balsa_app.trash)) == 0)
 	mailbox = &balsa_app.trash;
-    else if (g_strcmp0(url, balsa_app.inbox->url) == 0)
+    else if (g_strcmp0(url, libbalsa_mailbox_get_url(balsa_app.inbox)) == 0)
 	mailbox = &balsa_app.inbox;
-    else if (g_strcmp0(url, balsa_app.outbox->url) == 0)
+    else if (g_strcmp0(url, libbalsa_mailbox_get_url(balsa_app.outbox)) == 0)
 	mailbox = &balsa_app.outbox;
-    else if (g_strcmp0(url, balsa_app.sentbox->url) == 0)
+    else if (g_strcmp0(url, libbalsa_mailbox_get_url(balsa_app.sentbox)) == 0)
 	mailbox = &balsa_app.sentbox;
-    else if (g_strcmp0(url, balsa_app.draftbox->url) == 0)
+    else if (g_strcmp0(url, libbalsa_mailbox_get_url(balsa_app.draftbox)) == 0)
 	mailbox = &balsa_app.draftbox;
     else
         mailbox = NULL;
 
     if (special)
         *special = mailbox;
-	
+
     return mailbox ? remove_mailbox_from_nodes(*mailbox) : NULL;
 }
 
@@ -1203,13 +1168,13 @@ add_local_mailbox(BalsaMailboxNode *root, const gchar * name,
 		g_warning("Not accessible mailbox %s", path);
 	    return NULL;
 	}
-	mailbox->name = g_strdup(name);
+	libbalsa_mailbox_set_name(mailbox, name);
 	
 	mbnode = balsa_mailbox_node_new_from_mailbox(mailbox);
 	
 	if (balsa_app.debug)
 	    g_print(_("Local mailbox %s loaded as: %s\n"),
-		    mailbox->name,
+		    libbalsa_mailbox_get_name(mailbox),
 		    g_type_name(G_OBJECT_TYPE(mailbox)));
         if (balsa_app.check_mail_upon_startup
             && libbalsa_mailbox_get_subscribe(mailbox) !=
@@ -1249,7 +1214,7 @@ add_local_folder(BalsaMailboxNode * root, const char *d_name,
  * IMAP folder scanner functions 
  * --------------------------------------------------------------------- */
 static gchar*
-get_parent_folder_name(const gchar* path, char delim)
+get_parent_folder_name(const gchar* path, gint delim)
 {
     const gchar* last_delim = strrchr(path, delim);
     return last_delim ? g_strndup(path, last_delim-path)
@@ -1329,7 +1294,7 @@ imap_scan_create_mbnode(BalsaMailboxNode * root, imap_scan_item * isi,
    add given mailbox unless its base name begins on dot.
 */
 static void
-add_imap_entry(const char *fn, char delim, gboolean noscan,
+add_imap_entry(const char *fn, int delim, gboolean noscan,
 	       gboolean selectable, gboolean marked, void *data)
 {
     imap_scan_tree *tree = (imap_scan_tree *) data;
@@ -1360,7 +1325,7 @@ imap_scan_destroy_tree(imap_scan_tree * tree)
 }
 
 static void
-handle_imap_path(const char *fn, char delim, int noselect, int noscan,
+handle_imap_path(const char *fn, int delim, int noselect, int noscan,
 		 int marked, void *data)
 {
     if (!noselect) {
@@ -1425,3 +1390,161 @@ mark_imap_path(const gchar * fn, gpointer data)
     	g_debug("… not found.");
     }
 }
+
+/*
+ * Setters
+ */
+
+void
+balsa_mailbox_node_set_dir(BalsaMailboxNode * mbnode, const gchar * dir)
+{
+    g_return_if_fail(BALSA_IS_MAILBOX_NODE(mbnode));
+
+    g_free(mbnode->dir);
+    mbnode->dir = g_strdup(dir);
+}
+
+void
+balsa_mailbox_node_set_name(BalsaMailboxNode * mbnode, const gchar * name)
+{
+    g_return_if_fail(BALSA_IS_MAILBOX_NODE(mbnode));
+
+    g_free(mbnode->name);
+    mbnode->name = g_strdup(name);
+}
+
+void
+balsa_mailbox_node_set_config_prefix(BalsaMailboxNode * mbnode, const gchar * config_prefix)
+{
+    g_return_if_fail(BALSA_IS_MAILBOX_NODE(mbnode));
+
+    g_free(mbnode->config_prefix);
+    mbnode->config_prefix = g_strdup(config_prefix);
+}
+
+void
+balsa_mailbox_node_set_last_use_time(BalsaMailboxNode * mbnode)
+{
+    g_return_if_fail(BALSA_IS_MAILBOX_NODE(mbnode));
+
+    time(&mbnode->last_use);
+}
+
+void
+balsa_mailbox_node_change_style(BalsaMailboxNode * mbnode,
+                                BalsaMailboxNodeStyle set,
+                                BalsaMailboxNodeStyle clear)
+{
+    g_return_if_fail(BALSA_IS_MAILBOX_NODE(mbnode));
+
+    mbnode->style |= set;
+    mbnode->style &= ~clear;
+}
+
+void
+balsa_mailbox_node_set_list_inbox(BalsaMailboxNode * mbnode, guint list_inbox)
+{
+    g_return_if_fail(BALSA_IS_MAILBOX_NODE(mbnode));
+
+    mbnode->list_inbox = !!list_inbox;
+}
+
+void
+balsa_mailbox_node_set_subscribed(BalsaMailboxNode * mbnode, guint subscribed)
+{
+    g_return_if_fail(BALSA_IS_MAILBOX_NODE(mbnode));
+
+    mbnode->subscribed = !!subscribed;
+}
+
+/*
+ * Getters
+ */
+
+LibBalsaMailbox *
+balsa_mailbox_node_get_mailbox(BalsaMailboxNode * mbnode)
+{
+    g_return_val_if_fail(BALSA_IS_MAILBOX_NODE(mbnode), NULL);
+
+    return mbnode->mailbox;
+}
+
+const gchar *
+balsa_mailbox_node_get_name(BalsaMailboxNode * mbnode)
+{
+    g_return_val_if_fail(BALSA_IS_MAILBOX_NODE(mbnode), NULL);
+
+    return mbnode->name;
+}
+
+const gchar *
+balsa_mailbox_node_get_config_prefix(BalsaMailboxNode * mbnode)
+{
+    g_return_val_if_fail(BALSA_IS_MAILBOX_NODE(mbnode), NULL);
+
+    return mbnode->config_prefix;
+}
+
+time_t
+balsa_mailbox_node_get_last_use_time(BalsaMailboxNode * mbnode)
+{
+    g_return_val_if_fail(BALSA_IS_MAILBOX_NODE(mbnode), 0);
+
+    return mbnode->last_use;
+}
+
+LibBalsaServer *
+balsa_mailbox_node_get_server(BalsaMailboxNode * mbnode)
+{
+    g_return_val_if_fail(BALSA_IS_MAILBOX_NODE(mbnode), NULL);
+
+    return mbnode->server;
+}
+
+BalsaMailboxNodeStyle
+balsa_mailbox_node_get_style(BalsaMailboxNode * mbnode)
+{
+    g_return_val_if_fail(BALSA_IS_MAILBOX_NODE(mbnode), 0);
+
+    return mbnode->style;
+}
+
+guint
+balsa_mailbox_node_get_subscribed(BalsaMailboxNode * mbnode)
+{
+    g_return_val_if_fail(BALSA_IS_MAILBOX_NODE(mbnode), 0);
+
+    return mbnode->subscribed;
+}
+
+guint
+balsa_mailbox_node_get_list_inbox(BalsaMailboxNode * mbnode)
+{
+    g_return_val_if_fail(BALSA_IS_MAILBOX_NODE(mbnode), 0);
+
+    return mbnode->list_inbox;
+}
+
+gint
+balsa_mailbox_node_get_delim(BalsaMailboxNode * mbnode)
+{
+    g_return_val_if_fail(BALSA_IS_MAILBOX_NODE(mbnode), 0);
+
+    return mbnode->delim;
+}
+
+const gchar *
+balsa_mailbox_node_get_dir(BalsaMailboxNode * mbnode)
+{
+    g_return_val_if_fail(BALSA_IS_MAILBOX_NODE(mbnode), NULL);
+
+    return mbnode->dir;
+}
+
+BalsaMailboxNode *
+balsa_mailbox_node_get_parent(BalsaMailboxNode * mbnode)
+{
+    g_return_val_if_fail(BALSA_IS_MAILBOX_NODE(mbnode), NULL);
+
+    return mbnode->parent;
+}
