@@ -29,17 +29,14 @@
 #include "mailbox-node.h"
 #include "toolbar-factory.h"
 
-#define BALSA_TYPE_WINDOW		       (balsa_window_get_type ())
-#define BALSA_WINDOW(obj)		       (G_TYPE_CHECK_INSTANCE_CAST (obj, BALSA_TYPE_WINDOW, BalsaWindow))
-#define BALSA_WINDOW_CLASS(klass)	       (G_TYPE_CHECK_CLASS_CAST (klass, BALSA_TYPE_WINDOW, BalsaWindowClass))
-#define BALSA_IS_WINDOW(obj)		       (G_TYPE_CHECK_INSTANCE_TYPE (obj, BALSA_TYPE_WINDOW))
-#define BALSA_IS_WINDOW_CLASS(klass)	       (G_TYPE_CHECK_CLASS_TYPE (klass, BALSA_TYPE_WINDOW))
-#define BALSA_WINDOW_GET_CLASS(window)                       \
-    (G_TYPE_INSTANCE_GET_CLASS ((window), BALSA_TYPE_WINDOW, \
-				BalsaWindowClass))
+#define BALSA_TYPE_WINDOW balsa_window_get_type()
+
+G_DECLARE_DERIVABLE_TYPE(BalsaWindow,
+                         balsa_window,
+                         BALSA,
+                         WINDOW,
+                         GtkApplicationWindow)
 
-typedef struct _BalsaWindow BalsaWindow;
-typedef struct _BalsaWindowClass BalsaWindowClass;
 typedef enum {
     BALSA_PROGRESS_NONE = 0,
     BALSA_PROGRESS_ACTIVITY,
@@ -47,41 +44,6 @@ typedef enum {
 } BalsaWindowProgress;
 
 
-struct _BalsaWindow {
-    GtkApplicationWindow window;
-
-    GtkWidget *toolbar;
-    GtkWidget *sos_bar;
-    GtkWidget *bottom_bar;
-    GtkWidget *progress_bar;
-    GtkWidget *statusbar;
-    GtkWidget *mblist;
-    GtkWidget *sos_entry;       /* SenderOrSubject filter entry */
-    GtkWidget *notebook;
-    GtkWidget *preview;		/* message is child */
-    GtkWidget *paned_master;
-    GtkWidget *paned_slave;
-    GtkWidget *current_index;
-    GtkWidget *filter_choice;
-    GtkWidget *vbox;
-    GtkWidget *content;
-
-    guint set_message_id;
-
-    /* Progress bar stuff: */
-    BalsaWindowProgress progress_type;
-    guint activity_handler;
-    guint activity_counter;
-    GSList *activity_messages;
-
-    gboolean new_mail_notification_sent;
-
-    /* Support GNetworkMonitor: */
-    gboolean network_available;
-    time_t last_check_time;
-    guint network_changed_source_id;
-};
-
 struct _BalsaWindowClass {
     GtkApplicationWindowClass parent_class;
 
@@ -106,7 +68,6 @@ enum {
     FILTER_RECIPIENT = 1
 };
 
-GType balsa_window_get_type(void);
 GtkWidget *balsa_window_new(GtkApplication *application);
 gboolean balsa_window_fix_paned(BalsaWindow *window);
 GtkWidget *balsa_window_find_current_index(BalsaWindow * window);
@@ -151,6 +112,12 @@ void balsa_window_set_statusbar(BalsaWindow     * window,
                                 LibBalsaMailbox * mailbox);
 void balsa_window_set_thread_messages(BalsaWindow * window, gboolean thread_messages);
 
+/*
+ * Getter
+ */
+
+GtkStatusbar * balsa_window_get_statusbar(BalsaWindow * window);
+
 #if defined(__FILE__) && defined(__LINE__)
 # ifdef __FUNCTION__
 #  define BALSA_DEBUG_MSG(message)  if (balsa_app.debug)  fprintf(stderr, "[%lu] %12s | %4d | %30s: %s\n", (unsigned long) time(NULL), __FILE__, __LINE__, __FUNCTION__, message)
