@@ -52,10 +52,8 @@
 
 #include "libinit_balsa/assistant_init.h"
 
-#ifdef HAVE_GPGME
 #include "libbalsa-gpgme.h"
 #include "libbalsa-gpgme-cb.h"
-#endif
 
 /* We need separate variable for storing command line requests to check the
    mail because such selection cannot be stored in balsa_app and later
@@ -171,7 +169,7 @@ initial_open_unread_mailboxes()
         for (l = gl; l; l = l->next) {
             LibBalsaMailbox *mailbox = LIBBALSA_MAILBOX(l->data);
 
-            printf("opening %s..\n", mailbox->name);
+            printf("opening %s..\n", libbalsa_mailbox_get_name(mailbox));
             balsa_mblist_open_mailbox(mailbox);
         }
         g_list_free(gl);
@@ -186,7 +184,7 @@ initial_open_inbox()
     if (!balsa_app.inbox)
 	return FALSE;
 
-    printf("opening %s..\n", balsa_app.inbox->name);
+    printf("opening %s..\n", libbalsa_mailbox_get_name(balsa_app.inbox));
     balsa_mblist_open_mailbox_hidden(balsa_app.inbox);
 
     return FALSE;
@@ -199,7 +197,7 @@ balsa_get_stats(long *unread, long *unsent)
     if (balsa_app.inbox && libbalsa_mailbox_open(balsa_app.inbox, NULL)) {
         /* set threading type to load messages */
         libbalsa_mailbox_set_threading(balsa_app.inbox);
-        *unread = balsa_app.inbox->unread_messages;
+        *unread = libbalsa_mailbox_get_unread_messages(balsa_app.inbox);
         libbalsa_mailbox_close(balsa_app.inbox, FALSE);
     } else *unread = -1;
     if(balsa_app.draftbox && libbalsa_mailbox_open(balsa_app.outbox, NULL)){
@@ -262,7 +260,7 @@ scan_mailboxes_idle_cb()
         gl = balsa_mblist_find_all_unread_mboxes(NULL);
         for (l = gl; l; l = l->next) {
             LibBalsaMailbox *mailbox = l->data;
-            g_ptr_array_add(url_array, g_strdup(mailbox->url));
+            g_ptr_array_add(url_array, g_strdup(libbalsa_mailbox_get_url(mailbox)));
         }
         g_list_free(gl);
     }
@@ -292,7 +290,7 @@ scan_mailboxes_idle_cb()
     }
 
     if (cmd_open_inbox || balsa_app.open_inbox_upon_startup) {
-        g_ptr_array_add(url_array, g_strdup(balsa_app.inbox->url));
+        g_ptr_array_add(url_array, g_strdup(libbalsa_mailbox_get_url(balsa_app.inbox)));
     }
 
     if (url_array->len) {
@@ -334,7 +332,8 @@ periodic_expunge_cb(void)
     GSList *list = NULL, *l;
 
     /* should we enforce expunging now and then? Perhaps not... */
-    if(!balsa_app.expunge_auto) return TRUE;
+    if (!balsa_app.expunge_auto)
+        return TRUE;
 
     libbalsa_information(LIBBALSA_INFORMATION_DEBUG,
                          _("Compressing mail folders…"));
@@ -342,13 +341,15 @@ periodic_expunge_cb(void)
 			   (GtkTreeModelForeachFunc)mbnode_expunge_func,
 			   &list);
 
-    for (l = list; l; l = l->next) {
+    for (l = list; l != NULL; l = l->next) {
         BalsaMailboxNode *mbnode = l->data;
-        if (mbnode->mailbox && libbalsa_mailbox_is_open(mbnode->mailbox)
-            && !mbnode->mailbox->readonly) {
+        LibBalsaMailbox *mailbox = balsa_mailbox_node_get_mailbox(mbnode);
+
+        if (mailbox != NULL && libbalsa_mailbox_is_open(mailbox) &&
+            !libbalsa_mailbox_get_readonly(mailbox)) {
             time_t tm = time(NULL);
-            if (tm-mbnode->last_use > balsa_app.expunge_timeout)
-                libbalsa_mailbox_sync_storage(mbnode->mailbox, TRUE);
+            if (tm - balsa_mailbox_node_get_last_use_time(mbnode) > balsa_app.expunge_timeout)
+                libbalsa_mailbox_sync_storage(mailbox, TRUE);
         }
         g_object_unref(mbnode);
     }
@@ -519,11 +520,9 @@ balsa_startup_cb(GApplication *application,
     LIBXML_TEST_VERSION
 #endif
 
-#ifdef HAVE_GPGME
     /* initialise the gpgme library and set the callback funcs */
     libbalsa_gpgme_init(lb_gpgme_passphrase, lb_gpgme_select_key,
 			lb_gpgme_accept_low_trust_key);
-#endif
 
     balsa_app_init();
 
@@ -563,12 +562,10 @@ balsa_startup_cb(GApplication *application,
         return;
     }
 
-#ifdef HAVE_GPGME
     balsa_app.has_openpgp =
         libbalsa_gpgme_check_crypto_engine(GPGME_PROTOCOL_OpenPGP);
     balsa_app.has_smime =
         libbalsa_gpgme_check_crypto_engine(GPGME_PROTOCOL_CMS);
-#endif /* HAVE_GPGME */
 
     accel_map_load();
 }
