@@ -32,9 +32,7 @@
 #include "balsa-print-object.h"
 #include "balsa-print-object-decor.h"
 #include "balsa-print-object-header.h"
-#ifdef HAVE_GPGME
 #include "libbalsa-gpgme.h"
-#endif
 
 #if HAVE__NL_MEASUREMENT_MEASUREMENT
 #include <langinfo.h>
@@ -98,7 +96,7 @@ print_header_footer(GtkPrintContext * context, cairo_t * cairo_ctx,
     g_free(pagebuf);
     cairo_move_to(cairo_ctx, pdata->setup.c_x0, pdata->c_header_y);
     pango_cairo_show_layout(cairo_ctx, layout);
-    g_object_unref(G_OBJECT(layout));
+    g_object_unref(layout);
 
     /* footer (if available) */
     if (pdata->footer) {
@@ -113,7 +111,7 @@ print_header_footer(GtkPrintContext * context, cairo_t * cairo_ctx,
 	pango_layout_set_text(layout, pdata->footer, -1);
 	cairo_move_to(cairo_ctx, pdata->setup.c_x0, pdata->c_footer_y);
 	pango_cairo_show_layout(cairo_ctx, layout);
-	g_object_unref(G_OBJECT(layout));
+	g_object_unref(layout);
     }
 }
 
@@ -159,7 +157,6 @@ find_alt_part(LibBalsaMessageBody *parts,
 }
 
 
-#ifdef HAVE_GPGME
 static inline GList *
 begin_crypto_frame(GList 		   *bpo_list,
 				   BalsaPrintSetup *psetup,
@@ -179,7 +176,6 @@ begin_crypto_frame(GList 		   *bpo_list,
 	}
 	return bpo_list;
 }
-#endif				/* HAVE_GPGME */
 
 
 static GList *
@@ -194,9 +190,7 @@ print_single_part(GList 			  *bpo_list,
 		bpo_list = balsa_print_object_separator(bpo_list, psetup);
 	}
 	if (add_signature) {
-#ifdef HAVE_GPGME
 		bpo_list = begin_crypto_frame(bpo_list, psetup, TRUE, body->was_encrypted);
-#endif				/* HAVE_GPGME */
 	}
 	return balsa_print_objects_append_from_body(bpo_list, context, body, psetup);
 }
@@ -210,15 +204,12 @@ scan_body(GList *bpo_list, GtkPrintContext * context, BalsaPrintSetup * psetup,
 	  LibBalsaMessageBody * body, gboolean no_first_sep)
 {
     gboolean add_signature = FALSE;
-#ifdef HAVE_GPGME
     gboolean have_crypto_frame;
 	gboolean is_mp_signed;
-#endif				/* HAVE_GPGME */
     while (body) {
 	gchar *conttype;
 
 	conttype = libbalsa_message_body_get_mime_type(body);
-#ifdef HAVE_GPGME
 	add_signature = body->sig_info &&
 	    g_ascii_strcasecmp(conttype, "application/pgp-signature") &&
 	    g_ascii_strcasecmp(conttype, "application/pkcs7-signature") &&
@@ -233,19 +224,16 @@ scan_body(GList *bpo_list, GtkPrintContext * context, BalsaPrintSetup * psetup,
 	} else {
 	    have_crypto_frame = FALSE;
 	}
-#endif				/* HAVE_GPGME */
 
 	if (g_ascii_strncasecmp(conttype, "multipart/", 10)) {
 		bpo_list = print_single_part(bpo_list, context, psetup, body, no_first_sep, add_signature);
 		no_first_sep = FALSE;
-#ifdef HAVE_GPGME
 	} else if (add_signature && !is_mp_signed) {
 		if (!no_first_sep) {
 			bpo_list = balsa_print_object_separator(bpo_list, psetup);
 			no_first_sep = TRUE;
 		}
 		bpo_list = begin_crypto_frame(bpo_list, psetup, TRUE, body->was_encrypted);
-#endif				/* HAVE_GPGME */
 	}
 
 	if (body->parts) {
@@ -261,20 +249,14 @@ scan_body(GList *bpo_list, GtkPrintContext * context, BalsaPrintSetup * psetup,
 	}
 
 	/* end the frame for an embedded message or encrypted stuff */
-	if (!g_ascii_strcasecmp(conttype, "message/rfc822")
-#ifdef HAVE_GPGME
-	    || have_crypto_frame
-#endif
-	    )
+	if (!g_ascii_strcasecmp(conttype, "message/rfc822") || have_crypto_frame)
 	    bpo_list = balsa_print_object_frame_end(bpo_list, psetup);
 
-#ifdef HAVE_GPGME
 	if (add_signature) {
 	    bpo_list = balsa_print_object_separator(bpo_list, psetup);
 	    bpo_list = balsa_print_object_header_crypto(bpo_list, context, body, psetup);
 	    bpo_list = balsa_print_object_frame_end(bpo_list, psetup);
 	}
-#endif				/* HAVE_GPGME */
 	g_free(conttype);
 
 	body = body->next;
@@ -294,6 +276,7 @@ begin_print(GtkPrintOperation * operation, GtkPrintContext * context,
     gchar *subject;
     gchar *date;
     GString *footer_string;
+    InternetAddressList *from_list;
 
     /* initialise the context */
     page_setup = gtk_print_context_get_page_setup(context);
@@ -358,9 +341,9 @@ begin_print(GtkPrintOperation * operation, GtkPrintContext * context,
     	g_free(date);
     }
 
-    if (pdata->message->headers->from) {
-	gchar *from =
-	    internet_address_list_to_string(pdata->message->headers->from, FALSE);
+    from_list = libbalsa_message_get_headers(pdata->message)->from;
+    if (from_list != NULL) {
+	gchar *from = internet_address_list_to_string(from_list, FALSE);
 
 	libbalsa_utf8_sanitize(&from, balsa_app.convert_unknown_8bit,
 			       NULL);
@@ -394,7 +377,7 @@ begin_print(GtkPrintOperation * operation, GtkPrintContext * context,
 	/* remember in the context */
 	pdata->footer = g_string_free(footer_string, FALSE);
     }
-    g_object_unref(G_OBJECT(layout));
+    g_object_unref(layout);
 
     /* add the message headers */
     pdata->setup.c_y_pos = 0.0;	/* to simplify calculating the layout... */
@@ -406,7 +389,7 @@ begin_print(GtkPrintOperation * operation, GtkPrintContext * context,
     /* add the mime bodies */
     pdata->print_parts = 
 	scan_body(pdata->print_parts, context, &pdata->setup,
-		  pdata->message->body_list, FALSE);
+		  libbalsa_message_get_body_list(pdata->message), FALSE);
 
     /* done */
     gtk_print_operation_set_n_pages(operation, pdata->setup.page_count);
@@ -443,7 +426,7 @@ draw_page(GtkPrintOperation * operation, GtkPrintContext * context,
     while (p) {
 	BalsaPrintObject *po = BALSA_PRINT_OBJECT(p->data);
 
-	if (po->on_page == page_nr)
+	if (balsa_print_object_get_page(po) == page_nr)
 	    balsa_print_object_draw(po, context, cairo_ctx);
 
 	p = g_list_next(p);
@@ -653,10 +636,10 @@ message_prefs_widget(GtkPrintOperation * operation,
 			   gtk_page_setup_get_page_height(pg_setup, GTK_UNIT_POINTS), 
 			   balsa_app.margin_bottom,
 			   GTK_GRID(grid), 1);
-    g_signal_connect(G_OBJECT(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(print_prefs->margin_top))),
+    g_signal_connect(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(print_prefs->margin_top)),
 		     "value-changed", G_CALLBACK(check_margins),
 		     gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(print_prefs->margin_bottom)));
-    g_signal_connect(G_OBJECT(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(print_prefs->margin_bottom))),
+    g_signal_connect(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(print_prefs->margin_bottom)),
 		     "value-changed", G_CALLBACK(check_margins),
 		     gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(print_prefs->margin_top)));
     print_prefs->margin_left =
@@ -671,10 +654,10 @@ message_prefs_widget(GtkPrintOperation * operation,
 			   gtk_page_setup_get_page_width(pg_setup, GTK_UNIT_POINTS), 
 			   balsa_app.margin_right,
 			   GTK_GRID(grid), 3);
-    g_signal_connect(G_OBJECT(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(print_prefs->margin_left))),
+    g_signal_connect(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(print_prefs->margin_left)),
 		     "value-changed", G_CALLBACK(check_margins),
 		     gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(print_prefs->margin_right)));
-    g_signal_connect(G_OBJECT(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(print_prefs->margin_right))),
+    g_signal_connect(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(print_prefs->margin_right)),
 		     "value-changed", G_CALLBACK(check_margins),
 		     gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(print_prefs->margin_left)));
 
@@ -741,7 +724,7 @@ message_print_page_setup(GtkWindow * parent)
 					balsa_app.print_settings);
 
     if (balsa_app.page_setup)
-	g_object_unref(G_OBJECT(balsa_app.page_setup));
+	g_object_unref(balsa_app.page_setup);
 
     balsa_app.page_setup = new_page_setup;
 }
@@ -759,7 +742,7 @@ message_print(LibBalsaMessage * msg, GtkWindow * parent)
     print = gtk_print_operation_new();
     g_assert(print != NULL);
 
-    g_object_ref(G_OBJECT(msg));
+    g_object_ref(msg);
 
     gtk_print_operation_set_n_pages(print, 1);
     gtk_print_operation_set_unit(print, GTK_UNIT_POINTS);
@@ -803,10 +786,9 @@ message_print(LibBalsaMessage * msg, GtkWindow * parent)
     /* clean up */
     if (err)
 	g_error_free(err);
-    g_list_foreach(print_data->print_parts, (GFunc) g_object_unref, NULL);
-    g_list_free(print_data->print_parts);
+    g_list_free_full(print_data->print_parts, g_object_unref);
     g_free(print_data->footer);
     g_free(print_data);
-    g_object_unref(G_OBJECT(print));
-    g_object_unref(G_OBJECT(msg));
+    g_object_unref(print);
+    g_object_unref(msg);
 }
