@@ -69,15 +69,27 @@ static void config_filters_load(void);
 
 static inline gboolean is_special_name(const gchar *name);
 
-#define folder_section_path(mn) \
-    BALSA_MAILBOX_NODE(mn)->config_prefix ? \
-    g_strdup(BALSA_MAILBOX_NODE(mn)->config_prefix) : \
-    config_get_unused_group(FOLDER_SECTION_PREFIX)
+static gchar *
+mailbox_section_path(LibBalsaMailbox * mailbox)
+{
+    const gchar *config_prefix =
+        libbalsa_mailbox_get_config_prefix(mailbox);
 
-#define mailbox_section_path(mbox) \
-    LIBBALSA_MAILBOX(mbox)->config_prefix ? \
-    g_strdup(LIBBALSA_MAILBOX(mbox)->config_prefix) : \
-    config_get_unused_group(MAILBOX_SECTION_PREFIX)
+    return config_prefix != NULL ?
+        g_strdup(config_prefix) :
+        config_get_unused_group(MAILBOX_SECTION_PREFIX);
+}
+
+static gchar *
+folder_section_path(BalsaMailboxNode * mbnode)
+{
+    const gchar *config_prefix =
+        balsa_mailbox_node_get_config_prefix(mbnode);
+
+    return config_prefix != NULL ?
+        g_strdup(config_prefix) :
+        config_get_unused_group(FOLDER_SECTION_PREFIX);
+}
 
 static gchar *
 address_book_section_path(LibBalsaAddressBook *address_book)
@@ -121,24 +133,32 @@ migrate_imap_mailboxes(const gchar *key, const gchar *value, gpointer data)
 		libbalsa_conf_pop_group();
 		mbnode = balsa_mailbox_node_new_from_config(key);
 	    if (mbnode != NULL) {
+                LibBalsaServer *server = balsa_mailbox_node_get_server(mbnode);
 	    	gchar *folder_key;
 	    	gchar *oldname;
 
 	    	/* do not add the same folder multiple times */
-	    	folder_key = g_strconcat(mbnode->server->user, "@", mbnode->server->host, NULL);
-	    	oldname = g_strdup(mbnode->name);
+	    	folder_key = g_strconcat(libbalsa_server_get_user(server), "@",
+                                         libbalsa_server_get_host(server), NULL);
+	    	oldname = g_strdup(balsa_mailbox_node_get_name(mbnode));
 	    	if (g_hash_table_contains(migrated, folder_key)) {
 	    		g_object_unref(mbnode);
 	    		g_free(folder_key);
 	    	} else {
+                        gchar *tmp;
+
 	    		g_hash_table_add(migrated, folder_key);
-		    	g_free(mbnode->name);
-		    	mbnode->name = g_strdup(mbnode->server->host);
+		    	balsa_mailbox_node_set_name(mbnode, libbalsa_server_get_host(server));
 		    	balsa_mblist_mailbox_node_append(NULL, mbnode);
 
-		    	g_free(mbnode->config_prefix);
-		    	mbnode->config_prefix = config_get_unused_group(FOLDER_SECTION_PREFIX);
-		    	g_signal_connect_swapped(mbnode->server, "config-changed", G_CALLBACK(config_folder_update), mbnode);
+		    	tmp =  config_get_unused_group(FOLDER_SECTION_PREFIX);
+		    	balsa_mailbox_node_set_config_prefix(mbnode, tmp);
+                        g_free(tmp);
+
+		    	g_signal_connect_swapped(balsa_mailbox_node_get_server(mbnode),
+                                                 "config-changed",
+                                                 G_CALLBACK(config_folder_update),
+                                                 mbnode);
 	    	}
 
 	    	if (!is_special_name(value)) {
@@ -271,17 +291,19 @@ config_mailbox_set_as_special(LibBalsaMailbox * mailbox, specialType which)
     default:
 	return;
     }
-    if ((old_mailbox = *special)) {
-        *special = NULL;
-        g_free(old_mailbox->config_prefix);
-        old_mailbox->config_prefix = NULL;
+    if ((old_mailbox = *special) != NULL) {
+        gchar *basename;
 
-        g_free(old_mailbox->name);
-        old_mailbox->name = g_path_get_basename(old_mailbox->url);
+        *special = NULL;
+        libbalsa_mailbox_set_config_prefix(old_mailbox, NULL);
+
+        basename = g_path_get_basename(libbalsa_mailbox_get_url(old_mailbox));
+        libbalsa_mailbox_set_name(old_mailbox, basename);
+        g_free(basename);
 
         if (!LIBBALSA_IS_MAILBOX_LOCAL(old_mailbox)
             || !libbalsa_path_is_below_dir(libbalsa_mailbox_local_get_path
-                                           (old_mailbox),
+                                           ((LibBalsaMailboxLocal *) old_mailbox),
                                            balsa_app.local_mail_directory))
             config_mailbox_add(old_mailbox, NULL);
 
@@ -293,8 +315,7 @@ config_mailbox_set_as_special(LibBalsaMailbox * mailbox, specialType which)
 	g_object_unref(mbnode);
     }
     config_mailbox_delete(mailbox);
-    g_free(mailbox->name);
-    mailbox->name = g_strdup(specialNames[which]);
+    libbalsa_mailbox_set_name(mailbox, specialNames[which]);
     config_mailbox_add(mailbox, specialNames[which]);
 
     *special = mailbox;
@@ -307,7 +328,8 @@ config_mailbox_set_as_special(LibBalsaMailbox * mailbox, specialType which)
 
     switch(which) {
     case SPECIAL_SENT: 
-	balsa_mblist_mru_add(&balsa_app.fcc_mru, mailbox->url); break;
+	balsa_mblist_mru_add(&balsa_app.fcc_mru, libbalsa_mailbox_get_url(mailbox));
+        break;
     case SPECIAL_TRASH:
         libbalsa_filters_set_trash(balsa_app.trash); break;
     default: break;
@@ -388,7 +410,8 @@ gint config_folder_add(BalsaMailboxNode * mbnode, const char *key_arg)
 }				/* config_mailbox_add */
 
 /* removes from the configuration only */
-gint config_mailbox_delete(const LibBalsaMailbox * mailbox)
+gint
+config_mailbox_delete(LibBalsaMailbox * mailbox)
 {
     gchar *tmp;			/* the key in the mailbox section name */
     gint res;
@@ -401,7 +424,7 @@ gint config_mailbox_delete(const LibBalsaMailbox * mailbox)
 }				/* config_mailbox_delete */
 
 gint
-config_folder_delete(const BalsaMailboxNode * mbnode)
+config_folder_delete(BalsaMailboxNode * mbnode)
 {
     gchar *tmp;			/* the key in the mailbox section name */
     gint res;
@@ -465,7 +488,7 @@ config_mailbox_init(const gchar * prefix)
     if (mailbox == NULL)
 	return FALSE;
     if (LIBBALSA_IS_MAILBOX_REMOTE(mailbox)) {
-	LibBalsaServer *server = LIBBALSA_MAILBOX_REMOTE_SERVER(mailbox);
+	LibBalsaServer *server = LIBBALSA_MAILBOX_REMOTE_GET_SERVER(mailbox);
         libbalsa_server_connect_signals(server,
                                         G_CALLBACK(ask_password), NULL);
 	g_signal_connect_swapped(server, "config-changed",
@@ -486,7 +509,7 @@ config_mailbox_init(const gchar * prefix)
 	    special = &balsa_app.inbox;
 	else if (strcmp(OUTBOX_NAME, key) == 0) {
 	    special = &balsa_app.outbox;
-            mailbox->no_reassemble = TRUE;
+            libbalsa_mailbox_set_no_reassemble(mailbox, TRUE);
         } else if (strcmp(SENTBOX_NAME, key) == 0)
 	    special = &balsa_app.sentbox;
 	else if (strcmp(DRAFTS_NAME, key) == 0)
@@ -526,7 +549,8 @@ config_folder_init(const gchar * prefix)
     g_return_val_if_fail(prefix != NULL, FALSE);
 
     if( (folder = balsa_mailbox_node_new_from_config(prefix)) ) {
-	g_signal_connect_swapped(folder->server, "config-changed",
+	g_signal_connect_swapped(balsa_mailbox_node_get_server(folder),
+                                 "config-changed",
                                  G_CALLBACK(config_folder_update),
 				 folder);
 	balsa_mblist_mailbox_node_append(NULL, folder);
@@ -701,13 +725,11 @@ config_global_load(void)
     geometry_manager_init("SourceView", 500, 400, FALSE);
     geometry_manager_init("IMAPSubscriptions", 200, 160, FALSE);
     geometry_manager_init("IMAPSelectParent", 200, 160, FALSE);
-#ifdef HAVE_GPGME
     geometry_manager_init("KeyDialog", 400, 200, FALSE);
     geometry_manager_init("KeyList", 300, 200, FALSE);
 #ifdef ENABLE_AUTOCRYPT
     geometry_manager_init("AutocryptDB", 300, 200, FALSE);
 #endif  /* ENABLE_AUTOCRYPT */
-#endif  /* HAVE_GPGME */
 
     /* FIXME: PKGW: why comment this out? Breaks my Transfer context menu. */
     if (balsa_app.mblist_width < 100)
@@ -890,7 +912,7 @@ config_global_load(void)
 
     /* ... Printing */
     if (balsa_app.page_setup)
-	g_object_unref(G_OBJECT(balsa_app.page_setup));
+	g_object_unref(balsa_app.page_setup);
     balsa_app.page_setup = restore_gtk_page_setup();
     balsa_app.margin_left = libbalsa_conf_get_double("LeftMargin");
     balsa_app.margin_top = libbalsa_conf_get_double("TopMargin");
@@ -1067,10 +1089,8 @@ config_global_load(void)
     g_free(balsa_app.compose_headers);
     balsa_app.compose_headers =
 	libbalsa_conf_get_string("ComposeHeaders=Recipients Subject");
-#ifdef HAVE_GPGME
     balsa_app.warn_reply_decrypted =
     	libbalsa_conf_get_bool("WarnReplyDecrypted=true");
-#endif
 
     /* Obsolete. */
     libbalsa_conf_get_bool_with_default("RequestDispositionNotification=false",
@@ -1475,9 +1495,7 @@ config_save(void)
     libbalsa_conf_set_string("ComposeHeaders", balsa_app.compose_headers);
     libbalsa_conf_set_bool("ExternEditorEditHeaders", balsa_app.edit_headers);
     libbalsa_conf_set_string("QuoteString", balsa_app.quote_str);
-#ifdef HAVE_GPGME
     libbalsa_conf_set_bool("WarnReplyDecrypted", balsa_app.warn_reply_decrypted);
-#endif
 
     libbalsa_conf_pop_group();
 
@@ -1663,7 +1681,7 @@ config_identity_load(const gchar * key, const gchar * value, gpointer data)
     gchar *smtp_server_name;
 
     libbalsa_conf_push_group(key);
-    ident = libbalsa_identity_new_config(value);
+    ident = libbalsa_identity_new_from_config(value);
     smtp_server_name = libbalsa_conf_get_string("SmtpServer");
     libbalsa_identity_set_smtp_server(ident,
                                       find_smtp_server_by_name
@@ -1671,7 +1689,8 @@ config_identity_load(const gchar * key, const gchar * value, gpointer data)
     g_free(smtp_server_name);
     libbalsa_conf_pop_group();
     balsa_app.identities = g_list_prepend(balsa_app.identities, ident);
-    if (g_ascii_strcasecmp(default_ident, ident->identity_name) == 0)
+    if (g_ascii_strcasecmp(libbalsa_identity_get_identity_name(ident),
+                           default_ident) == 0)
         balsa_app.current_ident = ident;
 
     return FALSE;
@@ -1699,7 +1718,7 @@ config_identities_load()
 	libbalsa_conf_push_group("identity-default");
         balsa_app.identities =
             g_list_prepend(NULL,
-                           libbalsa_identity_new_config("default"));
+                           libbalsa_identity_new_from_config("default"));
 	libbalsa_conf_pop_group();
     }
 
@@ -1723,8 +1742,9 @@ config_identities_save(void)
     gchar *prefix;
 
     libbalsa_conf_push_group("identity");
-    libbalsa_conf_set_string("CurrentIdentity", 
-                            balsa_app.current_ident->identity_name);
+    libbalsa_conf_set_string("CurrentIdentity",
+                             libbalsa_identity_get_identity_name
+                             (balsa_app.current_ident));
     libbalsa_conf_pop_group();
 
     config_remove_groups(IDENTITY_SECTION_PREFIX);
@@ -1733,7 +1753,7 @@ config_identities_save(void)
     for (list = balsa_app.identities; list; list = list->next) {
 	ident = LIBBALSA_IDENTITY(list->data);
 	prefix = g_strconcat(IDENTITY_SECTION_PREFIX, 
-			     ident->identity_name, NULL);
+			     libbalsa_identity_get_identity_name(ident), NULL);
         libbalsa_identity_save(ident, prefix);
 	g_free(prefix);
     }
@@ -1802,10 +1822,8 @@ config_load_mailbox_view(const gchar * url)
     if (libbalsa_conf_has_key("Position"))
         view->position = libbalsa_conf_get_int("Position");
 
-#ifdef HAVE_GPGME
     if (libbalsa_conf_has_key("CryptoMode"))
         view->gpg_chk_mode = libbalsa_conf_get_int("CryptoMode");
-#endif
 
     if (libbalsa_conf_has_key("Total"))
         view->total = libbalsa_conf_get_int("Total");
@@ -1861,10 +1879,8 @@ config_save_mailbox_view(const gchar * url, LibBalsaMailboxView * view)
         if (view->position   != libbalsa_mailbox_get_position(NULL))
             libbalsa_conf_set_int("Position", view->position);
     }
-#ifdef HAVE_GPGME
     if (view->gpg_chk_mode   != libbalsa_mailbox_get_crypto_mode(NULL))
 	libbalsa_conf_set_int("CryptoMode",  view->gpg_chk_mode);
-#endif
     /* To avoid accumulation of config entries with only message counts,
      * we save them only if used in this session. */
     if (view->used && view->mtime != 0) {
@@ -1988,11 +2004,15 @@ config_filters_save(void)
 void
 config_mailbox_filters_save(LibBalsaMailbox * mbox)
 {
+    const gchar *url;
     gchar * tmp;
 
-    g_return_if_fail(mbox);
-    tmp = mailbox_filters_section_lookup(mbox->url ? mbox->url : mbox->name);
-    if (!mbox->filters) {
+    g_return_if_fail(LIBBALSA_IS_MAILBOX(mbox));
+
+    url = libbalsa_mailbox_get_url(mbox);
+    tmp = mailbox_filters_section_lookup(url !=NULL ? url :
+                                         libbalsa_mailbox_get_name(mbox));
+    if (libbalsa_mailbox_get_filters(mbox) == NULL) {
 	if (tmp) {
 	    libbalsa_conf_remove_group(tmp);
 	    g_free(tmp);
@@ -2004,7 +2024,7 @@ config_mailbox_filters_save(LibBalsaMailbox * mbox)
 	tmp=config_get_unused_group(MAILBOX_FILTERS_SECTION_PREFIX);
 	libbalsa_conf_push_group(tmp);
 	g_free(tmp);
-	libbalsa_conf_set_string(MAILBOX_FILTERS_URL_KEY,mbox->url);
+	libbalsa_conf_set_string(MAILBOX_FILTERS_URL_KEY, url);
     }
     else {
 	libbalsa_conf_push_group(tmp);
