@@ -60,7 +60,7 @@ static void store_address_add_address(StoreAddressInfo * info,
                                       InternetAddress * address,
                                       InternetAddress * group);
 static void store_address_add_lbaddress(StoreAddressInfo * info,
-                                        const LibBalsaAddress *address);
+                                        LibBalsaAddress *address);
 static void store_address_add_list(StoreAddressInfo * info,
                                    const gchar * label,
 				   InternetAddressList * list);
@@ -114,13 +114,13 @@ balsa_store_address_from_messages(GList * messages)
                           (GWeakNotify) store_address_weak_notify, info);
     }
 
-    g_signal_connect(G_OBJECT(info->dialog), "response",
+    g_signal_connect(info->dialog, "response",
                      G_CALLBACK(store_address_response), info);
     gtk_widget_show(GTK_WIDGET(info->dialog));
 }
 
 void
-balsa_store_address(const LibBalsaAddress *address)
+balsa_store_address(LibBalsaAddress *address)
 {
     StoreAddressInfo *info = NULL;
 
@@ -132,7 +132,7 @@ balsa_store_address(const LibBalsaAddress *address)
 
     store_address_add_lbaddress(info, address);
 
-    g_signal_connect(G_OBJECT(info->dialog), "response",
+    g_signal_connect(info->dialog, "response",
                      G_CALLBACK(store_address_response), info);
     gtk_widget_show_all(GTK_WIDGET(info->dialog));
 }
@@ -178,8 +178,7 @@ store_address_response(GtkWidget * dialog, gint response,
         g_object_weak_unref(G_OBJECT(list->data),
                             (GWeakNotify) store_address_weak_notify, info);
     }
-    g_list_foreach(info->entries_list, (GFunc) g_free, NULL);
-    g_list_free(info->entries_list);
+    g_list_free_full(info->entries_list, g_free);
     store_address_free(info);
 }
 
@@ -332,12 +331,15 @@ store_address_note_frame(StoreAddressInfo *info)
     gtk_container_set_border_width(GTK_CONTAINER(info->notebook), 4);
 
     for (list = info->message_list; list; list = list->next) {
+        LibBalsaMessageHeaders *headers;
+
         message = LIBBALSA_MESSAGE(list->data);
-	if (message->headers) {
-	    store_address_add_list(info, _("From: "), message->headers->from);
-	    store_address_add_list(info, _("To: "), message->headers->to_list);
-	    store_address_add_list(info, _("CC: "), message->headers->cc_list);
-	    store_address_add_list(info, _("BCC: "), message->headers->bcc_list);
+        headers = libbalsa_message_get_headers(message);
+	if (headers != NULL) {
+	    store_address_add_list(info, _("From: "), headers->from);
+	    store_address_add_list(info, _("To: "), headers->to_list);
+	    store_address_add_list(info, _("CC: "), headers->cc_list);
+	    store_address_add_list(info, _("BCC: "), headers->bcc_list);
 	}
     }
 
@@ -376,32 +378,25 @@ store_address_add_address(StoreAddressInfo * info,
 
     text = internet_address_to_string(ia, FALSE);
     address = libbalsa_address_new();
-    if (ia->name != NULL) {
-    	address->full_name = g_strdup(ia->name);
-    } else {
-    	address->full_name = g_strdup((group != NULL) ? group->name : NULL);
-    }
+    libbalsa_address_set_full_name(address,
+                                   ia->name != NULL ? ia->name :
+                                   group != NULL ? group->name : NULL);
     if (INTERNET_ADDRESS_IS_GROUP(ia)) {
         InternetAddressList *members;
         int j;
 
-        address->address_list = NULL;
         members = INTERNET_ADDRESS_GROUP(ia)->members;
 
         for (j = 0; j < internet_address_list_length(members); j++) {
             InternetAddress *member_address =
                 internet_address_list_get_address(members, j);
             if (INTERNET_ADDRESS_IS_MAILBOX(member_address))
-                address->address_list =
-                    g_list_prepend(address->address_list,
-                                   g_strdup(INTERNET_ADDRESS_MAILBOX
-                                            (member_address)->addr));
+                libbalsa_address_append_addr
+                    (address, INTERNET_ADDRESS_MAILBOX(member_address)->addr);
         }
-        address->address_list = g_list_reverse(address->address_list);
+        libbalsa_address_append_addr(address, INTERNET_ADDRESS_MAILBOX(ia)->addr);
     } else {
-        address->address_list =
-            g_list_prepend(NULL,
-                           g_strdup(INTERNET_ADDRESS_MAILBOX(ia)->addr));
+        libbalsa_address_append_addr(address, INTERNET_ADDRESS_MAILBOX(ia)->addr);
     }
     ew = libbalsa_address_get_edit_widget(address, entries, NULL, NULL);
     g_object_unref(address);
@@ -418,19 +413,23 @@ store_address_add_address(StoreAddressInfo * info,
 
 static void
 store_address_add_lbaddress(StoreAddressInfo * info,
-                            const LibBalsaAddress *address)
+                            LibBalsaAddress *address)
 {
     gchar *label_text;
     GtkWidget **entries, *ew;
+    const gchar *addr;
+    const gchar *full_name;
+
+    addr = libbalsa_address_get_addr(address);
+    g_return_if_fail(addr != NULL);
 
-    g_return_if_fail(address->address_list);
     entries = g_new(GtkWidget *, NUM_FIELDS);
     info->entries_list = g_list_append(info->entries_list, entries);
 
     ew = libbalsa_address_get_edit_widget(address, entries, NULL, NULL);
 
-    label_text = g_strdup(address->full_name ? address->full_name :
-                          address->address_list->data);
+    full_name = libbalsa_address_get_full_name(address);
+    label_text = g_strdup(full_name != NULL ? full_name : addr);
     if (g_utf8_strlen(label_text, -1) > 15)
         /* truncate to an arbitrary length: */
         *g_utf8_offset_to_pointer(label_text, 15) = '\0';
